package ru.daotech.agroinvest

import android.app.Activity
import android.app.Application
import android.app.Service
import com.google.firebase.FirebaseApp
import dagger.android.AndroidInjector
import dagger.android.DispatchingAndroidInjector
import dagger.android.HasActivityInjector
import dagger.android.HasServiceInjector
import ly.count.android.sdk.Countly
import ly.count.android.sdk.CountlyConfig
import ru.daotech.agroinvest.di.DaggerAppComponent
import ru.daotech.agroinvest.di.modules.DatabaseModule
import ru.daotech.agroinvest.di.modules.NetworkModule
import timber.log.Timber
import javax.inject.Inject

const val COUNTLY_DEFAULT_DEVICE_ID = "NOT_LOGGED_USER"
const val CUSTOM_HEADER_VALUE_INDEX = "foo"
const val CUSTOM_HEADER_VALUE = "bar"

class App : Application(), HasActivityInjector, HasServiceInjector {

    @Inject
    lateinit var dispatchingAndroidInjector: DispatchingAndroidInjector<Activity>
    @Inject
    lateinit var dispatchingServiceInjector: DispatchingAndroidInjector<Service>

    override fun activityInjector(): AndroidInjector<Activity> = dispatchingAndroidInjector

    override fun serviceInjector(): AndroidInjector<Service> = dispatchingServiceInjector

    override fun onCreate() {
        super.onCreate()
        FirebaseApp.initializeApp(this)

        if(BuildConfig.countlyUrl.isNotEmpty() && BuildConfig.countlyKey.isNotEmpty()) {

            val customHeaderValues: HashMap<String, String> = HashMap()
            customHeaderValues[CUSTOM_HEADER_VALUE_INDEX] = CUSTOM_HEADER_VALUE

            val config: CountlyConfig = CountlyConfig()
                .setContext(this)
                .setAppKey(BuildConfig.countlyKey)
                .setServerURL(BuildConfig.countlyUrl)
                .setDeviceId(COUNTLY_DEFAULT_DEVICE_ID)
                //.setIdMode(DeviceId.Type.OPEN_UDID)
                .enableCrashReporting()
                .setLoggingEnabled(true)
                .setViewTracking(true)
                .setAutoTrackingUseShortName(true)
                .setRequiresConsent(true)
                .setConsentEnabled(
                    arrayOf(
                        Countly.CountlyFeatureNames.push,
                        Countly.CountlyFeatureNames.sessions,
                        Countly.CountlyFeatureNames.location,
                        Countly.CountlyFeatureNames.attribution,
                        Countly.CountlyFeatureNames.crashes,
                        Countly.CountlyFeatureNames.events,
                        Countly.CountlyFeatureNames.users,
                        Countly.CountlyFeatureNames.views
                    )
                )
                .addCustomNetworkRequestHeaders(customHeaderValues)
                .setPushIntentAddMetadata(true)
            //.setRemoteConfigAutomaticDownload(true) { error ->
            //    if (error == null) {
            //        Timber.d("Automatic remote config download has completed")
            //    } else {
            //        Timber.d("Automatic remote config download encountered a problem, $error")
            //    }
            //}
            //.setRemoteConfigAutomaticDownload(true, null)

            Countly.sharedInstance().init(config)

            //val predefinedFields = HashMap<String, String>()
            //Countly.userData.setUserData(predefinedFields)
            //Countly.userData.setProperty("name", "AGRO INVEST");
            //Countly.userData.save()

        }

        val baseUrl = "${BuildConfig.schema}${BuildConfig.hostName}"
        DaggerAppComponent.builder()
            .context(this)
            .network(NetworkModule(baseUrl))
            .database(DatabaseModule())
            .build().inject(this)
        if (BuildConfig.DEBUG) {
            Timber.plant(Timber.DebugTree())
        }
    }
}