package ru.daotech.agroinvest.adapters

import android.annotation.SuppressLint
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.ListView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import kotlinx.android.synthetic.main.item_collection_history.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.autoNotify
import ru.daotech.agroinvest.database.entities.TrolleyInfoImpl
import ru.daotech.agroinvest.utils.DateUtil
import kotlin.properties.Delegates

class CollectionHistoryAdapter : RecyclerView.Adapter<CollectionHistoryAdapter.ViewHolderOther>() {

    private lateinit var trolleyOfDayList: ListView

    var serverTimestamp: Int = 0

    var collectionHistoryList: List<CollectionHistoryModelView>
            by Delegates.observable(emptyList()) { _, old, new ->
                autoNotify(old, new) { o, n -> o.tiedDayOfDate == n.tiedDayOfDate }
            }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolderOther {
        val view = LayoutInflater.from(parent.context)
            .inflate(R.layout.item_collection_history, parent, false)
        return ViewHolderOther(view)
    }

    override fun getItemCount() = collectionHistoryList.size

    override fun onBindViewHolder(holder: ViewHolderOther, position: Int) {
        holder.bind(collectionHistoryList[position])
    }

    inner class ViewHolderOther(itemView: View) : RecyclerView.ViewHolder(itemView) {
        @SuppressLint("SetTextI18n")
        fun bind(itemTrolleyInfo: CollectionHistoryModelView) {

            itemView.collection_history_date.text = itemTrolleyInfo.tiedDayOfDate
            val adapter = SubCollectionHistoryListAdapter(
                itemView.context,
                trolleyList = itemTrolleyInfo.trolleyList
            )
            trolleyOfDayList = itemView.trolley_collection_list_view!!
            trolleyOfDayList.adapter = adapter

            setListViewHeightBasedOnChildren(trolleyOfDayList)
        }
    }

    private fun setListViewHeightBasedOnChildren(listView: ListView) {
        val listAdapter = listView.adapter
            ?: return

        var totalHeight = 0
        var i = 0
        while (i < listAdapter.count) {
            val listItem = listAdapter.getView(i, null, listView)
            listItem.measure(0, 0)
            totalHeight += listItem.measuredHeight
            i++
        }

        val params = listView.layoutParams as ViewGroup.LayoutParams
        params.height = totalHeight + (listView.dividerHeight * (listAdapter.count - 1))
        listView.layoutParams = params
    }
}

class SubCollectionHistoryListAdapter(
    context: Context,
    private val trolleyList: List<TrolleyInfoImpl>
) : ArrayAdapter<TrolleyInfoImpl>(
    context,
    R.layout.item_collection_history_trolley_info,
    trolleyList
) {

    @SuppressLint("ViewHolder", "InflateParams")
    override fun getView(position: Int, view: View?, parent: ViewGroup): View {
        val inflater = context
            .getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
        val rowView = inflater.inflate(
            R.layout.item_collection_history_trolley_info,
            null,
            true
        )

        val titleText = rowView.findViewById(R.id.trolley_collection) as TextView
        val subtitleText = rowView.findViewById(R.id.trolley_collection_date) as TextView

        titleText.text = trolleyList[position].trolley.toString()
        subtitleText.text = DateUtil.getCollectionHistoryTimeDateString(
            trolleyList[position].registeredAt
        )

        return rowView
    }
}

data class CollectionHistoryModelView(
    val trolleyList: List<TrolleyInfoImpl>,
    val tiedDayOfDate: String
)