package ru.daotech.agroinvest.adapters

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.jakewharton.rxbinding2.view.clicks
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.item_section.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.autoNotify
import ru.daotech.agroinvest.database.entities.Section
import timber.log.Timber
import kotlin.properties.Delegates

class SectionAdapter(
    private val sectionClicks: PublishSubject<Section>
) : RecyclerView.Adapter<SectionAdapter.ViewHolderOther>() {

    var sectionList: List<Section> by Delegates.observable(emptyList()) { _, old, new ->
        autoNotify(old, new) { o, n -> o.id == n.id }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolderOther {
        val view = LayoutInflater.from(parent.context)
            .inflate(R.layout.item_section, parent, false)
        return ViewHolderOther(view)
    }

    override fun getItemCount() = sectionList.size

    override fun onBindViewHolder(holder: ViewHolderOther, position: Int) {
        holder.bind(sectionList[position])
    }

    inner class ViewHolderOther(itemView: View) : RecyclerView.ViewHolder(itemView) {

        fun bind(itemSection: Section) {
            itemView.section_title.text = itemSection.name

            itemView.clicks()
                .map {
                    itemSection
                }
                .doOnError(Timber::e)
                .subscribe(sectionClicks)
        }
    }
}
