package ru.daotech.agroinvest.adapters

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import kotlinx.android.synthetic.main.item_support_message.view.*
import kotlinx.android.synthetic.main.item_user_message.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.autoNotify
import ru.daotech.agroinvest.database.entities.SupportMessage
import ru.daotech.agroinvest.models.Worker
import ru.daotech.agroinvest.utils.DateUtil
import ru.daotech.agroinvest.utils.ServiceUtil.Companion.getShortenFullName
import kotlin.properties.Delegates

class SupportNotificationAdapter :
    RecyclerView.Adapter<SupportNotificationAdapter.BaseViewHolder<*>>() {

    companion object {
        private const val TYPE_SUPPORT = 0
        private const val TYPE_WORKER = 1
    }

    var supportNotificationList: List<BaseMessageModelView> by Delegates.observable(emptyList()) { _, old, new ->
        autoNotify(old, new) { o, n -> o.message.id == n.message.id }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder<*> {
        return when (viewType) {
            TYPE_SUPPORT -> {
                val view = LayoutInflater.from(parent.context)
                    .inflate(R.layout.item_support_message, parent, false)
                SupportMessageViewHolder(view)
            }
            TYPE_WORKER -> {
                val view = LayoutInflater.from(parent.context)
                    .inflate(R.layout.item_user_message, parent, false)
                WorkerMessageViewHolder(view)
            }
            else -> throw IllegalArgumentException("Invalid view type")
        }
    }

    override fun getItemCount() = supportNotificationList.size

    override fun onBindViewHolder(holder: BaseViewHolder<*>, position: Int) {
        val element = supportNotificationList[position]
        when (holder) {
            is SupportMessageViewHolder -> holder.bind(element as SupportMessageModelView)
            is WorkerMessageViewHolder -> holder.bind(element as WorkerMessageModelView)
            else -> throw IllegalArgumentException()
        }
    }

    inner class SupportMessageViewHolder(itemView: View) :
        BaseViewHolder<SupportMessageModelView>(itemView) {

        override fun bind(item: SupportMessageModelView) {
            with(item.message) {
                itemView.support_notifications_message.text = message
                itemView.item_support_message_time.text =
                    DateUtil.getTimeTimerChatMessageFormat(time)
            }
        }
    }

    inner class WorkerMessageViewHolder(itemView: View) :
        BaseViewHolder<WorkerMessageModelView>(itemView) {

        override fun bind(item: WorkerMessageModelView) {
            with(item.message) {
                itemView.support_notifications_worker_message.text = message
                itemView.item_support_message_worker_time.text =
                    DateUtil.getTimeTimerChatMessageFormat(time)
                itemView.support_notifications_worker_name.text =
                    getShortenFullName(item.worker.name)
            }
        }
    }

    override fun getItemViewType(position: Int): Int {
        val comparable = supportNotificationList[position]
        return when (comparable) {
            is SupportMessageModelView -> TYPE_SUPPORT
            is WorkerMessageModelView -> TYPE_WORKER
            else -> throw IllegalArgumentException("Invalid type of data " + position)
        }
    }

    abstract class BaseViewHolder<T>(itemView: View) : RecyclerView.ViewHolder(itemView) {
        abstract fun bind(item: T)
    }
}

data class SupportMessageModelView(
    override val message: SupportMessage
) : BaseMessageModelView(message)

data class WorkerMessageModelView(
    override val message: SupportMessage,
    val worker: Worker
) : BaseMessageModelView(message)

abstract class BaseMessageModelView(
    open val message: SupportMessage
)