package ru.daotech.agroinvest.adapters

import android.annotation.SuppressLint
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import kotlinx.android.synthetic.main.item_task.view.*
import org.joda.time.format.DateTimeFormat
import org.joda.time.format.DateTimeFormatter
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.autoNotify
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.utils.DateUtil
import ru.daotech.agroinvest.utils.ServiceUtil
import kotlin.properties.Delegates


class TaskHistoryAdapter(
    private val showNames: Boolean
) : RecyclerView.Adapter<TaskHistoryAdapter.ViewHolderOther>() {

    var serverTimestamp: Int = 0

    var taskList: List<TaskWithProperties> by Delegates.observable(emptyList()) { _, old, new ->
        autoNotify(old, new) { o, n -> o.id == n.id }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolderOther {
        val view = LayoutInflater.from(parent.context)
            .inflate(R.layout.item_task_history, parent, false)
        return ViewHolderOther(view)
    }

    override fun getItemCount() = taskList.size

    override fun onBindViewHolder(holder: ViewHolderOther, position: Int) {
        holder.bind(taskList[position])
    }

    inner class ViewHolderOther(itemView: View) : RecyclerView.ViewHolder(itemView) {
        @SuppressLint("SetTextI18n")
        fun bind(itemTask: TaskWithProperties) {
            if (showNames) {
                itemView.task_finish_at.setCompoundDrawablesWithIntrinsicBounds(
                    null,
                    null,
                    null,
                    null
                )

                itemView.task_finish_at.text =
                    if (itemTask.workerName != null) itemTask.workerName else "Не назначено"

            } else {
                itemView.task_finish_at.setCompoundDrawablesWithIntrinsicBounds(
                    null,
                    null,
                    itemView.context.getDrawable(
                        if (itemTask.isFinished && itemTask.isTaskInfoSent) {
                            R.drawable.ic_cloud_done_25dp
                        } else {
                            R.drawable.ic_cloud_queue_25dp
                        }
                    ), // TODO: Add another state (ic_done_all) when task data is synced with CC server
                    null
                )

                if (itemTask.startedAt != null && itemTask.finishedAt != null) {

                    val finishedTimeMark =
                        DateUtil.getDateTimeWithDelta(itemTask.finishedAt!!, serverTimestamp)

                    val fmt: DateTimeFormatter = DateTimeFormat.forPattern("d MMMM в H:mm")

                    val resultMills = ServiceUtil.calculateTaskIntervalsSum(
                        taskWithProperties = itemTask,
                        serverTimestamp = serverTimestamp
                    )

                    val diffMinutes = resultMills / (60 * 1000) % 60
                    val diffHours = resultMills / (60 * 60 * 1000) % 24

                    itemView.task_finish_at.text =
                        """Завершено ${fmt.print(finishedTimeMark)} за $diffHours ч.$diffMinutes мин."""
                }
            }

            itemView.task_section.text =
                "Ряд " + itemTask.rowNumber?.toString() + " — " + itemTask.section?.name

            itemView.task_type_line_container.setBackgroundColor(
                itemView.context.getColor(R.color.colorPrimary)
            )
        }
    }
}
