package ru.daotech.agroinvest.business

import ly.count.android.sdk.Countly
import org.joda.time.DateTime
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.asMap
import ru.daotech.agroinvest.models.InternetConnectionStateJson
import ru.daotech.agroinvest.models.InternetConnectivityState
import ru.daotech.agroinvest.utils.DateUtil
import timber.log.Timber
import java.util.*

open class CountlyEventControllerImpl(val appStateManager: AppStateManager) :
    CountlyEventController {

    override fun sendCountlyEvent(
        eventType: String,
        internetConnectivityState: InternetConnectivityState
    ) {
        try {
            if (Countly.sharedInstance().isInitialized) {
                val currentNetworkState = InternetConnectionStateJson(
                    rowNumber = when (appStateManager.getCurrentRowNumber().isBlank()) {
                        true -> NULL_STATE_VALUE
                        false -> appStateManager.getCurrentRowNumber()
                    },
                    downSpeed = internetConnectivityState.downSpeed,
                    upSpeed = internetConnectivityState.upSpeed,
                    time = DateUtil.getDateTimeWithDelta(
                        DateTime.now(),
                        appStateManager.getServerTimestamp()
                    ),
                    connectionState = appStateManager.connectionReceiver.isConnected,
                    workerName = when (appStateManager.getCurrentWorkerSimple()?.name.isNullOrBlank()) {
                        true -> NULL_STATE_VALUE
                        false -> appStateManager.getCurrentWorkerSimple()?.name
                    },
                    personalNumber = when (appStateManager.getCurrentWorkerSimple()?.personalNumber.isNullOrBlank()) {
                        true -> NULL_STATE_VALUE
                        false -> appStateManager.getCurrentWorkerSimple()?.personalNumber
                    },
                    networkInfo = when (internetConnectivityState.networkInfo.isNullOrBlank()) {
                        true -> NULL_STATE_VALUE
                        false -> internetConnectivityState.networkInfo
                    },
                    networkCapabilities = when (internetConnectivityState.networkCapabilities.isNullOrBlank()) {
                        true -> NULL_STATE_VALUE
                        false -> internetConnectivityState.networkCapabilities
                    },
                    workerId = when (appStateManager.getCurrentWorkerSimple()?.id.isNullOrBlank()) {
                        true -> NULL_STATE_VALUE
                        false -> appStateManager.getCurrentWorkerSimple()?.id
                    },
                    sessionId = appStateManager.getSessionId(),
                    deviceTimeMark = when (appStateManager.getDeviceTimeInfo().isNullOrEmpty()) {
                        true -> NULL_STATE_VALUE
                        false -> appStateManager.getDeviceTimeInfo()
                    },
                    serverTimeMark = when (appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp.isNullOrEmpty()) {
                        true -> NULL_STATE_VALUE
                        false -> appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp
                    },
                    deviceId = NULL_STATE_VALUE, // FIXME: Get Secure.ANDROID_ID
                    deltaTimestamp = DEFAULT_DELTA_TIMESTAMP //appStateManager.getServerTimestamp()
                )

                val segmentation = HashMap(currentNetworkState.asMap())

                Countly.sharedInstance().recordEvent(
                    eventType,
                    segmentation,
                    COUNTLY_SHARED_EVENT_COUNT
                )
            }

        } catch (ex: java.lang.Exception) {
            Timber.e(ex)
        }
    }

    companion object {
        const val NULL_STATE_VALUE = "null"
        const val COUNTLY_SHARED_EVENT_COUNT = 1
        const val DEFAULT_DELTA_TIMESTAMP = 0
    }
}

interface CountlyEventController {

    fun sendCountlyEvent(
        eventType: String,
        internetConnectivityState: InternetConnectivityState
    )
}