package ru.daotech.agroinvest.business

import android.net.Uri
import io.reactivex.Completable
import io.reactivex.Flowable
import io.reactivex.Observable
import io.reactivex.Single
import ru.daotech.agroinvest.database.entities.*
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.models.LastFragmentState
import ru.daotech.agroinvest.models.Rating
import ru.daotech.agroinvest.models.Worker
import ru.daotech.agroinvest.network.models.ChatDataResponse
import ru.daotech.agroinvest.service.events.BroadcastMessage
import ru.daotech.agroinvest.service.events.ServerData
import ru.daotech.agroinvest.service.events.SupportMessageInfo
import ru.daotech.agroinvest.service.jsonmodels.*
import java.io.File

interface Repository {

    fun signIn(workerCode: String): Observable<Unit>

    fun saveTask(taskPayload: TaskPayload): Observable<Unit>

    fun getMyTasksByRowNumber(rowNumber: String): Observable<List<Task>>

    fun getMyStartedTasks(): Observable<List<Task>>

    fun getMyPausedTasks(): Observable<List<Task>>

    fun getMyTasksFlowable(): Flowable<List<Task>>

    fun getTasksHistory(): Observable<List<TaskWithProperties>>

    fun getRootSections(): Observable<List<Section>>

    fun getRatings(): Observable<List<Rating>>

    fun getSection(sectionId: String): Observable<Section>

    fun getSectionParent(parentId: String?): Observable<List<Section>>

    fun getSectionTasks(sectionId: String): Observable<List<TaskWithProperties>>

    fun getBroadcastMessageSubject(): Observable<BroadcastMessage>

    fun getScreenshotMessageSubject(): Observable<Unit>

    fun getTaskWithProperties(taskId: String): Observable<TaskWithProperties>

    fun signOut(workerId: String): Observable<Boolean>

    fun signOutFromQueue()

    fun loadCurrentAppStatusByUrl(): Observable<ServerDataPayload>

    fun signalToTaskList(workerId: String): Observable<Unit>

    fun downloadAppUrl(appUrl: String, appFolder: String, identifier: String): Observable<File>

    fun signalToTaskStart(taskId: String): Observable<Unit>

    fun signalToTaskCancel(taskId: String): Observable<Unit>

    fun signalToTaskFinish(taskId: String): Observable<Unit>

    fun getServerDataSubject(): Observable<ServerData>

    fun signalStartedDataToUpload(taskId: String): Observable<Unit>

    fun signalFinishedDataToUpload(taskId: String): Observable<Unit>

    fun signalCanceledDataToUpload(taskId: String): Observable<Unit>

    fun flushCamundaQueue(): Observable<Unit>

    fun notShowAttentionAgain(): Observable<Unit>

    fun getLastFragmentInfo(): Observable<LastFragmentState>?

    fun saveCurrentFragmentState(currentInterval: Long, taskId: String): Observable<Unit>

    fun clearLastTaskStatePreferences(): Observable<Unit>

    fun getCurrentWorker(): Observable<Worker>

    fun uploadDatabaseFileDump(files: List<File>): Observable<Unit>

    fun sendAccidentReport(reportMessage: String): Observable<Unit>

    fun sendAccidentComplexReport(
        taskWithProperties: TaskWithProperties?,
        reportMessage: String
    ): Observable<Unit>

    fun fixTimerStartMark(taskId: String): Observable<Unit>

    fun signalAccidentReportToUpload(
        reportMessage: String,
        taskId: String?
    ): Observable<Unit>

    fun signalTrolleyTiedToUpload(trolleyNumber: String): Observable<Unit>

    fun signalTrolleyUntiedToUpload(trolleyNumber: String): Observable<Unit>

    fun getCollectionHistory(): Observable<List<TrolleyInfoImpl>>

    fun signalToTrolleyInfoHistoryList(workerId: String): Observable<Unit>

    fun saveWorkerInfo(worker: Worker): Observable<Unit>

    fun saveTrolleyInfo(trolleyInfoPayload: TrolleyInfoPayload): Observable<Unit>

    fun unbindTrolley(trolleyUnbindPayload: TrolleyUnbindPayload): Observable<Unit>

    fun getAllSupportMessagesByCurrentWorker(): Flowable<List<SupportMessage>>

    fun getUnreadSupportMessageCount(): Flowable<Int>

    fun saveSupportMessageInfo(
        supportMessageInfo: SupportMessagePayload,
        messageCode: Int
    ): Observable<Unit>

    fun getSupportMessageSubject(): Observable<SupportMessageInfo>

    fun updateUnreadStatusSupportMessages(supportMessages: List<SupportMessage>): Completable

    fun signalToGroupPropertyCreate(
        taskId: String,
        properties: List<TaskPropertyImpl>,
        groupId: String,
        barcodeValue: String
    ): Observable<Unit>

    fun signalGroupPropertyDataToUpload(
        taskId: String,
        properties: List<TaskPropertyImpl>,
        groupId: String
    ): Observable<Unit>

    fun sendBarcodeBoxValueRequest(barcodeBoxValue: String): Observable<Unit>

    fun uploadScreenshotImage(file: File): Observable<Unit>

    fun uploadPropertyImages(uriList: List<Uri?>): Observable<Unit>

    fun sendSupportUserMessage(message: String): Observable<Unit>

    fun updatePersonalChat(chatResponse: ChatDataResponse): Single<Unit>

    fun getRecentUsers(): Observable<List<RecentUser>>

    fun signalToTaskListByCacheUrl(): Observable<Unit>
}