package ru.daotech.agroinvest.business

import ru.daotech.agroinvest.database.entities.ActionReportImpl
import ru.daotech.agroinvest.database.entities.CamundaImpl
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.models.RequestTypeEnum
import ru.daotech.agroinvest.models.Worker
import ru.daotech.agroinvest.network.models.GroupPropertyRequest
import ru.daotech.agroinvest.network.models.PropertyRequest
import ru.daotech.agroinvest.network.models.SuperTaskRequest
import ru.daotech.agroinvest.network.models.VariableRequest

interface VariableRequestBuilder {

    fun getVariableForTaskUpdateRequest(
        taskStartPayloadRequest: SuperTaskRequest.PayloadRequest,
        worker: Worker
    ): VariableRequest

    fun getVariableForPropertyRequest(
        taskStartPayloadRequest: PropertyRequest.PayloadRequest,
        worker: Worker
    ): VariableRequest

    fun getVariableForActionReport(
        actionReport: ActionReportImpl
    ): VariableRequest

    fun getVariableForFileUpload(worker: Worker?): VariableRequest

    fun getVariableForSignOut(workerId: String): VariableRequest

    fun getVariableForTaskList(workerId: String): VariableRequest

    fun getSuperTaskRequestByTaskType(
        taskType: RequestTypeEnum,
        taskWithProperties: TaskWithProperties,
        requestElementPayload: SuperTaskRequest.PayloadRequest
    ): SuperTaskRequest.PayloadRequest?

    fun parseSuperTaskRequestData(request: CamundaImpl): SuperTaskRequest.PayloadRequest

    fun getVariableForBarcodeBox(barcodeValue: String, workerId: String): VariableRequest

    fun getVariableForGroupPropertyRequest(
        groupPropertyPayloadRequest: GroupPropertyRequest.PayloadRequest,
        worker: Worker
    ): VariableRequest

    fun getGroupPropertyRequestByTaskType(
        taskType: RequestTypeEnum,
        taskWithProperties: TaskWithProperties,
        requestElementPayload: GroupPropertyRequest.PayloadRequest,
        groupId: String
    ): GroupPropertyRequest.PayloadRequest?

    fun parseGroupPropertyRequestData(request: CamundaImpl): GroupPropertyRequest.PayloadRequest
}