package ru.daotech.agroinvest.business

import com.google.gson.Gson
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.database.entities.ActionReportImpl
import ru.daotech.agroinvest.database.entities.CamundaImpl
import ru.daotech.agroinvest.database.entities.TaskIntervalImpl
import ru.daotech.agroinvest.database.entities.TaskPropertyImpl
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.models.ObjectRequestTypeEnum
import ru.daotech.agroinvest.models.RequestTypeEnum
import ru.daotech.agroinvest.models.Worker
import ru.daotech.agroinvest.network.models.*
import ru.daotech.agroinvest.utils.DateUtil
import javax.inject.Inject


open class VariableRequestBuilderImpl @Inject constructor(
    private val appStateManager: AppStateManager
) : VariableRequestBuilder {

    override fun getVariableForTaskUpdateRequest(
        taskStartPayloadRequest: SuperTaskRequest.PayloadRequest,
        worker: Worker
    ): VariableRequest {
        return VariableRequest(
            variables = SuperTaskRequest(
                type = BaseVariable(RequestTypeEnum.TASK_UPDATE.requestType),
                meta = BaseVariable(
                    SuperTaskRequest.MetaRequest(
                        workerId = worker.id,
                        deviceId = null,
                        sessionId = appStateManager.getSessionId(),
                        deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                        serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                        deltaTimestamp = appStateManager.getServerTimestamp()
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                payload = BaseVariable(
                    taskStartPayloadRequest.toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                )
            ),
            businessKey = "${RequestTypeEnum.TASK_UPDATE.requestType}-" +
                    "${appStateManager.getCurrentWorkerSimple()?.id}"
        )
    }

    override fun getVariableForGroupPropertyRequest(
        groupPropertyPayloadRequest: GroupPropertyRequest.PayloadRequest,
        worker: Worker
    ): VariableRequest {
        return VariableRequest(
            variables = GroupPropertyRequest(
                type = BaseVariable(RequestTypeEnum.PROPERTIES_CREATE.requestType),
                meta = BaseVariable(
                    GroupPropertyRequest.MetaRequest(
                        workerId = worker.id,
                        deviceId = null,
                        sessionId = appStateManager.getSessionId(),
                        deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                        serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                        deltaTimestamp = appStateManager.getServerTimestamp()
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                payload = BaseVariable(
                    groupPropertyPayloadRequest.toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                )
            ),
            businessKey = "${RequestTypeEnum.PROPERTIES_CREATE.requestType}-" +
                    "${appStateManager.getCurrentWorkerSimple()?.id}"
        )
    }

    override fun getVariableForPropertyRequest(
        taskStartPayloadRequest: PropertyRequest.PayloadRequest,
        worker: Worker
    ): VariableRequest {
        return VariableRequest(
            variables = PropertyRequest(
                type = BaseVariable(RequestTypeEnum.TASK_UPDATE.requestType),
                meta = BaseVariable(
                    PropertyRequest.MetaRequest(
                        workerId = worker.id,
                        deviceId = null,
                        sessionId = appStateManager.getSessionId(),
                        deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                        serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                        deltaTimestamp = appStateManager.getServerTimestamp()
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                payload = BaseVariable(
                    taskStartPayloadRequest.toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                )
            ),
            businessKey = "${RequestTypeEnum.TASK_UPDATE.requestType}-" +
                    "${appStateManager.getCurrentWorkerSimple()?.id}"
        )
    }

    override fun getVariableForActionReport(
        actionReport: ActionReportImpl
    ): VariableRequest {
        return VariableRequest(
            variables = LogReportRequest(
                meta = BaseVariable(
                    LogReportRequest.MetaRequest(
                        workerId = appStateManager.getCurrentWorkerSimple()!!.id,
                        deviceId = null,
                        sessionId = appStateManager.getSessionId(),
                        deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                        serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                        deltaTimestamp = appStateManager.getServerTimestamp()
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                payload = BaseVariable(
                    value = LogReportRequest.PayloadRequest(actionReport).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                )
            ),
            businessKey = "actionReport-${appStateManager.getCurrentWorkerSimple()?.id}"
        )
    }

    override fun getVariableForFileUpload(worker: Worker?): VariableRequest {
        return VariableRequest(
            variables = FileUploadRequest(
                meta = BaseVariable(
                    SignInRequest.MetaRequest(
                        workerId = worker!!.id,
                        deviceId = null,
                        sessionId = appStateManager.getSessionId(),
                        deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                        serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                        deltaTimestamp = appStateManager.getServerTimestamp()
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                payload = BaseVariable(
                    FileUploadRequest.PayloadRequest(worker.id).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                )
            ),
            businessKey = "${RequestTypeEnum.FILE_UPLOAD.requestType}-" +
                    "${appStateManager.getCurrentWorkerSimple()?.id}"
        )
    }

    override fun getVariableForSignOut(workerId: String): VariableRequest {
        return VariableRequest(
            variables = SignOutRequest(
                meta = BaseVariable(
                    SignOutRequest.MetaRequest(
                        workerId = workerId,
                        deviceId = null,
                        sessionId = appStateManager.getSessionId(),
                        deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                        serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                        deltaTimestamp = appStateManager.getServerTimestamp()
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                payload = BaseVariable(
                    SignOutRequest.PayloadRequest(
                        workerId = workerId
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                )
            ),
            businessKey = "${RequestTypeEnum.SIGN_OUT.requestType}-" +
                    "${appStateManager.getCurrentWorkerSimple()?.id}"
        )
    }

    override fun getVariableForTaskList(workerId: String): VariableRequest {
        return VariableRequest(
            variables = TaskListRequest(
                meta = BaseVariable(
                    TaskListRequest.MetaRequest(
                        workerId = workerId,
                        deviceId = null,
                        sessionId = appStateManager.getSessionId(),
                        deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                        serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                        deltaTimestamp = appStateManager.getServerTimestamp()
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                payload = BaseVariable(
                    TaskListRequest.PayloadRequest(
                        workerId = workerId
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                )
            ),
            businessKey = "${RequestTypeEnum.TASK_LIST.requestType}-" +
                    "${appStateManager.getCurrentWorkerSimple()?.id}"
        )
    }

    fun getVariableForSignIn(workerCode: String): VariableRequest {
        return VariableRequest(
            variables = SignInRequest(
                meta = BaseVariable(
                    SignInRequest.MetaRequest(
                        workerId = null,
                        deviceId = null,
                        sessionId = appStateManager.getSessionId()
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                payload = BaseVariable(
                    SignInRequest.PayloadRequest(
                        code = workerCode
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                )
            ),
            businessKey = "${RequestTypeEnum.SIGN_IN.requestType}-$workerCode"
        )
    }

    fun getVariableForAccidentReport(
        taskWithProperties: TaskWithProperties?,
        reportMessage: String
    ): VariableRequest {

        // If a payload of task is empty, we need to send just a message
        if (taskWithProperties != null) {
            return VariableRequest(
                variables = TaskReportRequest(
                    meta = BaseVariable(
                        TaskReportRequest.MetaRequest(
                            workerId = appStateManager.getCurrentWorkerSimple()!!.id,
                            deviceId = null,
                            sessionId = appStateManager.getSessionId(),
                            deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                            serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                            deltaTimestamp = appStateManager.getServerTimestamp()
                        ).toString(),
                        type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                        valueInfo = JsonValueInfo()
                    ),
                    payload = BaseVariable(
                        value = TaskReportRequest.PayloadRequest(
                            taskWithProperties,
                            reportMessage
                        ).toString(),
                        type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                        valueInfo = JsonValueInfo()
                    )
                ),
                businessKey = "${RequestTypeEnum.TASK_REPORT.requestType}-" +
                        "${appStateManager.getCurrentWorkerSimple()?.id}"
            )
        } else {
            return VariableRequest(
                variables = TaskReportRequest(
                    meta = BaseVariable(
                        TaskReportRequest.MetaRequest(
                            workerId = appStateManager.getCurrentWorkerSimple()!!.id,
                            deviceId = null,
                            sessionId = appStateManager.getSessionId(),
                            deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                            serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                            deltaTimestamp = appStateManager.getServerTimestamp()
                        ).toString(),
                        type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                        valueInfo = JsonValueInfo()
                    ),
                    payload = BaseVariable(
                        value = TaskReportRequest.PayloadRequestMessage(
                            reportMessage
                        ).toString()
                    )
                ),
                businessKey = "${RequestTypeEnum.TASK_REPORT.requestType}-" +
                        "${appStateManager.getCurrentWorkerSimple()?.id}"
            )
        }
    }

    override fun getSuperTaskRequestByTaskType(
        taskType: RequestTypeEnum,
        taskWithProperties: TaskWithProperties,
        requestElementPayload: SuperTaskRequest.PayloadRequest
    ): SuperTaskRequest.PayloadRequest? {
        return when (taskType) {
            RequestTypeEnum.TASK_START -> {
                if (!requestElementPayload.startedAt.isNullOrBlank()) {
                    requestElementPayload.copy(
                        finishedAt = null,
                        intervals = taskWithProperties.intervals as List<TaskIntervalImpl>,
                        properties = (taskWithProperties.properties as List<*>).map { property ->
                            PropertyRequest.PropertyPayloadRequest(
                                property as TaskPropertyImpl
                            )
                        }
                    )
                } else {
                    requestElementPayload.copy(
                        startedAt = DateUtil.getCurrentDateTimestampWithDelta(
                            appStateManager.getServerTimestamp()
                        ),
                        finishedAt = null,
                        intervals = taskWithProperties.intervals as List<TaskIntervalImpl>,
                        properties = (taskWithProperties.properties as List<*>).map { property ->
                            PropertyRequest.PropertyPayloadRequest(
                                property as TaskPropertyImpl
                            )
                        }
                    )
                }
            }
            RequestTypeEnum.TASK_CANCEL -> {
                requestElementPayload.copy(
                    finishedAt = DateUtil.getCurrentDateTimestampWithDelta(
                        appStateManager.getServerTimestamp()
                    ),
                    isFinished = false,
                    intervals = taskWithProperties.intervals as List<TaskIntervalImpl>,
                    properties = (taskWithProperties.properties as List<*>).map { property ->
                        PropertyRequest.PropertyPayloadRequest(
                            property as TaskPropertyImpl
                        )
                    }
                )
            }
            RequestTypeEnum.TASK_FINISH -> {
                requestElementPayload.copy(
                    finishedAt = DateUtil.getCurrentDateTimestampWithDelta(
                        appStateManager.getServerTimestamp()
                    ),
                    isFinished = true,
                    intervals = taskWithProperties.intervals as List<TaskIntervalImpl>,
                    properties = (taskWithProperties.properties as List<*>).map { property ->
                        PropertyRequest.PropertyPayloadRequest(
                            property as TaskPropertyImpl
                        )
                    }
                )
            }
            else -> {
                null
            }
        }
    }

    override fun getGroupPropertyRequestByTaskType(
        taskType: RequestTypeEnum,
        taskWithProperties: TaskWithProperties,
        requestElementPayload: GroupPropertyRequest.PayloadRequest,
        groupId: String
    ): GroupPropertyRequest.PayloadRequest? {
        return when (taskType) {
            RequestTypeEnum.PROPERTIES_CREATE -> {
                requestElementPayload.copy(
                    barcodeValue = appStateManager.getCurrentBarcodeGroupValue()!!,
                    plantId = appStateManager.getCurrentTaskPlant()!!.plantId,
                    properties = (taskWithProperties.properties as List<*>)
                        .map { property ->
                            PropertyRequest.PropertyPayloadRequest(
                                property as TaskPropertyImpl
                            )
                        }
                        .filter { property -> property.parentId == groupId }
                )
            }
            else -> {
                null
            }
        }
    }

    override fun parseSuperTaskRequestData(request: CamundaImpl): SuperTaskRequest.PayloadRequest {
        return Gson().fromJson(
            Gson().fromJson(
                request.payload,
                SuperTaskRequest.CamundaRequest::class.java
            ).variables.payload.value,
            SuperTaskRequest.PayloadRequest::class.java
        )
    }

    override fun parseGroupPropertyRequestData(request: CamundaImpl): GroupPropertyRequest.PayloadRequest {
        return Gson().fromJson(
            Gson().fromJson(
                request.payload,
                GroupPropertyRequest.CamundaRequest::class.java
            ).variables.payload.value,
            GroupPropertyRequest.PayloadRequest::class.java
        )
    }

    fun getVariableForTrolleyTiedRequest(trolleyNumber: String, type: String): VariableRequest {
        return VariableRequest(
            variables = TrolleyRequest(
                meta = BaseVariable(
                    TrolleyRequest.MetaRequest(
                        workerId = appStateManager.getCurrentWorkerSimple()!!.id,
                        deviceId = null,
                        sessionId = appStateManager.getSessionId(),
                        deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                        serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                        deltaTimestamp = appStateManager.getServerTimestamp()
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                payload = BaseVariable(
                    value = TrolleyRequest.PayloadRequest(
                        trolleyNumber = trolleyNumber,
                        timestamp = DateUtil.getCurrentDateTimestampWithDelta(
                            appStateManager.getServerTimestamp()
                        )
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                type = BaseVariable(type)
            ),
            businessKey = "${RequestTypeEnum.TROLLEY_INFO.requestType}-" +
                    "${appStateManager.getCurrentWorkerSimple()?.id}"
        )
    }

    override fun getVariableForBarcodeBox(barcodeValue: String, workerId: String): VariableRequest {
        return VariableRequest(
            variables = BarcodeBoxRequest(
                type = BaseVariable(RequestTypeEnum.BARCODE_BOX.requestType),
                meta = BaseVariable(
                    BarcodeBoxRequest.MetaRequest(
                        workerId = workerId,
                        deviceId = null,
                        sessionId = appStateManager.getSessionId(),
                        deviceTimeMark = appStateManager.getDeviceTimeInfo(),
                        serverTimeMark = appStateManager.getActualAppInfo()?.serverDataPayload?.timestamp,
                        deltaTimestamp = appStateManager.getServerTimestamp()
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                ),
                payload = BaseVariable(
                    BarcodeBoxRequest.PayloadRequest(
                        timestamp = DateUtil.getCurrentDateTimestampWithDelta(appStateManager.getServerTimestamp()),
                        barcodeValue = barcodeValue
                    ).toString(),
                    type = ObjectRequestTypeEnum.OBJECT.objectRequestType,
                    valueInfo = JsonValueInfo()
                )
            ),
            businessKey = "${RequestTypeEnum.BARCODE_BOX.requestType}-" +
                    "${appStateManager.getCurrentWorkerSimple()?.id}"
        )
    }
}
