package ru.daotech.agroinvest.components

import android.os.Bundle
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentTransaction
import ru.daotech.agroinvest.R


class BaseNavigationFragment<T : Fragment>(
    private val fragmentManager: FragmentManager,
    private val necessaryFragment: T,
    private val transitionCode: Int?,
    val fragmentContainer: Int?,
    val fragmentTag: String?
) {

    fun addBaseFragment(argumentsValue: Map<String, String>?) {
        val fragmentTransaction = buildBaseView(argumentsValue)
        fragmentTransaction.add(
            necessaryFragment,
            fragmentTag
        )
        fragmentTransaction.commit()
    }

    fun replaceBaseFragment(argumentsValue: Map<String, String>?) {
        val fragmentTransaction = buildBaseView(argumentsValue)
        fragmentTransaction.replace(
            fragmentContainer!!,
            necessaryFragment,
            fragmentTag
        )
        fragmentTransaction.commit()
    }

    private fun buildBaseView(argumentsValue: Map<String, String>?): FragmentTransaction {
        val fragmentTransaction = fragmentManager.beginTransaction()
        val arguments = Bundle()

        when (transitionCode) {
            HORIZONTAL_TRANSITION_CODE -> {
                fragmentTransaction.setCustomAnimations(
                    R.anim.enter_from_right,
                    R.anim.exit_to_left,
                    R.anim.enter_from_left,
                    R.anim.exit_to_right
                )
            }
            FREE_TRANSITION_CODE -> {

            }
            FADE_TRANSITION_CODE -> {
                fragmentTransaction.setCustomAnimations(
                    R.anim.fade_in,
                    R.anim.fade_out
                )
            }
        }

        argumentsValue
            ?.forEach { (key, value) ->
                arguments.putString(key, value)
            }
        arguments
            .takeIf { !it.isEmpty }
            .apply {
                necessaryFragment.arguments = arguments
            }

        necessaryFragment.retainInstance = true
        return fragmentTransaction
    }

    companion object {
        const val FREE_TRANSITION_CODE = 0
        const val HORIZONTAL_TRANSITION_CODE = 1
        const val FADE_TRANSITION_CODE = 2
    }
}