package ru.daotech.agroinvest.components

import android.annotation.SuppressLint
import android.content.Context
import android.os.Bundle
import android.view.View
import android.view.View.GONE
import android.view.View.VISIBLE
import androidx.fragment.app.FragmentManager
import com.jakewharton.rxbinding2.view.clicks
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.fragment_detailed_task.view.*
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.BuildConfig
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.components.BaseNavigationFragment.Companion.HORIZONTAL_TRANSITION_CODE
import ru.daotech.agroinvest.coordinators.NFC_TROLLEY_NUMBER_DETAILED_ARGS
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.fragments.barcode_box.BarcodeBoxFragment
import ru.daotech.agroinvest.fragments.detailed_task.DetailedTaskFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.DetailedTaskProcessingTab
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.choose_group_container.ChooseGroupFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.group_properties_container.GroupPropertyFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.header_field_container.HeaderFieldFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.timer_container.TimerMviFragment
import ru.daotech.agroinvest.fragments.pages.accident.AccidentReportFragment
import ru.daotech.agroinvest.fragments.trolley_tied.TrolleyTiedFragment
import ru.daotech.agroinvest.utils.*
import java.util.concurrent.TimeUnit

open class NavigationFragmentComponentImpl(val appStateManager: AppStateManager) :
    NavigationFragmentComponent {

    override fun buildAndShowTrolleyFragment(
        currentFragmentManager: FragmentManager,
        nfcMessageArgument: String?
    ) {
        val fragmentTransaction = currentFragmentManager.beginTransaction()

        val trolleyTiedFragment = TrolleyTiedFragment()
        val arguments = Bundle()
        fragmentTransaction.setCustomAnimations(
            R.anim.enter_from_right,
            R.anim.exit_to_left,
            R.anim.enter_from_left,
            R.anim.exit_to_right
        )
        arguments.putString(NFC_TROLLEY_NUMBER_DETAILED_ARGS, nfcMessageArgument)
        trolleyTiedFragment.arguments = arguments
        trolleyTiedFragment.retainInstance = true
        fragmentTransaction.add(
            R.id.nav_host_fragment,
            trolleyTiedFragment
        )
        fragmentTransaction.commit()
        currentFragmentManager.beginTransaction().apply {
            currentFragmentManager.fragments.forEach {
                if (it is DetailedTaskFragment) {
                    this.hide(it)
                }
            }
            commit()
        }
    }

    @SuppressLint("SetTextI18n")
    override fun buildAndShowTimerFragment(
        view: View,
        context: Context,
        currentChildFragmentManager: FragmentManager,
        taskWithProperties: TaskWithProperties,
        onGoToBarcodeBoxFragmentSubject: PublishSubject<Unit>
    ) {
        val fragmentTransaction = currentChildFragmentManager.beginTransaction()

        view.task_properties_row_number.text = context.getString(R.string.raw_title_prefix) +
                taskWithProperties.rowNumber?.toString()
        view.task_properties_job_type.text = taskWithProperties.section?.name

        view.properties_container.removeAllViews()

        // Launch task, if it has in progress status now
        val timerFragment = TimerMviFragment.newInstance()
        val arguments = Bundle()
        fragmentTransaction.setCustomAnimations(
            R.anim.enter_from_right,
            R.anim.exit_to_left,
            R.anim.enter_from_left,
            R.anim.exit_to_right
        )
        arguments.putParcelable(TASK_WITH_PROPERTIES_PARCELABLE, taskWithProperties)
        timerFragment.arguments = arguments
        timerFragment.retainInstance = true
        fragmentTransaction.replace(
            R.id.sub_detailed_task_fragment_container,
            timerFragment,
            TIMER_FRAGMENT_TAG
        )
        fragmentTransaction.commit()

        Observable.just(Unit)
            .delay(1, TimeUnit.SECONDS)
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeOn(AndroidSchedulers.mainThread())
            .map {
                if (taskWithProperties.properties.any { property ->
                        property.type == TASK_PROPERTY_GROUP_TYPE
                    }) {
                    view.scan_phenology_button.visibility = VISIBLE
                    view.scan_phenology_button.clicks()
                        .throttleFirst(2, TimeUnit.SECONDS)
                        .doOnNext {
                            buildBarcodeBoxDialogFragment(
                                currentChildFragmentManager,
                                taskWithProperties.id
                            )
//                            onGoToBarcodeBoxFragmentSubject.onNext(Unit)
                        }
                        .subscribe()
                    view.sub_header_field_fragment_container.visibility = GONE
                } else if (taskWithProperties.properties.any { property ->
                        property.type == TASK_PROPERTY_HEADER_TYPE
                    }) {
                    // Build child fragments with forms,
                    // when the defined properties exist
                    buildFormSubFragment(
                        view,
                        currentChildFragmentManager,
                        taskWithProperties
                    )
                } else if (taskWithProperties.sectionId == BuildConfig.phenologySectionId) {
                    view.scan_phenology_button.visibility = VISIBLE
                    view.scan_phenology_button.clicks()
                        .throttleFirst(2, TimeUnit.SECONDS)
                        .doOnNext {
                            buildBarcodeBoxDialogFragment(
                                currentChildFragmentManager,
                                taskWithProperties.id
                            )
//                            onGoToBarcodeBoxFragmentSubject.onNext(Unit)
                        }
                        .subscribe()
                }
            }
            .subscribe()
    }

    override fun buildFormSubFragment(
        view: View,
        currentChildFragmentManager: FragmentManager,
        taskWithProperties: TaskWithProperties
    ) {
        view.sub_header_field_fragment_container.visibility = VISIBLE

        val headerFieldFragment = HeaderFieldFragment.newInstance()
        BaseNavigationFragment(
            fragmentManager = currentChildFragmentManager,
            necessaryFragment = headerFieldFragment,
            transitionCode = HORIZONTAL_TRANSITION_CODE,
            fragmentTag = HEADER_FIELD_FRAGMENT_TAG,
            fragmentContainer = null
        ).addBaseFragment(
            mapOf(
                KEY_TASK_ID_HEADER_FIELD to taskWithProperties.id
            )
        )
    }

    override fun buildGroupPropertyFragment(
        currentChildFragmentManager: FragmentManager,
        taskId: String
    ) {
        val groupPropertyFragment = GroupPropertyFragment.newInstance()
        BaseNavigationFragment(
            fragmentManager = currentChildFragmentManager,
            necessaryFragment = groupPropertyFragment,
            transitionCode = HORIZONTAL_TRANSITION_CODE,
            fragmentTag = GROUP_PROPERTY_FRAGMENT_TAG,
            fragmentContainer = null
        ).addBaseFragment(
            mapOf(
                KEY_TASK_ID_GROUP_PROPERTY to taskId
            )
        )
    }

    override fun buildGroupPropertyFragmentByBarcode(
        currentChildFragmentManager: FragmentManager,
        taskId: String,
        barcodeValue: String
    ) {
        val groupPropertyFragment = GroupPropertyFragment.newInstance()
        BaseNavigationFragment(
            fragmentManager = currentChildFragmentManager,
            necessaryFragment = groupPropertyFragment,
            transitionCode = HORIZONTAL_TRANSITION_CODE,
            fragmentTag = GROUP_PROPERTY_FRAGMENT_TAG,
            fragmentContainer = null
        ).addBaseFragment(
            mapOf(
                KEY_TASK_ID_GROUP_PROPERTY to taskId,
                KEY_BARCODE_VALUE_GROUP_PROPERTY to barcodeValue
            )
        )
    }

    override fun buildBarcodeBoxDialogFragment(
        currentChildFragmentManager: FragmentManager,
        taskId: String
    ) {
        val barcodeBoxFragment = BarcodeBoxFragment.newInstance()
        BaseNavigationFragment(
            fragmentManager = currentChildFragmentManager,
            necessaryFragment = barcodeBoxFragment,
            transitionCode = HORIZONTAL_TRANSITION_CODE,
            fragmentTag = BARCODE_BOX_FRAGMENT_TAG,
            fragmentContainer = null
        ).addBaseFragment(
            mapOf(
                KEY_TASK_ID_BARCODE_BOX to taskId
            )
        )
    }

    override fun buildAndShowChooseGroupTypeDialog(
        fragmentManager: FragmentManager,
        currentTaskId: String
    ) {
        val chooseGroupFragment = ChooseGroupFragment.newInstance()
        BaseNavigationFragment(
            fragmentManager = fragmentManager,
            necessaryFragment = chooseGroupFragment,
            transitionCode = HORIZONTAL_TRANSITION_CODE,
            fragmentTag = CHOOSE_GROUP_FRAGMENT_TAG,
            fragmentContainer = null
        ).addBaseFragment(
            mapOf(
                KEY_TASK_ID_GROUP_PROPERTY to currentTaskId
            )
        )
    }

    @SuppressLint("SetTextI18n")
    override fun buildAndShowDetailedTaskInProgressFragment(
        view: View,
        context: Context,
        currentChildFragmentManager: FragmentManager,
        taskWithProperties: TaskWithProperties,
        onDismissTaskStates: PublishSubject<Unit>,
        showTaskSimpleView: () -> Unit
    ) {
        // Show task pre-launch screen, if task's status is pause or new now
        val detailedTaskProcessingFragment =
            DetailedTaskProcessingTab.newInstance()
        BaseNavigationFragment(
            fragmentManager = currentChildFragmentManager,
            necessaryFragment = detailedTaskProcessingFragment,
            transitionCode = HORIZONTAL_TRANSITION_CODE,
            fragmentTag = TASK_IN_PROGRESS_FRAGMENT_TAG,
            fragmentContainer = R.id.sub_detailed_task_fragment_container
        ).replaceBaseFragment(null)

        showTaskSimpleView()

        view.task_properties_row_number.text = context.getString(R.string.raw_title_prefix) +
                taskWithProperties.rowNumber?.toString()
        view.task_properties_job_type.text = taskWithProperties.section?.name

        onDismissTaskStates.onNext(Unit)
    }

    override fun buildAndShowAccidentFragment(
        currentChildFragmentManager: FragmentManager?,
        taskWithProperties: TaskWithProperties
    ) {
        val fragmentTransaction = currentChildFragmentManager?.beginTransaction()
        val accidentReportFragment =
            AccidentReportFragment()
        val arguments = Bundle()
        fragmentTransaction?.setCustomAnimations(
            R.anim.enter_from_right,
            R.anim.exit_to_left,
            R.anim.enter_from_left,
            R.anim.exit_to_right
        )
        arguments.putString(DetailedTaskFragment.KEY_TASK_ID, taskWithProperties.id)
        accidentReportFragment.arguments = arguments
        accidentReportFragment.retainInstance = true
        fragmentTransaction?.add(
            R.id.nav_host_fragment,
            accidentReportFragment
        )
        fragmentTransaction?.commit()
        currentChildFragmentManager!!.beginTransaction().apply {
            currentChildFragmentManager.fragments.forEach {
                if (it is DetailedTaskFragment) {
                    this.hide(it)
                }
            }
            commit()
        }
    }
}

interface NavigationFragmentComponent {

    fun buildAndShowTrolleyFragment(
        currentFragmentManager: FragmentManager,
        nfcMessageArgument: String?
    )

    fun buildFormSubFragment(
        view: View,
        currentChildFragmentManager: FragmentManager,
        taskWithProperties: TaskWithProperties
    )

    fun buildGroupPropertyFragment(currentChildFragmentManager: FragmentManager, taskId: String)

    fun buildGroupPropertyFragmentByBarcode(
        currentChildFragmentManager: FragmentManager,
        taskId: String,
        barcodeValue: String
    )

    fun buildAndShowDetailedTaskInProgressFragment(
        view: View,
        context: Context,
        currentChildFragmentManager: FragmentManager,
        taskWithProperties: TaskWithProperties,
        onDismissTaskStates: PublishSubject<Unit>,
        showTaskSimpleView: () -> Unit
    )

    fun buildAndShowAccidentFragment(
        currentChildFragmentManager: FragmentManager?,
        taskWithProperties: TaskWithProperties
    )

    fun buildAndShowTimerFragment(
        view: View,
        context: Context,
        currentChildFragmentManager: FragmentManager,
        taskWithProperties: TaskWithProperties,
        onGoToBarcodeBoxFragmentSubject: PublishSubject<Unit>
    )

    fun buildBarcodeBoxDialogFragment(
        currentChildFragmentManager: FragmentManager,
        taskId: String
    )

    fun buildAndShowChooseGroupTypeDialog(
        fragmentManager: FragmentManager,
        currentTaskId: String
    )
}