package ru.daotech.agroinvest.container

import android.annotation.SuppressLint
import android.app.PendingIntent
import android.app.SearchManager
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.location.Location
import android.location.LocationListener
import android.location.LocationManager
import android.net.ConnectivityManager
import android.nfc.NdefMessage
import android.nfc.NfcAdapter
import android.os.Bundle
import android.text.InputType
import android.util.AttributeSet
import android.view.*
import android.view.View.INVISIBLE
import android.view.View.VISIBLE
import android.widget.Button
import android.widget.TextView
import androidx.annotation.ColorRes
import androidx.annotation.IdRes
import androidx.appcompat.app.ActionBarDrawerToggle
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.widget.SearchView
import androidx.core.content.ContextCompat
import androidx.core.view.GravityCompat
import androidx.core.view.forEach
import androidx.drawerlayout.widget.DrawerLayout.LOCK_MODE_LOCKED_CLOSED
import androidx.drawerlayout.widget.DrawerLayout.LOCK_MODE_UNLOCKED
import androidx.fragment.app.Fragment
import androidx.navigation.NavDestination
import androidx.navigation.findNavController
import androidx.navigation.fragment.FragmentNavigator
import androidx.navigation.fragment.NavHostFragment
import androidx.navigation.ui.AppBarConfiguration
import androidx.navigation.ui.setupActionBarWithNavController
import com.google.android.material.navigation.NavigationView
import com.google.android.material.snackbar.Snackbar
import com.hannesdorfmann.mosby3.mvi.MviActivity
import com.jakewharton.rxbinding2.support.design.widget.RxNavigationView
import dagger.android.AndroidInjection
import dagger.android.AndroidInjector
import dagger.android.DispatchingAndroidInjector
import dagger.android.support.HasSupportFragmentInjector
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.menu_header.view.*
import ly.count.android.sdk.Countly
import org.greenrobot.eventbus.EventBus
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.BuildConfig.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.coordinators.Navigator
import ru.daotech.agroinvest.di.modules.DATABASE_NAME
import ru.daotech.agroinvest.fragments.detailed_task.DetailedTaskFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.NfcScanned
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.timer_container.NfcTimerScanned
import ru.daotech.agroinvest.fragments.earnings.EarningsFragment
import ru.daotech.agroinvest.fragments.login.LoginFragment
import ru.daotech.agroinvest.fragments.pages.about.AboutPageFragment
import ru.daotech.agroinvest.fragments.ratings.RatingsFragment
import ru.daotech.agroinvest.fragments.sections.root.SectionsFragment
import ru.daotech.agroinvest.fragments.support_notification.SupportNotificationFragment
import ru.daotech.agroinvest.fragments.tasks.collectionHistory.CollectionHistoryFragment
import ru.daotech.agroinvest.fragments.tasks.myTasks.NfcAppScanned
import ru.daotech.agroinvest.fragments.tasks.myTasks.TasksFragment
import ru.daotech.agroinvest.fragments.tasks.tasksHistory.TasksHistoryFragment
import ru.daotech.agroinvest.fragments.trolley_tied.NfcTrolleyTiedScanned
import ru.daotech.agroinvest.hide
import ru.daotech.agroinvest.models.InternetConnectivityState
import ru.daotech.agroinvest.service.CamundaService
import ru.daotech.agroinvest.service.QueueService
import ru.daotech.agroinvest.service.SocketService
import ru.daotech.agroinvest.service.couch.CouchDbService
import ru.daotech.agroinvest.show
import ru.daotech.agroinvest.utils.*
import ru.daotech.agroinvest.utils.ServiceUtil.Companion.processNdefMessages
import ru.daotech.agroinvest.utils.ServiceUtil.Companion.viewToFile
import timber.log.Timber
import java.io.File
import java.util.concurrent.TimeUnit
import javax.inject.Inject

var GPSLocation: String? = null

class ContainerActivity : MviActivity<ContainerView, ContainerPresenter>(), ContainerView,
    HasSupportFragmentInjector, AppBarController {

    protected lateinit var snackbar: Snackbar

    @Inject
    lateinit var fragmentInjector: DispatchingAndroidInjector<Fragment>

    @Inject
    lateinit var containerPresenter: ContainerPresenter

    @Inject
    lateinit var navigator: Navigator

    @Inject
    lateinit var appStateManager: AppStateManager

    private var disposable = CompositeDisposable()
    private var connectivityStateDisposable = CompositeDisposable()

    private val onLoginInitSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onSignOutAccept: PublishSubject<Unit> = PublishSubject.create()
    private val toolbarVisibilitySubject: PublishSubject<Boolean> = PublishSubject.create()
    private val dismissSnackbarSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onReturnToMyTasksSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onStableConnectionDataUpload: PublishSubject<Unit> = PublishSubject.create()
    private val dismissSignOutResult: PublishSubject<Unit> = PublishSubject.create()
    private val forcedLogoutSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onSearchSubmitSubject: PublishSubject<String> = PublishSubject.create()
    private val exitApplicationSubject: PublishSubject<Unit> = PublishSubject.create()
    val onListenUnreadMessageSubject: PublishSubject<Unit> = PublishSubject.create()

    private var nfcAdapter: NfcAdapter? = null
    private var nfcPendingIntent: PendingIntent? = null
    var isInternetConnectedGlobal = true
    var isSocketConnectedGlobal = true

    private var locationManager: LocationManager? = null
    private var menu: Menu? = null
    private val menuItemSelectedObservable by lazy {
        RxNavigationView.itemSelections(navigation_view).share()
    }

    private lateinit var drawerToggle: ActionBarDrawerToggle

    override fun createPresenter(): ContainerPresenter {
        return containerPresenter
    }

    override fun onStart() {
        super.onStart()

        if (appStateManager.getCurrentWorkerSimple() != null
            && !appStateManager.isWorkSessionOverByDays()
        ) {
            exitApplicationSubject.onNext(Unit)
        }

        if (Countly.sharedInstance().isInitialized) {
            Countly.sharedInstance().onStart(this)
        }
    }

    override fun onStop() {
        if (Countly.sharedInstance().isInitialized) {
            Countly.sharedInstance().onStop()
        }
        super.onStop()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        AndroidInjection.inject(this)
        super.onCreate(savedInstanceState)

        try {
            setContentView(R.layout.activity_container)
        } catch (ex: NoClassDefFoundError) {
            Timber.e("Set content view exception: FAILED")
        }
        setupAppBar()

        val navController = findNavController(R.id.nav_host_fragment)
        navController.addOnDestinationChangedListener { _, destination, _ ->
            if (matchDestination(destination, R.id.loginFragment)) {
                onLoginInitSubject.onNext(Unit)
            }
        }
        navigator.init(navController)

        nfcAdapter = NfcAdapter.getDefaultAdapter(this)
        Timber.d("NFC supported${(nfcAdapter != null)}")
        Timber.d("NFC enabled${(nfcAdapter?.isEnabled).toString()}")

        nfcPendingIntent = PendingIntent.getActivity(
            this, 0,
            Intent(this, javaClass).addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP), 0
        )

        if (isRabbitMQEnable != null && isRabbitMQEnable) {
            startQueueService()
        }

        if (isCouchDBEnable != null && isCouchDBEnable) {
            startCouchDbService()
        }

        startCamundaService()
//        startSocketService()

        registerReceiver(
            appStateManager.connectionReceiver,
            IntentFilter(ConnectivityManager.CONNECTIVITY_ACTION)
        )

        if (savedInstanceState != null) {
            val title = savedInstanceState.getString(KEY_TOOLBAR_TITLE)
            activity_container_custom_toolbar.title = title

            val subtitle = savedInstanceState.getString(KEY_TOOLBAR_SUBTITLE)
            activity_container_custom_toolbar.subtitle = subtitle
        }

        // Create persistent LocationManager reference
        locationManager = getSystemService(LOCATION_SERVICE) as LocationManager?
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)

        outState.putString(KEY_TOOLBAR_TITLE, activity_container_custom_toolbar?.title.toString())
        outState.putString(KEY_TOOLBAR_SUBTITLE, activity_container_custom_toolbar?.subtitle.toString())
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menu?.let {
            menuInflater.inflate(R.menu.toolbar_menu, menu)

            this.menu = menu

            try {
                if (supportFragmentManager.findFragmentById(R.id.nav_host_fragment) != null) {
                    val navHostFragment =
                        supportFragmentManager.findFragmentById(R.id.nav_host_fragment) as NavHostFragment
                    if (navHostFragment.childFragmentManager.fragments.size > 0
                        && navHostFragment.childFragmentManager.fragments[0] !is DetailedTaskFragment
                    ) {
                        // Associate searchable configuration with the SearchView
                        val searchManager = getSystemService(Context.SEARCH_SERVICE) as SearchManager

                        (this.menu?.findItem(R.id.menu_search)?.actionView as SearchView).apply {
                            setSearchableInfo(searchManager.getSearchableInfo(componentName))

                            this.inputType = InputType.TYPE_CLASS_NUMBER
                            this.setIconifiedByDefault(true)

                            if (!appStateManager.getFilterRowNumber().isNullOrEmpty()) {
                                this.isIconified = false
                                this.setQuery(appStateManager.getFilterRowNumber(), false)
                                this.clearFocus()
                            }

                            this.setOnQueryTextListener(object : SearchView.OnQueryTextListener {
                                override fun onQueryTextChange(newText: String): Boolean {
                                    // Timber.d("SERCHTEXT CHANGE: " + newText)
                                    return false
                                }

                                override fun onQueryTextSubmit(query: String): Boolean {
                                    onSearchSubmitSubject.onNext(query)

                                    return false
                                }
                            })

                            this.setOnCloseListener(object : SearchView.OnCloseListener {
                                override fun onClose(): Boolean {
                                    onSearchSubmitSubject.onNext("")

                                    return false
                                }
                            })
                        }
                    }
                }
            } catch (ex: IndexOutOfBoundsException) {
                Timber.e(ex.localizedMessage)
            }
        }

        return true
    }

    override fun onPrepareOptionsMenu(menu: Menu?): Boolean {
        val navController = findNavController(R.id.nav_host_fragment)
        navController.addOnDestinationChangedListener { _, destination, _ ->
            navigation_view.menu.forEach { menuItem ->
                menuItem.isChecked = matchDestination(destination, menuItem.itemId)
            }
        }

        menu?.let {
            with(it) {
                // Socket connectivity state
                if (isSocketConnectedGlobal) {
                    (findItem(R.id.connectivity_socket_action) as MenuItem).isVisible = false
                } else {
                    (findItem(R.id.connectivity_socket_action) as MenuItem)
                        .setIcon(R.drawable.ic_cloud_off_queue_red_24dp)
                    (findItem(R.id.connectivity_socket_action) as MenuItem).isVisible = true
                }

                // Internet connectivity state
                if (isInternetConnectedGlobal) {
                    (findItem(R.id.connectivity_action) as MenuItem).isVisible = false
                } else {
                    (findItem(R.id.connectivity_action) as MenuItem)
                        .setIcon(R.drawable.ic_signal_wifi_off_24dp)
                    (findItem(R.id.connectivity_action) as MenuItem).isVisible = true
                }
            }
        }

        return super.onPrepareOptionsMenu(menu)
    }

    override fun supportFragmentInjector(): AndroidInjector<Fragment> {
        return fragmentInjector
    }

    override fun toolbarVisibilityChanged(): Observable<Boolean> = toolbarVisibilitySubject

    override fun internetConnectionChanged(): Observable<Pair<Boolean, InternetConnectivityState>> =
        appStateManager.connectionReceiver.connectivitySubject

    override fun screenshotSignalReceived(): Observable<File> =
        appStateManager.screenshotMessageEventSubject
            .map { viewToFile(window!!.decorView.rootView) }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)

        if (requestCode == GPS_PERMISSION_CODE) {
            if (
                grantResults.isNotEmpty()
                && grantResults[0] == PackageManager.PERMISSION_GRANTED
            ) {
                try {
                    locationManager?.requestLocationUpdates(
                        LocationManager.NETWORK_PROVIDER,
                        0L,
                        0f,
                        locationListener
                    )
                } catch (ex: SecurityException) {
                    Timber.e(ex)
                }
            }
        }

        if (requestCode == EXTERNAL_WRITE_PERMISSION_CODE) {
            if (
                grantResults.isNotEmpty()
                && grantResults[0] == PackageManager.PERMISSION_GRANTED
            ) {
                try {
                    Timber.i("EXTERNAL WRITE PERMISSIONS HAS GRANTED")
                } catch (ex: SecurityException) {
                    Timber.e(ex)
                }
            }
        }

        if (requestCode == EXTERNAL_READ_PERMISSION_CODE) {
            if (
                grantResults.isNotEmpty()
                && grantResults[0] == PackageManager.PERMISSION_GRANTED
            ) {
                try {
                    Timber.i("EXTERNAL READ PERMISSIONS HAS GRANTED")
                } catch (ex: SecurityException) {
                    Timber.e(ex)
                }
            }
        }
    }

    override fun onCreateView(name: String, context: Context, attrs: AttributeSet): View? {
        if (connectivityStateDisposable.size() != 0) {
            connectivityStateDisposable.clear()
        }

        connectivityStateDisposable.add(
            Observable.ambArray(
                appStateManager.connectionReceiver.connectivityViewSubject,
                appStateManager.connectionReceiver.connectivitySubject
            )
                .debounce(2, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .map { isInternetConnected ->
                    // Socket connectivity state
                    when (isInternetConnected) {
                        is Boolean -> {
                            if (isInternetConnectedGlobal != isInternetConnected) {
                                isInternetConnectedGlobal = isInternetConnected

                            }
                        }
                        is Pair<*, *> -> {
                            if (isInternetConnectedGlobal != (isInternetConnected.first as Boolean)) {
                                isInternetConnectedGlobal = isInternetConnected.first as Boolean
                            }
                        }
                    }

                    invalidateOptionsMenu()
                }
                .subscribe()
        )

        connectivityStateDisposable.add(
            appStateManager.onSocketConnectivityState
                .debounce(2, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .map { isSocketConnected ->
                    // Socket connectivity state
                    if (isSocketConnectedGlobal != isSocketConnected) {
                        isSocketConnectedGlobal = isSocketConnected

                    }

                    invalidateOptionsMenu()
                }
                .subscribe()
        )

        return super.onCreateView(name, context, attrs)
    }

    override fun loadingIntent(): Observable<Unit> = Observable.just(Unit)

    override fun onLoginInit(): Observable<Unit> = onLoginInitSubject

    override fun onDismissSnackbarIntent(): Observable<Unit> = dismissSnackbarSubject

    override fun onDismissSignOutResult(): Observable<Unit> = dismissSignOutResult

    override fun onListenUnreadMessageIntent(): Observable<Unit> = onListenUnreadMessageSubject

    override fun goToMyTasksIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.filter { it.itemId == R.id.tasksFragment } //IMPORTANT: Menu Id must be the same as Destination Id

    override fun returnToMyTasksIntent(): Observable<Unit> = onReturnToMyTasksSubject

    override fun goToTasksHistoryIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.filter { it.itemId == R.id.tasksHistoryFragment }

    override fun goToCollectionHistoryIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.filter { it.itemId == R.id.collectionHistoryFragment }

    override fun goToSectionsIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.filter { it.itemId == R.id.sectionsFragment }

    override fun goToAccidentReportIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.filter { it.itemId == R.id.accidentReportFragment }

    override fun goToAboutPageIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.filter { it.itemId == R.id.aboutPageFragment }

    override fun goToUpdateAppIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.filter { it.itemId == R.id.updateAppFragment }

    override fun goToRatingsIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.filter { it.itemId == R.id.ratingsFragment }

    override fun goToEarningsIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.filter { it.itemId == R.id.earningsFragment }

    override fun goToSupportMessageIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.filter { it.itemId == R.id.supportNotifyFragment }

    override fun finishDayIntent(): Observable<MenuItem> =
        menuItemSelectedObservable.doOnNext {
            when (it.itemId == R.id.item_finish_day) {
                true -> createFinishDayDialog()
            }
        }

    override fun onSignOutAcceptIntent(): Observable<Unit> = onSignOutAccept

    override fun onForcedLogoutIntent(): Observable<Unit> = forcedLogoutSubject

    override fun exitApplicationIntent(): Observable<Unit> = exitApplicationSubject

    override fun onSearchSubmitSubjectIntent(): Observable<String> = onSearchSubmitSubject

    override fun onUploadBackupSignal(): Observable<List<File>?> =
        appStateManager.uploadBackupMessageEventSubject
            .map { return@map ServiceUtil.exportDB(this.getDatabasePath(DATABASE_NAME).parent) }
            .doOnError { error -> Timber.e(error) }

    override fun onStableConnectionDataUploadIntent(): Observable<Unit> =
        onStableConnectionDataUpload

    override fun render(state: ContainerViewState) {
        manageErrors(state)

        with(state) {
            drawer_layout.closeDrawer(navigation_view)
            navigation_view.getHeaderView(0).update_app_version_title.text = name
            navigation_view.getHeaderView(0).personal_number_text.text =
                getString(R.string.personal_number_hash, personalNumber)

            if (isLoadInProgress) {
                window.addFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE)
                progressBarContainer.show()
            } else {
                window.clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE)
                progressBarContainer.hide()
            }

            navigation_view?.let { currentMenu ->
                currentMenu.menu.findItem(R.id.updateAppFragment).isVisible = isVersionIncrease
            }

            if (state.signOutResult != null) {
                if (state.signOutResult.isSuccessResult) {
                    showSnackbar(state.signOutResult.message, R.color.colorAccent)
                    forcedLogoutSubject.onNext(Unit)
                    dismissSignOutResult.onNext(Unit)

                    if (isRabbitMQEnable != null && isRabbitMQEnable) {
                        restartQueueService()
                    }

                    if (isSocketIOEnable != null && isSocketIOEnable) {
                        stopSocketService()
                    }
                } else {
                    showSnackbar(state.signOutResult.message, R.color.accentSnackbarBackground)
                    dismissSignOutResult.onNext(Unit)

                    if (isRabbitMQEnable != null && isRabbitMQEnable) {
                        restartQueueService()
                    }
                }
            }

            state.unreadMessageCount?.let { unreadMessageCount ->
                if (unreadMessageCount != 0) {
                    updateNavViewUnreadMessageCount(navigation_view, unreadMessageCount.toString())
                } else {
                    hideNavViewUnreadMessageCount(navigation_view)
                }
            }
        }
    }

    // On view handling errors
    private fun manageErrors(state: ContainerViewState) {
        with(state) {
            when {
                errorMessage != null -> {
                    restartQueueService()
                    showSnackbar(errorMessage)
                }
                broadcastMessage != null -> {
                    showSnackbar(
                        broadcastMessage.message,
                        if (broadcastMessage.type == "message")
                            R.color.colorAccent
                        else
                            R.color.accentSnackbarBackground,
                        withOkAction = true
                    )
                }
                else -> {
                }
            }
        }
    }

    override fun onResume() {
        super.onResume()
        nfcAdapter?.enableForegroundDispatch(
            this,
            nfcPendingIntent,
            null,
            null
        )
    }

    override fun hideAppBar() {
        activity_container_custom_toolbar.hide()
        toolbarVisibilitySubject.onNext(false)
        drawer_layout.setDrawerLockMode(LOCK_MODE_LOCKED_CLOSED)
    }

    override fun showAppBar() {
        activity_container_custom_toolbar.show()
        toolbarVisibilitySubject.onNext(true)
        drawer_layout.setDrawerLockMode(LOCK_MODE_UNLOCKED)
    }

    fun showToggle() {
        drawerToggle.isDrawerIndicatorEnabled = true;
    }

    override fun setupAppBar() {
        setSupportActionBar(activity_container_custom_toolbar)

        supportActionBar?.apply {
            setDisplayHomeAsUpEnabled(true)
            setHomeButtonEnabled(true)
            setDisplayShowTitleEnabled(false)
        }

        drawerToggle = ActionBarDrawerToggle(
            this,
            drawer_layout,
            activity_container_custom_toolbar,
            R.string.drawer_open,
            R.string.drawer_close
        )

        drawer_layout.addDrawerListener(drawerToggle)
        drawerToggle.syncState()
        setupAppBarWithNavController()
    }

    private fun updateNavViewUnreadMessageCount(navView: NavigationView, count: String) {
        val item = navView.menu.findItem(R.id.supportNotifyFragment)
        item.setActionView(R.layout.item_message_count_badge)
        item.actionView.visibility = VISIBLE
        item.actionView.findViewById<TextView>(R.id.cart_badge).text = count
    }

    private fun hideNavViewUnreadMessageCount(navView: NavigationView) {
        val item = navView.menu.findItem(R.id.supportNotifyFragment)
        item.setActionView(R.layout.item_message_count_badge)
        item.actionView.visibility = INVISIBLE
    }

    override fun onBackPressed() {
        val navController = findNavController(R.id.nav_host_fragment)
        val navHost = nav_host_fragment.childFragmentManager.fragments.firstOrNull()
        val currentDestinationName =
            (navController.currentDestination as FragmentNavigator.Destination).className

        navHost?.let { navHost ->
            when {
                (navHost is LoginFragment ||
                        currentDestinationName == LoginFragment().javaClass.name) -> {
                    drawer_layout.closeDrawer(GravityCompat.START)
                    return
                }
            }

            when {
                (navHost is TasksFragment
                        || navHost is TasksHistoryFragment
                        || navHost is CollectionHistoryFragment
                        || navHost is RatingsFragment
                        || navHost is EarningsFragment
                        || navHost is SectionsFragment
                        || navHost is SupportNotificationFragment
                        || navHost is AboutPageFragment)
                        && supportFragmentManager.backStackEntryCount == 0
                        && (navController.currentDestination?.id == R.id.tasksFragment
                        || currentDestinationName == TasksHistoryFragment().javaClass.name
                        || currentDestinationName == CollectionHistoryFragment().javaClass.name
                        || currentDestinationName == RatingsFragment().javaClass.name
                        || currentDestinationName == EarningsFragment().javaClass.name
                        || currentDestinationName == SectionsFragment().javaClass.name
                        || currentDestinationName == SupportNotificationFragment().javaClass.name
                        || currentDestinationName == AboutPageFragment().javaClass.name) -> {
                    when (drawer_layout.isDrawerOpen(GravityCompat.START)) {
                        true -> {
                            drawer_layout.closeDrawer(GravityCompat.START)
                        }
                        else -> {
                            drawer_layout.openDrawer(GravityCompat.START)
                        }
                    }
                }
                navController.currentDestination?.id == R.id.updateAppFragment -> {
                    onReturnToMyTasksSubject.onNext(Unit)
                }
                else -> {
                    try {
                        super.onBackPressed()
                    } catch (ex: IllegalStateException) {
                        Timber.e(ex)
                    }
                }
            }
        }
    }

    override fun setupAppBarWithNavController() {
        val navController = findNavController(R.id.nav_host_fragment)

        val topLevelDestinationIds =
            setOf(
                R.id.tasksFragment,
                R.id.tasksHistoryFragment,
                R.id.collectionHistoryFragment,
                R.id.sectionsFragment,
                R.id.updateAppFragment,
                R.id.ratingsFragment,
                R.id.earningsFragment
            )

        val appBarConfiguration = AppBarConfiguration(topLevelDestinationIds, drawer_layout)

        activity_container_custom_toolbar?.setNavigationOnClickListener {
            if (topLevelDestinationIds.contains(navController.currentDestination?.id)) {
                drawer_layout.openDrawer(navigation_view)
            } else {
                this.onBackPressed()
            }
        }

        setupActionBarWithNavController(navController, appBarConfiguration)
    }

    override fun hideOnBackAppBarItem() {
        hideAppBar()
        actionBar?.setDisplayHomeAsUpEnabled(false)
        actionBar?.setDisplayShowHomeEnabled(false)
        actionBar?.setHomeButtonEnabled(false)
        activity_container_custom_toolbar.navigationIcon = null
    }

    private fun matchDestination(destination: NavDestination, @IdRes destId: Int): Boolean {
        var currentDestination: NavDestination? = destination
        while (currentDestination!!.id != destId && currentDestination.parent != null) {
            currentDestination = currentDestination.parent
        }
        return currentDestination.id == destId
    }

    @SuppressLint("InflateParams")
    private fun createFinishDayDialog() {
        val factory = LayoutInflater.from(this)
        val finishDayDialogView = factory.inflate(R.layout.dialog_fragment_signout, null)
        val finishDayDialog: AlertDialog = AlertDialog.Builder(this).create()
        finishDayDialog.setView(finishDayDialogView)
        finishDayDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        finishDayDialogView.findViewById<Button>(R.id.cancel_signout_button)
            ?.setOnClickListener { finishDayDialog.dismiss() }
        finishDayDialogView.findViewById<Button>(R.id.confirm_signout_button)
            ?.setOnClickListener {
                finishDayDialog.dismiss()
                onSignOutAccept.onNext(Unit)
            }
        finishDayDialog.show()
    }

    override fun onNewIntent(newIntent: Intent?) {
        super.onNewIntent(newIntent)

        newIntent?.let { intent ->
            if (intent.action == NfcAdapter.ACTION_NDEF_DISCOVERED) {
                // Retrieve the raw NDEF message from the tag
                val rawMessages = intent.getParcelableArrayExtra(NfcAdapter.EXTRA_NDEF_MESSAGES)
                // Complete variant: parse NDEF messages
                if (rawMessages != null) {
                    val messages = rawMessages.map { rawMessage -> rawMessage as NdefMessage }
                    val message = processNdefMessages(messages)
                    if (!message.isNullOrEmpty()) {
                        // NFC message from DetailedTaskFragment
                        EventBus.getDefault().post(
                            NfcScanned(message)
                        )
                        // NFC message from TasksFragment
                        EventBus.getDefault().post(
                            NfcAppScanned(message)
                        )
                        // NFC message from TimerMviFragment
                        EventBus.getDefault().post(
                            NfcTimerScanned(message)
                        )
                        // NFC message from TrolleyTiedFragment
                        EventBus.getDefault().post(
                            NfcTrolleyTiedScanned(message)
                        )
                    }
                }
            }
        }
    }

    override fun onPause() {
        nfcAdapter?.disableForegroundDispatch(this)
        super.onPause()
    }

    override fun onDestroy() {
        super.onDestroy()
        disposable.dispose()

        if (isRabbitMQEnable != null && isRabbitMQEnable) {
            stopQueueService()
        }

        if (isCouchDBEnable != null && isCouchDBEnable) {
            stopCouchDbService()
        }

        if (isSocketIOEnable != null && isSocketIOEnable) {
            stopSocketService()
        }

        stopCamundaService()

        unregisterReceiver(appStateManager.connectionReceiver)

        forcedLogoutSubject.onNext(Unit)
    }

    private fun showSnackbar(
        message: String,
        @ColorRes snackbarBackgroundColor: Int = R.color.accentSnackbarBackground,
        withOkAction: Boolean = false
    ) {
        snackbar = if (withOkAction) {
            Snackbar.make(
                findViewById(android.R.id.content)!!,
                message,
                Snackbar.LENGTH_LONG
            )
                .apply {
                    addCallback(object : Snackbar.Callback() {
                        @SuppressLint("SwitchIntDef")
                        override fun onDismissed(transientBottomBar: Snackbar?, event: Int) {
                            when (event) {
                                DISMISS_EVENT_SWIPE -> {
                                    dismissSnackbarSubject.onNext(Unit)
                                    dismiss()
                                }
                                DISMISS_EVENT_TIMEOUT -> {
                                    dismissSnackbarSubject.onNext(Unit)
                                    dismiss()
                                }
                                else -> {
                                    super.onDismissed(transientBottomBar, event)
                                }
                            }
                        }
                    })
                    setAction("ОК") {
                        dismissSnackbarSubject.onNext(Unit)
                        this.dismiss()
                    }
                    setActionTextColor(
                        ContextCompat.getColor(
                            this@ContainerActivity,
                            android.R.color.white
                        )
                    )
                }
        } else {
            Snackbar.make(findViewById(android.R.id.content)!!, message, Snackbar.LENGTH_SHORT)
        }
        val backgroundColor =
            ContextCompat.getColor(this@ContainerActivity, snackbarBackgroundColor)

        snackbar.view.setBackgroundColor(backgroundColor)
        snackbar.show()
    }

    protected fun dismissSnackbar() {
        if (::snackbar.isInitialized) {
            snackbar.dismiss()
        }
    }

    private fun restartQueueService() {
        stopQueueService()
        startQueueService()
    }

    private fun startQueueService() {
        Intent(this@ContainerActivity, QueueService::class.java).also { intent ->
            startService(intent)
        }
    }

    private fun stopQueueService() {
        Intent(this@ContainerActivity, QueueService::class.java).also { intent ->
            stopService(intent)
        }
    }

    private fun startCamundaService() {
        Intent(this@ContainerActivity, CamundaService::class.java).also { intent ->
            startService(intent)
        }
    }

    private fun stopCamundaService() {
        Timber.e("CAMUNDA SERVICE: DESTROY INITIATED")

        Intent(this@ContainerActivity, CamundaService::class.java).also { intent ->
            stopService(intent)
        }
    }

    fun startSocketService() {
        Intent(this@ContainerActivity, SocketService::class.java).also { intent ->
            startService(intent)
        }
    }

    private fun startCouchDbService() {
        Intent(this@ContainerActivity, CouchDbService::class.java).also { intent ->
            startService(intent)
        }
    }

    private fun stopSocketService() {
        Intent(this@ContainerActivity, SocketService::class.java).also { intent ->
            stopService(intent)
        }
    }

    private fun stopCouchDbService() {
        Intent(this@ContainerActivity, CouchDbService::class.java).also { intent ->
            stopService(intent)
        }
    }
}

private val locationListener: LocationListener = object : LocationListener {
    override fun onLocationChanged(location: Location) {
        GPSLocation = ("" + location.longitude + ":" + location.latitude)
    }

    override fun onStatusChanged(provider: String, status: Int, extras: Bundle) {}
    override fun onProviderEnabled(provider: String) {}
    override fun onProviderDisabled(provider: String) {}
}
