package ru.daotech.agroinvest.container

import ru.daotech.agroinvest.models.Worker
import ru.daotech.agroinvest.service.events.BroadcastMessage
import ru.daotech.agroinvest.service.events.SupportMessageInfo

sealed class ContainerPartialViewState {

    abstract fun reduce(previousState: ContainerViewState): ContainerViewState

    data class InitialData(val worker: Worker) : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                name = worker.name,
                personalNumber = worker.personalNumber,
                isFinishedDayDialogOpen = false,
                signOutResult = null,
                isLoadInProgress = false,
                errorMessage = null
            )
        }
    }

    object EmptyState : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object ListenUnreadMessageCountState : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToMyTasks : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToTasksHistory : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToCollectionHistory : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToSections : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToAccidentReport : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToAboutPage : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToUpdateApp : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToRatings : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToEarnings : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToBarcodeBox : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object GoingToSupportNotify : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object FinishDay : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    object LoadInProgress : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                isLoadInProgress = true,
                isFinishedDayDialogOpen = false,
                errorMessage = null
            )
        }
    }

    object InvalidateToolbar : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState
        }
    }

    data class UpdateUnreadMessageCountMenu(val unreadMessageCount: Int) : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                unreadMessageCount = unreadMessageCount,
                errorMessage = null
            )
        }
    }


    data class ShowBroadcastMessage(val message: BroadcastMessage) : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                broadcastMessage = message,
                errorMessage = null
            )
        }
    }

    data class ShowSupportMessage(val message: SupportMessageInfo) : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                supportMessage = message.message,
                errorMessage = null
            )
        }
    }

    data class ShowUpdateAppVersionMenuItem(val isVersionIncrease: Boolean) : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                isVersionIncrease = isVersionIncrease,
                errorMessage = null
            )
        }
    }

    object DismissSnackbar : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                broadcastMessage = null,
                signOutResult = null,
                isSnackBarNeedShowing = false,
                errorMessage = null
            )
        }
    }

    object UploadDataSuccess : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy()
        }
    }

    object LoginInit : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                isFinishedDayDialogOpen = false,
                signOutResult = null,
                isLoadInProgress = false,
                errorMessage = null
            )
        }
    }

    object DismissSignOutResult : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                signOutResult = null,
                errorMessage = null
            )
        }
    }

    data class ShowSnackbarError(val errorMessage: String) : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                errorMessage = errorMessage
            )
        }
    }

    data class LogoutSuccess(val signOutResult: SignOutResult) : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                signOutResult = signOutResult,
                isFinishedDayDialogOpen = false,
                isLoadInProgress = false,
                errorMessage = null
            )
        }
    }

    data class LogoutError(val signOutResult: SignOutResult) : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                signOutResult = signOutResult,
                isSnackBarNeedShowing = true,
                isFinishedDayDialogOpen = false,
                isLoadInProgress = false,
                errorMessage = null
            )
        }
    }

    data class InternetConnectionChanged(val isConnected: Boolean) : ContainerPartialViewState() {
        override fun reduce(previousState: ContainerViewState): ContainerViewState {
            return previousState.copy(
                isInternetConnected = isConnected,
                isSocketConnected = isConnected,
                errorMessage = null
            )
        }
    }
}