package ru.daotech.agroinvest.coordinators

import androidx.navigation.NavController
import androidx.navigation.NavOptions
import androidx.navigation.navOptions
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.fragments.detailed_task.DetailedTaskFragmentArgs
import ru.daotech.agroinvest.fragments.login.LoginFragmentArgs
import ru.daotech.agroinvest.fragments.pages.accident.AccidentReportFragmentArgs
import ru.daotech.agroinvest.fragments.sections.detailed.DetailedSectionFragmentArgs
import ru.daotech.agroinvest.fragments.tasks.myTasks.TasksFragmentArgs
import ru.daotech.agroinvest.fragments.tasks.sectionTasks.SectionTasksFragmentArgs
import ru.daotech.agroinvest.fragments.trolley_tied.TrolleyTiedFragmentArgs
import java.lang.ref.WeakReference
import javax.inject.Inject

const val NFC_CODE_DETAILED_ARGS: String = "nfcCode"
const val NFC_TROLLEY_NUMBER_DETAILED_ARGS: String = "trolleyNumber"

class NavigatorImpl @Inject constructor() : Navigator {
    private lateinit var navController: WeakReference<NavController>

    private val navOptionsWithInclusivePopup =
        NavOptions.Builder()
            .setEnterAnim(R.anim.fade_in)
            .setExitAnim(R.anim.fade_out)
            .setPopEnterAnim(R.anim.fade_in)
            .setPopExitAnim(R.anim.fade_out)
            .setPopUpTo(R.id.nav_graph, true)
            .setLaunchSingleTop(true)
            .build()

    override fun init(navController: NavController) {
        this.navController = WeakReference(navController)
    }

    override fun showTasks() {
        navController.get()?.navigate(R.id.tasksFragment, null, navOptionsWithInclusivePopup)
    }

    override fun showContainerWithDetailedTask(taskId: String) {
        val args = TasksFragmentArgs.Builder(taskId).build().toBundle()
        navController.get()?.navigate(R.id.tasksFragment, args, navOptionsWithInclusivePopup)
    }

    override fun showDetailedTask(taskId: String) {
        val args = DetailedTaskFragmentArgs.Builder(taskId, "").build().toBundle()
        navController.get()?.navigate(R.id.detailedTaskFragment, args)
    }

    override fun showDetailedProcessingTask(taskId: String, nfcValue: String) {
        val args = DetailedTaskFragmentArgs.Builder(taskId, nfcValue).build().toBundle()
        args.putString(NFC_CODE_DETAILED_ARGS, nfcValue)
        navController.get()?.navigate(R.id.detailedTaskFragment, args)
    }

    override fun showTrolleyTied(trolleyNumber: String?) {
        val args = TrolleyTiedFragmentArgs.Builder(trolleyNumber).build().toBundle()
        args.putString(NFC_TROLLEY_NUMBER_DETAILED_ARGS, trolleyNumber)
        navController.get()?.navigate(R.id.trolleyTiedFragment, args)
    }

    override fun showTrolleyTiedFromDetailed(trolleyNumber: String?) {
        val args = TrolleyTiedFragmentArgs.Builder(trolleyNumber).build().toBundle()
        args.putString(NFC_TROLLEY_NUMBER_DETAILED_ARGS, trolleyNumber)
        val currentNavController = navController.get()
        currentNavController!!.navigate(
            R.id.trolleyTiedFragment,
            args,
            navOptions {
                popUpTo = R.id.detailedTaskFragment
                launchSingleTop = false
                anim {
                    enter = R.anim.fade_in
                }
            }
        )
    }

    override fun showDetailedSection(sectionId: String) {
        val args = DetailedSectionFragmentArgs.Builder(sectionId).build().toBundle()
        navController.get()?.navigate(R.id.detailedSectionFragment, args)
    }

    override fun showGroupProperty(taskId: String) {
        val args = DetailedSectionFragmentArgs.Builder(taskId).build().toBundle()
        navController.get()?.navigate(R.id.groupPropertyFragment, args)
    }

    override fun showSectionTasks(sectionId: String) {
        val args = SectionTasksFragmentArgs.Builder(sectionId).build().toBundle()
        navController.get()?.navigate(R.id.sectionTasksFragment, args)
    }

    override fun showAuthorization() {
        navController.get()?.let { navController ->
            val nullString: String? = null
            val args = LoginFragmentArgs.Builder(nullString).build()
            navController.navigate(
                R.id.loginFragment,
                args.toBundle(),
                navOptionsWithInclusivePopup
            )
        }
    }

    override fun backToLoginScreen(barcode: String) {
        val loginArgs = LoginFragmentArgs.Builder(barcode).build()
        navController.get()?.let { navController ->
            navController.popBackStack(R.id.loginFragment, true)
            navController.navigate(R.id.loginFragment, loginArgs.toBundle())
        }
    }

    override fun showBarcodeScanner() {
        navController.get()?.navigate(R.id.barcodeScannerFragment)
    }

    override fun exitApplication() {
        val nullString: String? = null
        val args = LoginFragmentArgs.Builder(nullString).build()
        navController.get()
            ?.navigate(R.id.loginFragment, args.toBundle(), navOptionsWithInclusivePopup)
    }

    override fun showWelcomeScreen() {
        navController.get()?.navigate(
            R.id.welcomeFragment,
            null,
            navOptionsWithInclusivePopup
        )
    }

    override fun showSections() {
        navController.get()?.navigate(R.id.sectionsFragment, null, navOptionsWithInclusivePopup)
    }

    override fun showTaskHistory() {
        navController.get()?.navigate(R.id.tasksHistoryFragment, null, navOptionsWithInclusivePopup)
    }

    override fun showAccidentReport(taskId: String) {
        val args = AccidentReportFragmentArgs.Builder(taskId).build().toBundle()
        val currentNavController = navController.get()
        currentNavController!!.navigate(
            R.id.accidentReportFragment,
            args,
            navOptions {
                popUpTo = R.id.detailedTaskFragment
                launchSingleTop = false
                anim {
                    enter = R.anim.fade_in
                }
            }
        )
    }

    override fun showSplashPage() {
        navController.get()?.navigate(R.id.splashFragment, null)
    }

    override fun showSupportNotify() {
        navController.get()?.navigate(R.id.supportNotifyFragment, null)
    }

    override fun showCollectionHistoryReport() {
        navController.get()?.navigate(R.id.collectionHistoryFragment, null, navOptionsWithInclusivePopup)
    }

    override fun showAccidentReport() {
        navController.get()?.navigate(R.id.accidentReportFragment, null)
    }

    override fun showAboutPage() {
        navController.get()?.navigate(R.id.aboutPageFragment, null, navOptionsWithInclusivePopup)
    }

    override fun showUpdateApp() {
        navController.get()?.navigate(R.id.updateAppFragment, null)
    }

    override fun showRatings() {
        navController.get()?.navigate(R.id.ratingsFragment, null, navOptionsWithInclusivePopup)
    }

    override fun showEarnings() {
        navController.get()?.navigate(R.id.earningsFragment, null, navOptionsWithInclusivePopup)
    }

    override fun showBarcodeBox() {
        navController.get()?.navigate(R.id.barcodeBoxFragment, null)
    }
}
