package ru.daotech.agroinvest.database

import io.reactivex.Completable
import io.reactivex.Flowable
import io.reactivex.Observable
import io.reactivex.Single
import ru.daotech.agroinvest.database.entities.*
import ru.daotech.agroinvest.database.relation.TaskWithPropertiesImpl
import ru.daotech.agroinvest.network.models.VariableRequest
import ru.daotech.agroinvest.service.jsonmodels.SupportMessagePayload

interface DatabaseRepository {

    fun getMyTasksByRowNumber(
        namespace: String,
        workerId: String,
        rowNumber: String
    ): Observable<List<TaskImpl>>

    fun getMyStartedTasks(namespace: String, workerId: String): Observable<List<TaskImpl>>

    fun getMyPausedTasks(namespace: String, workerId: String): Observable<List<TaskImpl>>

    fun getMyTasksFlowable(
        namespace: String,
        workerId: String,
        rowNumber: String? = null
    ): Flowable<List<TaskImpl>>

    fun getTasksHistory(
        namespace: String,
        workerId: String
    ): Observable<List<TaskWithPropertiesImpl>>

    fun getRootSections(namespace: String): Observable<List<SectionImpl>>

    fun getSectionParent(parentId: String?, namespace: String): Observable<List<SectionImpl>>

    fun getSection(sectionId: String): Single<SectionImpl>

    fun getSectionTasks(
        sectionId: String,
        namespace: String
    ): Observable<List<TaskWithPropertiesImpl>>

    fun getTaskWithProperties(taskId: String, sessionId: String): Single<TaskWithPropertiesImpl>

    fun getNotUploadedTaskWithProperties(taskId: String): Observable<List<TaskWithPropertiesImpl>>

    fun getNotUploadedCamundaRequests(): Observable<List<CamundaImpl>>

    fun getNotUploadedCamundaRequestsWOTaskListCount(): Observable<Int>

    fun upsertTask(task: TaskImpl): Completable

    fun upsertTaskPropertiesList(taskPropertyList: List<TaskPropertyImpl>): Completable

    fun upsertTaskIntervalsList(taskIntervalsList: List<TaskIntervalImpl>): Completable

    fun upsertTaskMeasurementsList(taskMeasurementsList: List<TaskMeasurementImpl>): Completable

    fun insertTaskInterval(taskInterval: TaskIntervalImpl): Completable

    fun updateTaskInterval(updatedTaskInterval: TaskIntervalImpl): Completable

    fun upsertTaskSectionsList(taskSectionsList: List<SectionImpl>): Completable

    fun queueCamundaRequest(
        type: String,
        request: VariableRequest,
        namespace: String,
        resourceId: String?
    ): Single<Long>

    fun upsertCamundaRequest(camundaImpl: CamundaImpl): Completable

    fun insertSupportMessage(
        supportMessage: SupportMessagePayload,
        workerId: String,
        time: String,
        messageCode: Int
    ): Completable

    fun getTaskUpdateRequestByTaskId(taskId: String): Observable<List<CamundaImpl>>

    fun getTaskCount(
        namespace: String,
        workerId: String,
        startPeriod: String,
        finishPeriod: String
    ): Observable<Int>

    fun getTaskExistence(namespace: String, taskId: String, workerId: String): Observable<Int>

    fun setTaskWorker(
        namespace: String,
        taskId: String,
        workerId: String?,
        workerName: String?,
        inProgress: Boolean,
        orderBy: Int
    ): Completable

    fun upsertTrolleyInfo(trolleyInfoImpl: TrolleyInfoImpl): Completable

    fun getFullTrolleyInfoHistoryList(
        workerId: String,
        namespace: String
    ): Observable<List<TrolleyInfoImpl>>

    fun getSupportMessageCount(workerId: String): Flowable<Int>

    fun getAllSupportMessagesByWorkerId(workerId: String): Flowable<List<SupportMessageImpl>>

    fun deleteAllMessages(): Completable

    fun updateUnreadStatusSupportMessages(supportMessageList: List<SupportMessageImpl>): Completable

    fun upsertTaskMeasurement(taskMeasurement: TaskMeasurementImpl): Completable

    fun upsertTaskPlantsList(taskPlantList: List<TaskPlantImpl>): Completable

    fun clearTasksData(namespace: String, workerId: String): Completable

    fun getRecentUsers(): Observable<List<RecentUserImpl>>

    fun upsertRecentUser(user: RecentUserImpl)

    fun clearAllCamundaRequests(): Observable<Unit>

    fun clearAllIntervals(): Observable<Unit>

    fun clearAllProperties(): Observable<Unit>

    fun clearAllSections(): Observable<Unit>
}
