package ru.daotech.agroinvest.database.dao

import androidx.room.*
import io.reactivex.Completable
import io.reactivex.Observable
import io.reactivex.Single
import ru.daotech.agroinvest.database.entities.CamundaImpl

@Dao
abstract class CamundaDao : BaseDao<CamundaImpl>() {
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    abstract fun insertCamunda(camunda: CamundaImpl): Single<Long>

    @Query("UPDATE camunda SET responseCode = 200, responseBody = 'DUPLICATE' WHERE id != :id and (type = :type and resourceId = :resourceId or type = 'taskList') and responseCode IS NULL")
    abstract fun removeNotSentDuplicates(
        type: String,
        id: Long,
        resourceId: String?
    ): Completable

    @Query("SELECT * FROM camunda WHERE responseCode IS NULL ORDER BY id")
    abstract fun queryNotSentRequests(): Observable<List<CamundaImpl>>

    @Query("SELECT count(id) FROM camunda WHERE responseCode IS NULL AND type != 'taskList'")
    abstract fun queryNotSentRequestsWOTaskListCount(): Observable<Int>

    @Query("SELECT * FROM camunda WHERE resourceId =:taskId AND (type = 'taskUpdate') AND (responseCode IS NULL) ORDER BY id DESC")
    abstract fun getTaskUpdateRequestByTaskId(taskId: String): Observable<List<CamundaImpl>>

    @Delete
    abstract fun clearAllCamundaRequestData(camundaList: List<CamundaImpl>): Completable

    @Query("SELECT * FROM camunda")
    abstract fun getAllCamundaData(): Observable<List<CamundaImpl>>
}