package ru.daotech.agroinvest.database.dao

import androidx.room.Dao
import androidx.room.Delete
import androidx.room.Query
import io.reactivex.Completable
import io.reactivex.Observable
import io.reactivex.Single
import ru.daotech.agroinvest.database.entities.SectionImpl

@Dao
abstract class SectionDao : BaseDao<SectionImpl>() {
    @Query("SELECT * FROM sections WHERE namespace = :namespace")
    abstract fun querySectionList(namespace: String): Observable<List<SectionImpl>>

    @Query("SELECT * FROM sections WHERE parentId = :parentId AND namespace = :namespace")
    abstract fun querySectionParent(parentId: String?, namespace: String): Observable<List<SectionImpl>>

    @Query("SELECT * FROM sections WHERE parentId IS NULL AND namespace = :namespace")
    abstract fun queryRootSections(namespace: String): Observable<List<SectionImpl>>

    @Query("SELECT * FROM sections WHERE id = :sectionId")
    abstract fun querySection(sectionId: String): Single<SectionImpl>

    @Delete
    abstract fun clearAllSections(sections: List<SectionImpl>): Completable

    @Query("SELECT * FROM sections")
    abstract fun getAllSectionsData(): Observable<List<SectionImpl>>
}