package ru.daotech.agroinvest.database.dao

import androidx.room.Dao
import androidx.room.Query
import io.reactivex.Completable
import io.reactivex.Flowable
import io.reactivex.Observable
import io.reactivex.Single
import ru.daotech.agroinvest.database.entities.TaskImpl
import ru.daotech.agroinvest.database.relation.TaskWithPropertiesImpl

@Dao
abstract class TaskDao : BaseDao<TaskImpl>() {

    @Query("SELECT * FROM tasks WHERE isFinished = 0 AND namespace = :namespace AND workerId = :workerId AND rowNumber = :rowNumber ORDER BY startAt ASC, orderBy ASC, rowNumber ASC")
    abstract fun queryTaskListByRowNumber(
        namespace: String,
        workerId: String,
        rowNumber: String
    ): Observable<List<TaskImpl>>

    @Query("SELECT * FROM tasks WHERE isFinished = 0 AND namespace = :namespace AND workerId = :workerId AND startedAt IS NOT NULL AND finishedAt IS NULL ORDER BY startAt ASC, orderBy ASC, rowNumber ASC")
    abstract fun queryTaskStartedList(
        namespace: String,
        workerId: String
    ): Observable<List<TaskImpl>>

    @Query("SELECT * FROM tasks WHERE isFinished = 0 AND namespace = :namespace AND workerId = :workerId ORDER BY startAt ASC, orderBy ASC, rowNumber ASC")
    abstract fun queryTaskListFlowable(
        namespace: String,
        workerId: String
    ): Flowable<List<TaskImpl>>

    @Query("SELECT * FROM tasks WHERE isFinished = 0 AND namespace = :namespace AND workerId = :workerId AND rowNumber LIKE '%' || :rowNumber || '%' ORDER BY startAt ASC, orderBy ASC, rowNumber ASC")
    abstract fun queryTaskListFilteredFlowable(
        namespace: String,
        workerId: String,
        rowNumber: String?
    ): Flowable<List<TaskImpl>>

    @Query("SELECT * FROM tasks WHERE isFinished = 1 AND namespace = :namespace AND workerId = :workerId ORDER BY finishedAt DESC")
    abstract fun queryTaskHistoryList(
        namespace: String,
        workerId: String
    ): Observable<List<TaskWithPropertiesImpl>>

    @Query("SELECT * FROM tasks WHERE isFinished = 0 AND sectionId = :sectionId AND namespace = :namespace ORDER BY rowNumber ASC")
    abstract fun querySectionTasksList(
        sectionId: String,
        namespace: String
    ): Observable<List<TaskWithPropertiesImpl>>

    @Query("SELECT * FROM tasks WHERE isFinished = 0 AND sectionId = :sectionId ORDER BY rowNumber ASC")
    abstract fun querySectionTasksList(sectionId: String): Observable<List<TaskImpl>>

    @Query("SELECT * FROM tasks WHERE id = :taskId AND namespace = :sessionId")
    abstract fun queryTaskWithProperties(
        taskId: String,
        sessionId: String
    ): Single<TaskWithPropertiesImpl>

    @Query("SELECT count(id) FROM tasks WHERE namespace = :namespace AND id = :taskId AND workerId = :workerId")
    abstract fun queryTaskExistence(
        taskId: String,
        workerId: String,
        namespace: String
    ): Observable<Int>

    @Query("UPDATE tasks SET workerId = :workerId, workerName = :workerName, inProgress = :inProgress, orderBy = :orderBy WHERE id = :taskId AND namespace = :namespace")
    abstract fun setTaskWorker(
        taskId: String,
        namespace: String,
        workerId: String?,
        workerName: String?,
        inProgress: Boolean,
        orderBy: Int
    ): Completable

    @Query("SELECT * FROM tasks WHERE id = :taskId")
    abstract fun queryNotUploadedTaskWithProperties(taskId: String): Observable<List<TaskWithPropertiesImpl>>

    @Query("SELECT count(id) FROM tasks WHERE isFinished = 1 AND namespace = :namespace AND workerId = :workerId AND finishedAt BETWEEN :startPeriod AND :finishPeriod")
    abstract fun queryTaskCount(
        namespace: String,
        workerId: String,
        startPeriod: String,
        finishPeriod: String
    ): Observable<Int>

    @Query("SELECT * FROM tasks WHERE namespace = :namespace AND workerId = :workerId AND isFinished = 0 AND startedAt IS NOT NULL AND finishedAt IS NOT NULL ORDER BY startAt ASC, orderBy ASC, rowNumber ASC")
    abstract fun queryTaskPausedList(
        namespace: String,
        workerId: String
    ): Observable<List<TaskImpl>>

    @Query("DELETE FROM tasks")
    abstract fun clearAllTasksData(): Completable
}