package ru.daotech.agroinvest.database.entities

import androidx.room.ColumnInfo
import androidx.room.Entity
import androidx.room.PrimaryKey
import com.beust.klaxon.Klaxon
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.models.RequestTypeEnum

interface ActionReport {
    val id: Long
    val createdAt: Long
    val updatedAt: Long
    val payload: String
    val actionType: String
    val isSent: Boolean
    val isScanned: Boolean
}

@Entity(tableName = "action_report")
data class ActionReportImpl(
    @PrimaryKey(autoGenerate = true)
    @ColumnInfo(name = "id") override val id: Long = 0,

    @ColumnInfo(name = "createdAt") override val createdAt: Long,

    @ColumnInfo(name = "updatedAt") override val updatedAt: Long,

    @ColumnInfo(name = "payload") override val payload: String,

    @ColumnInfo(name = "actionType") override val actionType: String,

    @ColumnInfo(name = "isSent") override val isSent: Boolean,

    @ColumnInfo(name = "isScanned") override val isScanned: Boolean
) : ActionReport {
    constructor(taskWithProperties: TaskWithProperties?, actionType: RequestTypeEnum) : this(
        createdAt = System.currentTimeMillis() / 1000,
        updatedAt = System.currentTimeMillis() / 1000,
        payload = if (taskWithProperties != null) {
            Klaxon().toJsonString(taskWithProperties)
        } else {
            ""
        },
        actionType = actionType.requestType,
        isSent = false,
        isScanned = actionType == RequestTypeEnum.TASK_SCAN_FINISH
                || actionType == RequestTypeEnum.TASK_SCAN_CANCEL
                || actionType == RequestTypeEnum.TASK_SCAN_START
                || actionType == RequestTypeEnum.TASK_SCAN_LIST
    )
}