package ru.daotech.agroinvest.database.entities

import androidx.room.ColumnInfo
import androidx.room.Entity
import androidx.room.PrimaryKey
import ru.daotech.agroinvest.network.models.CamundaResponse

interface Camunda {
    val id: Long
    val createdAt: Long
    val updatedAt: Long
    val type: String
    val payload: String
    val responseCode: Int?
    val responseBody: String?
    val namespace: String
    val resourceId: String?
}

@Entity(tableName = "camunda")
data class CamundaImpl(
    @PrimaryKey(autoGenerate = true)
    @ColumnInfo(name = "id") override val id: Long = 0,

    @ColumnInfo(name = "createdAt") override val createdAt: Long,

    @ColumnInfo(name = "updatedAt") override val updatedAt: Long,

    @ColumnInfo(name = "type") override val type: String,

    @ColumnInfo(name = "payload") override val payload: String,

    @ColumnInfo(name = "responseCode") override val responseCode: Int?,

    @ColumnInfo(name = "responseBody") override val responseBody: String?,

    @ColumnInfo(name = "namespace") override val namespace: String,

    @ColumnInfo(name = "resourceId") override val resourceId: String?
) : Camunda {
    constructor(type: String, payload: String, namespace: String, resourceId: String?) : this(
        type = type,
        payload = payload,
        namespace = namespace,
        createdAt = System.currentTimeMillis() / 1000,
        updatedAt = System.currentTimeMillis() / 1000,
        responseBody = null,
        responseCode = null,
        resourceId = resourceId
    )

    constructor(from: Camunda, responseCode: Int, responseBody: CamundaResponse?) : this(
        id = from.id,
        type = from.type,
        payload = from.payload,
        namespace = from.namespace,
        createdAt = from.createdAt,
        updatedAt = System.currentTimeMillis() / 1000,
        responseCode = responseCode,
        responseBody = responseBody.toString(),
        resourceId = from.resourceId
    )
}