package ru.daotech.agroinvest.database.entities

import androidx.room.ColumnInfo
import androidx.room.Entity
import androidx.room.ForeignKey
import ru.daotech.agroinvest.service.jsonmodels.TaskMeasurementPayload
import java.util.*

interface TaskMeasurement {
    val id: String
    val taskId: String
    val barcodeValue: String?
    val parentId: String?
    val plantId: String?
    val isMeasured: Boolean
}

@Entity(
    tableName = "task_measurement",
    foreignKeys = [ForeignKey(
        entity = TaskImpl::class,
        parentColumns = ["id"],
        childColumns = ["taskId"],
        onDelete = ForeignKey.CASCADE
    )],
    primaryKeys = ["id", "taskId"]
)
data class TaskMeasurementImpl(
    @ColumnInfo(name = "id") override val id: String,

    @ColumnInfo(name = "taskId") override var taskId: String,

    @ColumnInfo(name = "barcodeValue") override val barcodeValue: String?,

    @ColumnInfo(name = "parentId") override var parentId: String?,

    @ColumnInfo(name = "plantId") override var plantId: String?,

    @ColumnInfo(name = "isMeasured") override var isMeasured: Boolean
) : TaskMeasurement {

    constructor(from: TaskMeasurementPayload, taskId: String) : this(
        id = UUID.randomUUID().toString(),
        barcodeValue = from.barcode,
        parentId = from.parentId!!,
        plantId = from.plantId!!,
        taskId = taskId,
        isMeasured = from.taskId == taskId
    )

    constructor(groupId: String, barcodeValue: String, taskId: String, plantId: String) : this(
        id = UUID.randomUUID().toString(),
        barcodeValue = barcodeValue,
        parentId = groupId,
        taskId = taskId,
        plantId = plantId,
        isMeasured = true
    )
}