package ru.daotech.agroinvest.database.entities

import androidx.room.ColumnInfo
import androidx.room.Entity
import androidx.room.ForeignKey
import ru.daotech.agroinvest.service.jsonmodels.TaskPropertyPayload

interface TaskProperty {
    val id: String
    val taskId: String
    val name: String
    val description: String
    val regexp: String
    val type: String
    val value: String
    val required: Boolean
    val stage: String
    val regex: Regex
    val parentId: String
    val timestamp: String?
    val workerId: String?
    val header: String?
}

@Entity(
    tableName = "task_properties",
    foreignKeys = [ForeignKey(
        entity = TaskImpl::class,
        parentColumns = ["id"],
        childColumns = ["taskId"],
        onDelete = ForeignKey.CASCADE
    )],
    primaryKeys = ["id", "taskId"]
)
data class TaskPropertyImpl(
    @ColumnInfo(name = "id") override val id: String,

    @ColumnInfo(name = "taskId") override val taskId: String,

    @ColumnInfo(name = "name") override val name: String,

    @ColumnInfo(name = "description") override val description: String,

    @ColumnInfo(name = "regexp") override val regexp: String,

    @ColumnInfo(name = "type") override val type: String, // Two variants: number and binary

    @ColumnInfo(name = "value") override val value: String,

    @ColumnInfo(name = "required") override val required: Boolean,

    @ColumnInfo(name = "stage") override val stage: String, // Two variants: start and finish

    @ColumnInfo(name = "parentId") override val parentId: String,

    @ColumnInfo(name = "timestamp") override val timestamp: String?, // Format - yyyy-MM-dd'T'HH-mm-ss

    @ColumnInfo(name = "workerId") override val workerId: String?,

    @ColumnInfo(name = "header") override val header: String?
) : TaskProperty {
    override val regex: Regex
        get() = Regex(regexp)

    constructor(from: TaskPropertyPayload, withTaskId: String) : this(
        id = from.id,
        taskId = withTaskId,
        name = from.name,
        description = from.description,
        regexp = from.regexp,
        type = from.type,
        value = from.value,
        required = from.required,
        stage = from.stage,
        parentId = from.parentId,
        timestamp = from.timestamp,
        workerId = from.workerId,
        header = "" //TODO: REMOVE FIELD!
    )

    constructor(from: TaskPropertyPayload, withTaskId: String, timestamp: String) : this(
        id = from.id,
        taskId = withTaskId,
        name = from.name,
        description = from.description,
        regexp = from.regexp,
        type = from.type,
        value = from.value,
        required = from.required,
        stage = from.stage,
        parentId = from.parentId,
        timestamp = timestamp,
        workerId = from.workerId,
        header = from.timestamp
    )

    constructor(from: TaskProperty, withTaskId: String) : this(
        id = from.id,
        taskId = withTaskId,
        name = from.name,
        description = from.description,
        regexp = from.regexp,
        type = from.type,
        value = from.value,
        required = from.required,
        stage = from.stage,
        parentId = from.parentId,
        timestamp = from.timestamp,
        workerId = from.workerId,
        header = from.header
    )
}