package ru.daotech.agroinvest.database.relation

import android.os.Parcelable
import androidx.room.Embedded
import androidx.room.Relation
import kotlinx.android.parcel.Parcelize
import ru.daotech.agroinvest.database.entities.*
import ru.daotech.agroinvest.models.Department
import ru.daotech.agroinvest.service.jsonmodels.Position
import ru.daotech.agroinvest.service.jsonmodels.SectionPayload

interface TaskWithProperties : Parcelable, Task {
    val properties: List<TaskProperty>
    val intervals: List<TaskInterval>
    val measurements: List<TaskMeasurement>
    val plants: List<TaskPlant>
}

@Parcelize
class TaskWithPropertiesImpl : TaskWithProperties {

    @Embedded
    lateinit var embeddedTask: TaskImpl

    @Relation(parentColumn = "id", entity = TaskPropertyImpl::class, entityColumn = "taskId")
    lateinit var relationProperties: List<TaskPropertyImpl>

    @Relation(parentColumn = "id", entity = TaskIntervalImpl::class, entityColumn = "taskId")
    lateinit var relationIntervals: List<TaskIntervalImpl>

    @Relation(parentColumn = "id", entity = TaskMeasurementImpl::class, entityColumn = "taskId")
    lateinit var relationMeasurements: List<TaskMeasurementImpl>

    @Relation(parentColumn = "id", entity = TaskPlantImpl::class, entityColumn = "taskId")
    lateinit var relationPlants: List<TaskPlantImpl>

    override val id: String
        get() = embeddedTask.id
    override val name: String
        get() = embeddedTask.name
    override val isFinished: Boolean
        get() = embeddedTask.isFinished
    override val orderBy: Int
        get() = embeddedTask.orderBy
    override val color: String
        get() = embeddedTask.color
    override val position: Position?
        get() = embeddedTask.position
    override val department: Department?
        get() = embeddedTask.department
    override val section: SectionPayload?
        get() = embeddedTask.section
    override val description: String
        get() = embeddedTask.description
    override val startAt: String
        get() = embeddedTask.startAt
    override val finishAt: String
        get() = embeddedTask.finishAt
    override val namespace: String
        get() = embeddedTask.namespace
    override val createdAt: String?
        get() = embeddedTask.createdAt
    override val updatedAt: String?
        get() = embeddedTask.updatedAt
    override val requireFinish: Boolean?
        get() = embeddedTask.requireFinish
    override val startedAt: String?
        get() = embeddedTask.startedAt
    override val finishedAt: String?
        get() = embeddedTask.finishedAt
    override val requireOnlineStart: Boolean?
        get() = embeddedTask.requireOnlineStart
    override val parentId: String?
        get() = embeddedTask.parentId
    override val workerId: String?
        get() = embeddedTask.workerId
    override val workerName: String?
        get() = embeddedTask.workerName
    override val rating: Float
        get() = embeddedTask.rating
    override val rowNumber: Int?
        get() = embeddedTask.rowNumber
    override val inProgress: Boolean
        get() = embeddedTask.inProgress
    override val sectionId: String?
        get() = embeddedTask.sectionId
    override val isTaskInfoSent: Boolean
        get() = embeddedTask.isTaskInfoSent
    override val properties: List<TaskProperty>
        get() = relationProperties
    override val intervals: List<TaskInterval>
        get() = relationIntervals
    override val measurements: List<TaskMeasurement>
        get() = relationMeasurements
    override val plants: List<TaskPlant>
        get() = relationPlants
    override val isHarvesting: Boolean?
        get() = embeddedTask.isHarvesting
    override val taskLimit: Int?
        get() = embeddedTask.taskLimit
}
