package ru.daotech.agroinvest.di.modules

import android.content.Context
import android.content.SharedPreferences
import android.preference.PreferenceManager
import dagger.Binds
import dagger.Module
import dagger.Provides
import dagger.android.ContributesAndroidInjector
import dagger.android.support.AndroidSupportInjectionModule
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.business.*
import ru.daotech.agroinvest.components.NavigationFragmentComponentImpl
import ru.daotech.agroinvest.components.TextViewComponentBuilder
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.coordinators.Navigator
import ru.daotech.agroinvest.coordinators.NavigatorImpl
import ru.daotech.agroinvest.dialog.builder.*
import ru.daotech.agroinvest.network.NetworkRepository
import ru.daotech.agroinvest.service.CamundaService
import ru.daotech.agroinvest.service.QueueService
import ru.daotech.agroinvest.service.SocketService
import ru.daotech.agroinvest.service.couch.CouchDbService
import javax.inject.Singleton

@Module(includes = [AndroidSupportInjectionModule::class, AppModule.AppBinds::class])
class AppModule {

    @Module
    internal interface AppBinds {
        @ContainerScope
        @ContributesAndroidInjector(modules = [ContainerModule::class])
        fun containerActivity(): ContainerActivity

        @ContributesAndroidInjector
        fun queueService(): QueueService

        @ContributesAndroidInjector
        fun camundaService(): CamundaService

        @ContributesAndroidInjector
        fun socketService(): SocketService

        @ContributesAndroidInjector
        fun couchDbService(): CouchDbService

        @Singleton
        @Binds
        fun bindNavigator(navigator: NavigatorImpl): Navigator

        @Singleton
        @Binds
        fun bindRepository(repository: RepositoryImpl): Repository

        @Singleton
        @Binds
        fun bindVariableBuilder(repository: VariableRequestBuilderImpl): VariableRequestBuilder
    }

    @Provides
    @Singleton
    fun provideSharedPreferences(context: Context): SharedPreferences {
        return PreferenceManager.getDefaultSharedPreferences(context)
    }

    @Provides
    @Singleton
    fun provideAppStateManager(sharedPreferences: SharedPreferences): AppStateManager {
        return AppStateManager(sharedPreferences)
    }

    @Provides
    @Singleton
    fun provideEventObservableObject(
        appStateManager: AppStateManager,
        networkRepository: NetworkRepository
    ): EventObservableObject {
        return EventObservableObject(
            appStateManager,
            networkRepository
        )
    }


    @Provides
    @Singleton
    fun provideCountlyEventController(appStateManager: AppStateManager): CountlyEventControllerImpl {
        return CountlyEventControllerImpl(appStateManager)
    }

    @Provides
    @Singleton
    fun provideNavigationFragmentComponent(appStateManager: AppStateManager): NavigationFragmentComponentImpl {
        return NavigationFragmentComponentImpl(appStateManager)
    }

    @Provides
    @Singleton
    fun provideTextViewComponentBuilder(): TextViewComponentBuilder {
        return TextViewComponentBuilder()
    }

    @Provides
    @Singleton
    fun provideDetailedDialogBuidler(): DetailedDialogBuilder {
        return DetailedDialogBuilder()
    }

    @Provides
    @Singleton
    fun provideTasksDialogBuidler(): TasksDialogBuilder {
        return TasksDialogBuilder()
    }

    @Provides
    @Singleton
    fun provideTrolleyDialogBuidler(): TrolleyDialogBuilder {
        return TrolleyDialogBuilder()
    }

    @Provides
    @Singleton
    fun provideAboutDialogBuidler(): AboutDialogBuilder {
        return AboutDialogBuilder()
    }

    @Provides
    @Singleton
    fun provideGroupDialogBuilder(): GroupDialogBuilder {
        return GroupDialogBuilder()
    }

    @Provides
    @Singleton
    fun provideAccidentDialogBuidler(): AccidentReportDialogBuilder {
        return AccidentReportDialogBuilder()
    }

    @Provides
    @Singleton
    fun provideContainerDialogBuidler(): ContainerDialogBuilder {
        return ContainerDialogBuilder()
    }

    @Provides
    @Singleton
    fun provideBarcodeBoxDialogBuilder(): BarcodeBoxDialogBuilder {
        return BarcodeBoxDialogBuilder()
    }

    @Provides
    @Singleton
    fun provideHeaderDialogBuilder(): HeaderDialogBuilder {
        return HeaderDialogBuilder()
    }

    @Provides
    @Singleton
    fun provideLoginQueueDialogBuilder(): LoginDialogBuilder {
        return LoginDialogBuilder()
    }
}