package ru.daotech.agroinvest.di.modules

import dagger.Module
import dagger.Provides
import dagger.android.ContributesAndroidInjector
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.business.CountlyEventControllerImpl
import ru.daotech.agroinvest.business.Repository
import ru.daotech.agroinvest.container.ContainerPresenter
import ru.daotech.agroinvest.coordinators.RootCoordinator
import ru.daotech.agroinvest.fragments.barcode.BarcodeScannerFragment
import ru.daotech.agroinvest.fragments.barcode_box.BarcodeBoxFragment
import ru.daotech.agroinvest.fragments.detailed_task.DetailedTaskFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.DetailedTaskProcessingTab
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.choose_group_container.ChooseGroupFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.group_properties_container.GroupPropertyFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.header_field_container.HeaderFieldFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.timer_container.TimerMviFragment
import ru.daotech.agroinvest.fragments.earnings.EarningsFragment
import ru.daotech.agroinvest.fragments.login.LoginFragment
import ru.daotech.agroinvest.fragments.pages.accident.AccidentReportFragment
import ru.daotech.agroinvest.fragments.pages.about.AboutPageFragment
import ru.daotech.agroinvest.fragments.pages.update.UpdateAppFragment
import ru.daotech.agroinvest.fragments.ratings.RatingsFragment
import ru.daotech.agroinvest.fragments.sections.detailed.DetailedSectionFragment
import ru.daotech.agroinvest.fragments.sections.root.SectionsFragment
import ru.daotech.agroinvest.fragments.splash.SplashFragment
import ru.daotech.agroinvest.fragments.support_notification.SupportNotificationFragment
import ru.daotech.agroinvest.fragments.tasks.collectionHistory.CollectionHistoryFragment
import ru.daotech.agroinvest.fragments.tasks.myTasks.TasksFragment
import ru.daotech.agroinvest.fragments.tasks.sectionTasks.SectionTasksFragment
import ru.daotech.agroinvest.fragments.tasks.tasksHistory.TasksHistoryFragment
import ru.daotech.agroinvest.fragments.trolley_tied.TrolleyTiedFragment
import ru.daotech.agroinvest.fragments.welcome.WelcomeFragment
import javax.inject.Scope


@Scope
@Retention(AnnotationRetention.RUNTIME)
annotation class ContainerScope

@Module(includes = [ContainerModule.Fragments::class])
class ContainerModule {

    @Module
    internal interface Fragments {
        @TasksScope
        @ContributesAndroidInjector(modules = [TasksModule::class])
        fun tasksFragment(): TasksFragment

        @TasksHistoryScope
        @ContributesAndroidInjector(modules = [TasksHistoryModule::class])
        fun tasksHistoryFragment(): TasksHistoryFragment

        @SectionsScope
        @ContributesAndroidInjector(modules = [SectionsModule::class])
        fun sectionsFragment(): SectionsFragment

        @DetailedTaskScope
        @ContributesAndroidInjector(modules = [DetailedTaskModule::class])
        fun detailedTaskFragment(): DetailedTaskFragment

        @DetailedSectionScope
        @ContributesAndroidInjector(modules = [DetailedSectionModule::class])
        fun detailedSectionFragment(): DetailedSectionFragment

        @SectionTasksScope
        @ContributesAndroidInjector(modules = [SectionTasksModule::class])
        fun sectionTasksFragment(): SectionTasksFragment

        @LoginScope
        @ContributesAndroidInjector(modules = [LoginModule::class])
        fun loginFragment(): LoginFragment

        @SplashScope
        @ContributesAndroidInjector(modules = [SplashModule::class])
        fun splashFragment(): SplashFragment

        @BarcodeScannerScope
        @ContributesAndroidInjector(modules = [BarcodeScannerModule::class])
        fun barcodeScannerFragment(): BarcodeScannerFragment

        @WelcomeScope
        @ContributesAndroidInjector(modules = [WelcomeModule::class])
        fun welcomeFragment(): WelcomeFragment

        @DetailedTaskProcessingScope
        @ContributesAndroidInjector(modules = [DetailedTaskProcessingModule::class])
        fun detailedTaskProcessingTab(): DetailedTaskProcessingTab

        @AccidentReportScope
        @ContributesAndroidInjector(modules = [AccidentReportModule::class])
        fun accidentReportFragment(): AccidentReportFragment

        @AboutPageScope
        @ContributesAndroidInjector(modules = [AboutPageModule::class])
        fun aboutPageFragment(): AboutPageFragment

        @UpdateAppScope
        @ContributesAndroidInjector(modules = [UpdateAppModule::class])
        fun updateAppFragment(): UpdateAppFragment

        @RatingsScope
        @ContributesAndroidInjector(modules = [RatingsModule::class])
        fun ratingsFragment(): RatingsFragment

        @EarningsScope
        @ContributesAndroidInjector(modules = [EarningsModule::class])
        fun earningsFragment(): EarningsFragment

        @TimerContainerScope
        @ContributesAndroidInjector(modules = [TimerContainerModule::class])
        fun timerMviFragment(): TimerMviFragment

        @TrolleyTiedScope
        @ContributesAndroidInjector(modules = [TrolleyTiedModule::class])
        fun trolleyTiedFragment(): TrolleyTiedFragment

        @CollectionHistoryScope
        @ContributesAndroidInjector(modules = [CollectionHistoryModule::class])
        fun collectionHistoryFragment(): CollectionHistoryFragment

        @HeaderFieldScope
        @ContributesAndroidInjector(modules = [HeaderFieldModule::class])
        fun headerFieldFragment(): HeaderFieldFragment

        @GroupPropertyScope
        @ContributesAndroidInjector(modules = [GroupPropertyModule::class])
        fun groupPropertyFragment(): GroupPropertyFragment

        @BarcodeBoxScope
        @ContributesAndroidInjector(modules = [BarcodeScannerModule::class])
        fun barcodeBoxFragment(): BarcodeBoxFragment

        @ChooseGroupScope
        @ContributesAndroidInjector(modules = [ChooseGroupModule::class])
        fun chooseGroupFragment(): ChooseGroupFragment

        @SupportNotificationScope
        @ContributesAndroidInjector(modules = [SupportNotificationModule::class])
        fun supportNotificationFragment(): SupportNotificationFragment
    }

    @ContainerScope
    @Provides
    fun provideContainerPresenter(
        rootCoordinator: RootCoordinator,
        repository: Repository,
        appStateManager: AppStateManager,
        countlyEventController: CountlyEventControllerImpl
    ): ContainerPresenter {
        return ContainerPresenter(
            goToMyTasks = rootCoordinator::goToMyTask,
            goToTaskHistory = rootCoordinator::goToTaskHistory,
            goToCollectionHistory = rootCoordinator::goToCollectionHistory,
            goToAccidentReport = rootCoordinator::goToAccidentReport,
            goToAboutPage = rootCoordinator::goToAboutPage,
            goToUpdateApp = rootCoordinator::goToUpdateApp,
            goToRatings = rootCoordinator::goToRatings,
            goToEarnings = rootCoordinator::goToEarnings,
            goToSections = rootCoordinator::goToSection,
            goToSupportNotify = rootCoordinator::goToSupportNotify,
            exitApplication = rootCoordinator::exitApplication,
            repository = repository,
            appStateManager = appStateManager,
            countlyEventController = countlyEventController
        )
    }
}