package ru.daotech.agroinvest.di.modules

import android.content.Context
import androidx.room.Room
import androidx.room.migration.Migration
import androidx.sqlite.db.SupportSQLiteDatabase
import dagger.Module
import dagger.Provides
import dagger.android.support.AndroidSupportInjectionModule
import ru.daotech.agroinvest.database.AppDatabase
import ru.daotech.agroinvest.database.DatabaseRepository
import ru.daotech.agroinvest.database.DatabaseRepositoryImpl
import javax.inject.Singleton

const val DATABASE_NAME = "Agroinvest"

@Module(includes = [AndroidSupportInjectionModule::class])
class DatabaseModule {

    @Provides
    @Singleton
    fun provideDatabase(context: Context): AppDatabase {
        val migrationFrom1To2 = object : Migration(1, 2) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("CREATE TABLE IF NOT EXISTS `camunda` (`id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, `createdAt` INTEGER NOT NULL, `updatedAt` INTEGER NOT NULL, `type` TEXT NOT NULL, `payload` TEXT NOT NULL, `responseCode` INTEGER, `responseBody` TEXT, `namespace` TEXT NOT NULL)")
            }
        }

        val migrationFrom2To3 = object : Migration(2, 3) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("UPDATE tasks SET isTaskInfoSent = 1 WHERE isTaskInfoSent = 0")
            }
        }

        val migrationFrom3To4 = object : Migration(3, 4) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("ALTER TABLE camunda ADD COLUMN resourceId TEXT")
            }
        }

        val migrationFrom4To5 = object : Migration(4, 5) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("DELETE FROM tasks")
                database.execSQL("DELETE FROM sections")
                database.execSQL("DELETE FROM task_properties")
                database.execSQL("DELETE FROM task_intervals")
                database.execSQL("DELETE FROM camunda")
            }
        }

        val migrationFrom5To6 = object : Migration(5, 6) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("ALTER TABLE tasks ADD COLUMN workerName TEXT")
            }
        }

        val migrationFrom6To7 = object : Migration(6, 7) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("DELETE FROM tasks")
                database.execSQL("DELETE FROM sections")
                database.execSQL("DELETE FROM task_properties")
                database.execSQL("DELETE FROM task_intervals")
                database.execSQL("DELETE FROM camunda")
            }
        }

        val migrationFrom7To8 = object : Migration(7, 8) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("DELETE FROM tasks")
                database.execSQL("DELETE FROM sections")
                database.execSQL("DELETE FROM task_properties")
                database.execSQL("DELETE FROM task_intervals")
                database.execSQL("DELETE FROM camunda")

                database.execSQL("ALTER TABLE tasks ADD COLUMN inProgress INTEGER NOT NULL DEFAULT 0")
            }
        }

        val migrationFrom8To9 = object : Migration(8, 9) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("CREATE TABLE IF NOT EXISTS `action_report` (`id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, `createdAt` INTEGER NOT NULL, `updatedAt` INTEGER NOT NULL, `payload` TEXT NOT NULL, `actionType` TEXT NOT NULL, `isSent` INTEGER NOT NULL, `isScanned` INTEGER NOT NULL)")
            }
        }

        val migrationFrom9To10 = object : Migration(9, 10) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("ALTER TABLE task_properties ADD COLUMN parentId TEXT NOT NULL DEFAULT '' ")
            }
        }

        val migrationFrom10To11 = object : Migration(10, 11) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("ALTER TABLE task_properties ADD COLUMN timestamp TEXT")
            }
        }

        val migrationFrom11To12 = object : Migration(11, 12) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("ALTER TABLE task_properties ADD COLUMN workerId TEXT")
            }
        }

        val migrationFrom12To13 = object : Migration(12, 13) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("CREATE TABLE IF NOT EXISTS `trolley` (`id` TEXT PRIMARY KEY NOT NULL, `trolley` INTEGER NOT NULL, `registeredAt` TEXT NOT NULL, `workerId` TEXT NOT NULL, `type` TEXT NOT NULL, `namespace` TEXT NOT NULL)")
            }
        }

        val migrationFrom13To14 = object : Migration(13, 14) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("DELETE FROM tasks")
                database.execSQL("DELETE FROM sections")
                database.execSQL("DELETE FROM task_properties")
                database.execSQL("DELETE FROM task_intervals")
                database.execSQL("DELETE FROM camunda")
                database.execSQL("DELETE FROM action_report")
                database.execSQL("DELETE FROM trolley")
            }
        }

        val migrationFrom14To15 = object : Migration(14, 15) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("ALTER TABLE task_properties ADD COLUMN header TEXT")
            }
        }

        val migrationFrom15To16 = object : Migration(15, 16) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("CREATE TABLE IF NOT EXISTS `task_measurement` (`id` TEXT PRIMARY KEY NOT NULL, `barcodeValue` TEXT, `parentId` TEXT, `taskId` TEXT NOT NULL, FOREIGN KEY (taskId) REFERENCES tasks(id) ON UPDATE NO ACTION ON DELETE CASCADE)")
            }
        }

        val migrationFrom16To17 = object : Migration(16, 17) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("CREATE TABLE IF NOT EXISTS `support_message` (`id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, `workerId` TEXT NOT NULL, `message` TEXT NOT NULL, `time` TEXT NOT NULL, `isRead` INTEGER DEFAULT 0 NOT NULL)")
            }
        }

        val migrationFrom17To18 = object : Migration(17, 18) {
            override fun migrate(database: SupportSQLiteDatabase) {
            }
        }

        val migrationFrom18To19 = object : Migration(18, 19) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("ALTER TABLE tasks ADD COLUMN isHarvesting INTEGER")
            }
        }

        val migrationFrom19To20 = object : Migration(19, 20) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("DELETE FROM tasks")
                database.execSQL("DELETE FROM sections")
                database.execSQL("DELETE FROM task_properties")
                database.execSQL("DELETE FROM task_intervals")
                database.execSQL("DELETE FROM camunda")
                database.execSQL("DELETE FROM action_report")
                database.execSQL("DELETE FROM trolley")
                database.execSQL("DROP TABLE IF EXISTS task_measurement")
                database.execSQL("DROP TABLE IF EXISTS support_message")
                database.execSQL("CREATE TABLE IF NOT EXISTS `task_measurement` (`id` TEXT NOT NULL, `barcodeValue` TEXT, `parentId` TEXT, `taskId` TEXT NOT NULL, PRIMARY KEY (`id`, `taskId`), FOREIGN KEY (taskId) REFERENCES tasks(id) ON UPDATE NO ACTION ON DELETE CASCADE)")
                database.execSQL("CREATE TABLE IF NOT EXISTS `support_message` (`id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, `workerId` TEXT NOT NULL, `message` TEXT NOT NULL, `time` TEXT NOT NULL, `isRead` INTEGER DEFAULT 0 NOT NULL)")
                database.execSQL("ALTER TABLE `support_message` ADD COLUMN `messageCode` INTEGER NOT NULL DEFAULT 0")
            }
        }

        val migrationFrom20To21 = object : Migration(20, 21) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("CREATE TABLE IF NOT EXISTS `task_plant` (`id` TEXT NOT NULL, `name` TEXT NOT NULL, `taskId` TEXT NOT NULL, PRIMARY KEY (`id`, `taskId`), FOREIGN KEY (taskId) REFERENCES tasks(id) ON UPDATE NO ACTION ON DELETE CASCADE)")
            }
        }

        val migrationFrom21To22 = object : Migration(21, 22) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("ALTER TABLE `task_measurement` ADD COLUMN `plantId` TEXT")
            }
        }

        val migrationFrom22To23 = object : Migration(22, 23) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("ALTER TABLE `task_measurement` ADD COLUMN `isMeasured` INTEGER NOT NULL DEFAULT 0")
            }
        }

        val migrationFrom23To24 = object : Migration(23, 24) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("ALTER TABLE `tasks` ADD COLUMN `taskLimit` INTEGER")
            }
        }

        val migrationFrom24To25 = object : Migration(24, 25) {
            override fun migrate(database: SupportSQLiteDatabase) {
                database.execSQL("CREATE TABLE IF NOT EXISTS `recent_users` (`id` TEXT PRIMARY KEY NOT NULL, `date` INTEGER NOT NULL, `workerName` TEXT NOT NULL, `personalNumber` TEXT NOT NULL)")
            }
        }

        return Room.databaseBuilder(
            context,
            AppDatabase::class.java,
            DATABASE_NAME
        )
            .addMigrations(migrationFrom1To2)
            .addMigrations(migrationFrom2To3)
            .addMigrations(migrationFrom3To4)
            .addMigrations(migrationFrom4To5)
            .addMigrations(migrationFrom5To6)
            .addMigrations(migrationFrom6To7)
            .addMigrations(migrationFrom7To8)
            .addMigrations(migrationFrom8To9)
            .addMigrations(migrationFrom9To10)
            .addMigrations(migrationFrom10To11)
            .addMigrations(migrationFrom11To12)
            .addMigrations(migrationFrom12To13)
            .addMigrations(migrationFrom13To14)
            .addMigrations(migrationFrom14To15)
            .addMigrations(migrationFrom15To16)
            .addMigrations(migrationFrom16To17)
            .addMigrations(migrationFrom17To18)
            .addMigrations(migrationFrom18To19)
            .addMigrations(migrationFrom19To20)
            .addMigrations(migrationFrom20To21)
            .addMigrations(migrationFrom21To22)
            .addMigrations(migrationFrom22To23)
            .addMigrations(migrationFrom23To24)
            .addMigrations(migrationFrom24To25)
            .allowMainThreadQueries()
            .build()
    }

    @Singleton
    @Provides
    fun bindDatabaseRepository(database: AppDatabase): DatabaseRepository {
        return DatabaseRepositoryImpl(database)
    }
}
