package ru.daotech.agroinvest.di.modules

import dagger.Module
import dagger.Provides
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.business.CountlyEventControllerImpl
import ru.daotech.agroinvest.business.Repository
import ru.daotech.agroinvest.coordinators.DetaledTaskAccidentCoordinator
import ru.daotech.agroinvest.coordinators.RootCoordinator
import ru.daotech.agroinvest.coordinators.TrolleyTiedCoordinator
import ru.daotech.agroinvest.database.DatabaseRepository
import ru.daotech.agroinvest.fragments.detailed_task.DetailedTaskPresenter
import javax.inject.Scope

@Scope
@Retention(AnnotationRetention.RUNTIME)
annotation class DetailedTaskScope

@Module
class DetailedTaskModule {
    @DetailedTaskScope
    @Provides
    fun provideDetailedTaskPresenter(
        rootCoordinator: RootCoordinator,
        detailedTasksCoordinator: DetaledTaskAccidentCoordinator,
        repository: Repository,
        appStateManager: AppStateManager,
        databaseRepository: DatabaseRepository,
        trolleyTiedCoordinator: TrolleyTiedCoordinator,
        countlyEventController: CountlyEventControllerImpl
    ): DetailedTaskPresenter {
        return DetailedTaskPresenter(
            goToMyTasks = rootCoordinator::goToMyTask,
            goToBarcodeScan = rootCoordinator::goToBarcodeBox,
            repository = repository,
            appStateManager = appStateManager,
            goToAccidentReport = detailedTasksCoordinator::showAccidentReport,
            databaseRepository = databaseRepository,
            goToTrolleyTied = trolleyTiedCoordinator::showTrolleyTiedFromDetailed,
            goToGroupProperty = rootCoordinator::showGroupProperty,
            countlyEventController = countlyEventController
        )
    }
}