package ru.daotech.agroinvest.dialog

import android.content.Context
import android.net.Uri
import android.widget.ListView
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.FragmentActivity
import androidx.fragment.app.FragmentManager
import io.reactivex.subjects.BehaviorSubject
import io.reactivex.subjects.PublishSubject
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.models.Worker
import ru.daotech.agroinvest.service.jsonmodels.PlantPayload
import ru.daotech.agroinvest.service.jsonmodels.TaskPropertyPayload

interface DialogBuilderView {

    interface DetailedDialogView {

        fun buildTaskInfoDialog(
            context: Context,
            taskWithProperties: TaskWithProperties
        ): AlertDialog

        fun buildTaskFinishAttentionDialog(
            context: Context,
            showFinishedProgressbar: () -> Unit,
            hideKeyboard: () -> Unit,
            getCurrentNfcMark: () -> String?,
            checkFragmentInitiateBuilding: () -> Unit
        ): AlertDialog

        fun buildTaskCancelAttentionDialog(
            onTaskCancelAccept: PublishSubject<Unit>,
            context: Context,
            showCanceledProgressbar: () -> Unit
        ): AlertDialog

        fun buildTaskStartAttentionDialog(
            getCurrentNfcMark: () -> String?,
            removeNfcCodeFromArgs: () -> Unit,
            onTaskStartClickSubject: PublishSubject<String>,
            context: Context
        ): AlertDialog

        fun showResultDialog(isResultSuccess: Boolean)

        fun buildResponseAttentionDialog(
            context: Context,
            dismissResponseDialog: PublishSubject<Unit>
        ): AlertDialog
    }

    interface TasksDialogView {

        fun buildTaskAlreadyInProgressAttentionDialog(
            context: Context,
            dismissDialogSubject: PublishSubject<Unit>,
            goToDetailedTaskSubject: PublishSubject<String>,
            getCurrentRowNumber: () -> String
        ): AlertDialog
    }

    interface TrolleyDialogView {

        fun buildTrolleyAttentionDialog(
            context: Context,
            dialogMessage: String,
            trolleyNumber: String,
            isTakeNewTrolley: Boolean,
            onTrolleyTakeClick: PublishSubject<String>,
            onTrolleyUntiedClick: PublishSubject<String>,
            errorMessageSubject: PublishSubject<String>,
            appStateManager: AppStateManager
        ): AlertDialog
    }

    interface AboutDialogView {

        fun buildResponseAttentionDialog(context: Context): AlertDialog

        fun showResultDialog(isResultSuccess: Boolean)
    }

    interface AccidentReportDialogView {

        fun showResultDialog(resultState: Int)

        fun buildAccidentResponseDialog(context: Context): AlertDialog

        fun buildAccidentRequestDialog(
            context: Context,
            onSendResultReport: PublishSubject<String>,
            onSendResultReportWithTaskId: PublishSubject<Pair<String, String>>,
            taskWithProperties: TaskWithProperties?,
            currentWorkerSimple: Worker?
        ): AlertDialog
    }

    interface ContainerDialogView {

        fun buildFinishDayDialog(
            context: Context,
            onSignOutAccept: PublishSubject<Unit>
        ): AlertDialog
    }

    interface BarcodeBoxDialogView {

        fun buildBarcodeBoxAttentionDialog(
            context: Context,
            onBarcodeBoxValueSubjectInvoke: () -> Unit,
            progressbarDismiss: () -> Unit,
            buildBarcodeScanCallback: () -> Unit,
            barcodeValue: String
        ): AlertDialog

        fun buildResponseDialog(
            context: Context,
            fragmentManager: FragmentManager,
            fragmentActivity: FragmentActivity
        )

        fun buildAndShowErrorBarcodeScanDialog(
            context: Context,
            fragmentManager: FragmentManager,
            activity: FragmentActivity,
            message: String
        )
    }

    interface HeaderDialogView {

        fun buildAndShowErrorDialog(
            context: Context,
            message: String
        )

        fun buildAndShowAttentionDialog(
            propertyList: List<Any>,
            signalToSubmitForm: () -> Unit,
            plant: PlantPayload,
            context: Context
        ): AlertDialog
    }

    interface GroupDialogView {

        fun buildBottomSheetAttentionDialog(
            requestFocus: () -> Unit,
            showButtonProgressbar: () -> Unit,
            signalToFormSubmit: () -> Unit,
            retrieveSubject: BehaviorSubject<Pair<List<TaskPropertyPayload?>, List<Uri>>>?,
            context: Context
        ): AlertDialog
    }

    interface LoginDialogView {

        fun buildLoginQueueDialog(
            context: Context,
            remoteMessageQueue: String?
        )

        fun hideLoginQueueDialog()
    }
}