package ru.daotech.agroinvest.dialog.builder

import android.content.Context
import android.graphics.Color
import android.graphics.drawable.AnimatedVectorDrawable
import android.graphics.drawable.ColorDrawable
import android.graphics.drawable.Drawable
import android.view.LayoutInflater
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.vectordrawable.graphics.drawable.AnimatedVectorDrawableCompat
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.dialog.BaseDialogView
import ru.daotech.agroinvest.dialog.DialogBuilderView

open class AboutDialogBuilder : DialogBuilderView.AboutDialogView {

    lateinit var aboutDialogViewModel: AboutDialogViewModel

    override fun buildResponseAttentionDialog(context: Context): AlertDialog {
        val factory = LayoutInflater.from(context)
        val responseDialogView = factory
            .inflate(R.layout.dialog_success_motion, null)
        val responseDialog = AlertDialog.Builder(context).create()
        responseDialog.setView(responseDialogView)
        responseDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        responseDialogView.findViewById<Button>(R.id.result_export_button)
            ?.setOnClickListener { responseDialog.dismiss() }
        val doneIcon =
            responseDialogView.findViewById<ImageView>(R.id.done_container_view)
        val failedIcon =
            responseDialogView.findViewById<ImageView>(R.id.failed_container_view)

        // Build view current model state
        aboutDialogViewModel = AboutDialogViewModel(
            doneIcon,
            failedIcon,
            responseDialogView,
            responseDialog,
            context
        )

        return responseDialog
    }

    override fun showResultDialog(isResultSuccess: Boolean) {
        if (isResultSuccess) {
            aboutDialogViewModel.doneIcon.visibility = View.VISIBLE
            aboutDialogViewModel.failedIcon.visibility = View.INVISIBLE
            aboutDialogViewModel.dialogView
                .findViewById<TextView>(R.id.result_export_text_view)
                .text =
                aboutDialogViewModel.context.getString(R.string.success_export_result_text_view)

                val doneDrawable = aboutDialogViewModel.doneIcon.drawable
                drawAnimation(doneDrawable)
        } else {
            aboutDialogViewModel.doneIcon.visibility = View.INVISIBLE
            aboutDialogViewModel.failedIcon.visibility = View.VISIBLE
            aboutDialogViewModel.dialogView
                .findViewById<TextView>(R.id.result_export_text_view)
                .text =
                aboutDialogViewModel.context.getString(R.string.failed_export_result_text_view)

                val doneDrawable = aboutDialogViewModel.failedIcon.drawable
                drawAnimation(doneDrawable)
        }
        aboutDialogViewModel.dialog.show()
    }

    private fun drawAnimation(doneDrawable: Drawable) {
        if (doneDrawable is AnimatedVectorDrawableCompat) {
            doneDrawable.start()
        } else if (doneDrawable is AnimatedVectorDrawable) {
            doneDrawable.start()
        }
    }
}

data class AboutDialogViewModel(
    val doneIcon: ImageView,
    val failedIcon: ImageView,
    override val dialogView: View,
    override val dialog: AlertDialog,
    override val context: Context
) : BaseDialogView()
