package ru.daotech.agroinvest.dialog.builder

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.AnimatedVectorDrawable
import android.graphics.drawable.ColorDrawable
import android.graphics.drawable.Drawable
import android.view.LayoutInflater
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.vectordrawable.graphics.drawable.AnimatedVectorDrawableCompat
import com.jakewharton.rxbinding2.view.clicks
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.dialog_fragment_accident_report.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.dialog.BaseDialogView
import ru.daotech.agroinvest.dialog.DialogBuilderView
import ru.daotech.agroinvest.models.Worker
import ru.daotech.agroinvest.utils.ACCIDENT_FAILED_RESULT_STATE
import ru.daotech.agroinvest.utils.ACCIDENT_SEND_CONTINUATION_RESULT_STATE
import ru.daotech.agroinvest.utils.ACCIDENT_SUCCESS_RESULT_STATE

open class AccidentReportDialogBuilder : DialogBuilderView.AccidentReportDialogView {

    private lateinit var accidentReportDialogViewModel: AccidentReportDialogViewModel

    @SuppressLint("InflateParams")
    override fun buildAccidentResponseDialog(context: Context): AlertDialog {
        val factory = LayoutInflater.from(context)
        val taskAccidentAttentionDialogView = factory
            .inflate(R.layout.dialog_success_motion, null)
        val resultAttentionDialog = AlertDialog.Builder(context).create()
        resultAttentionDialog.setView(taskAccidentAttentionDialogView)
        resultAttentionDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        taskAccidentAttentionDialogView.findViewById<Button>(R.id.result_export_button)
            ?.setOnClickListener { resultAttentionDialog.dismiss() }
        val doneIcon =
            taskAccidentAttentionDialogView.findViewById<ImageView>(R.id.done_container_view)
        val failedIcon =
            taskAccidentAttentionDialogView.findViewById<ImageView>(R.id.failed_container_view)
        val sendContinuationIcon =
            taskAccidentAttentionDialogView.findViewById<ImageView>(R.id.send_continuation_container_view)

        accidentReportDialogViewModel = AccidentReportDialogViewModel(
            doneIcon,
            failedIcon,
            sendContinuationIcon,
            taskAccidentAttentionDialogView,
            resultAttentionDialog,
            context
        )

        return resultAttentionDialog
    }

    @SuppressLint("InflateParams")
    override fun buildAccidentRequestDialog(
        context: Context,
        onSendResultReport: PublishSubject<String>,
        onSendResultReportWithTaskId: PublishSubject<Pair<String, String>>,
        taskWithProperties: TaskWithProperties?,
        currentWorkerSimple: Worker?
    ): AlertDialog {
        val factory = LayoutInflater.from(context)
        val accidentReportDialogView = factory
            .inflate(R.layout.dialog_fragment_accident_report, null)
        val accidentReportDialog = AlertDialog.Builder(context).create()
        accidentReportDialog.setView(accidentReportDialogView)
        accidentReportDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        accidentReportDialogView.findViewById<Button>(R.id.task_accident_report_from_dialog_send)
            ?.setOnClickListener { accidentReportDialog.dismiss() }
        taskWithProperties?.let { task ->
            with(accidentReportDialogView) {
                task_accident_task_report_container.visibility = View.VISIBLE
                task_accident_section.text = task.section?.name
                task_accident_department.text = task.department?.name
                if (task.rowNumber != null) {
                    task_accident_row_number.text = task.rowNumber?.toString()
                } else {
                    task_accident_row_number.visibility = View.GONE
                }
                if (currentWorkerSimple != null) {
                    task_assigned_for.text = currentWorkerSimple.name
                } else {
                    task_assigned_for.visibility = View.GONE
                }
            }
        }

        accidentReportDialogView.findViewById<Button>(R.id.task_accident_report_from_dialog_send)
            .clicks()
            .map {
                accidentReportDialogView.task_accident_report_text_area.apply {
                    val textReport = this.text
                    if (textReport.isNullOrBlank()) {
                        this.error = "Поле не может быть пустым"
                        return@map
                    }
                    val reportMessage = textReport.toString()
                    if (taskWithProperties != null) {
                        onSendResultReportWithTaskId.onNext(
                            Pair(
                                taskWithProperties.id,
                                reportMessage
                            )
                        )
                    } else {
                        onSendResultReport.onNext(reportMessage)
                    }
                    accidentReportDialog.dismiss()
                }
            }
            .subscribe()

        return accidentReportDialog
    }

    override fun showResultDialog(resultState: Int) {
        when (resultState) {
            ACCIDENT_SUCCESS_RESULT_STATE -> {
                accidentReportDialogViewModel.doneIcon.visibility = View.VISIBLE
                accidentReportDialogViewModel.failedIcon.visibility = View.INVISIBLE
                accidentReportDialogViewModel.sendContinuationIcon.visibility = View.INVISIBLE
                accidentReportDialogViewModel.dialogView
                    .findViewById<TextView>(R.id.result_export_text_view)
                    .text =
                    accidentReportDialogViewModel.context.getString(R.string.success_export_result_text_view)

                val doneDrawable = accidentReportDialogViewModel.doneIcon.drawable
                drawAnimation(doneDrawable)
            }
            ACCIDENT_FAILED_RESULT_STATE -> {
                accidentReportDialogViewModel.doneIcon.visibility = View.INVISIBLE
                accidentReportDialogViewModel.failedIcon.visibility = View.VISIBLE
                accidentReportDialogViewModel.sendContinuationIcon.visibility = View.INVISIBLE
                accidentReportDialogViewModel.dialogView
                    .findViewById<TextView>(R.id.result_export_text_view)
                    .text =
                    accidentReportDialogViewModel.context.getString(R.string.failed_export_result_text_view)

                val doneDrawable = accidentReportDialogViewModel.failedIcon.drawable
                drawAnimation(doneDrawable)
            }
            ACCIDENT_SEND_CONTINUATION_RESULT_STATE -> {
                accidentReportDialogViewModel.doneIcon.visibility = View.INVISIBLE
                accidentReportDialogViewModel.failedIcon.visibility = View.INVISIBLE
                accidentReportDialogViewModel.sendContinuationIcon.visibility = View.VISIBLE
                accidentReportDialogViewModel.dialogView
                    .findViewById<TextView>(R.id.result_export_text_view)
                val sendContinuationDrawable =
                    accidentReportDialogViewModel.sendContinuationIcon.drawable
                drawAnimation(sendContinuationDrawable)
            }
        }
        accidentReportDialogViewModel.dialog.show()
    }

    private fun drawAnimation(doneDrawable: Drawable) {
        if (doneDrawable is AnimatedVectorDrawableCompat) {
            doneDrawable.start()
        } else if (doneDrawable is AnimatedVectorDrawable) {
            doneDrawable.start()
        }
    }
}

data class AccidentReportDialogViewModel(
    val doneIcon: ImageView,
    val failedIcon: ImageView,
    val sendContinuationIcon: ImageView,
    override val dialogView: View,
    override val dialog: AlertDialog,
    override val context: Context
) : BaseDialogView()