package ru.daotech.agroinvest.dialog.builder

import android.app.Dialog
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.View.VISIBLE
import android.view.ViewGroup
import android.view.Window
import android.view.animation.AnimationUtils
import android.widget.Button
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment
import androidx.fragment.app.FragmentActivity
import androidx.fragment.app.FragmentManager
import io.reactivex.Observable
import kotlinx.android.synthetic.main.dialog_success_barcode_retrived.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.dialog.DialogBuilderView
import ru.daotech.agroinvest.utils.BARCODE_DIALOG_FRAGMENT_TAG
import java.util.*
import java.util.concurrent.TimeUnit

open class BarcodeBoxDialogBuilder : DialogBuilderView.BarcodeBoxDialogView {

    override fun buildBarcodeBoxAttentionDialog(
        context: Context,
        onBarcodeBoxValueSubjectInvoke: () -> Unit,
        progressbarDismiss: () -> Unit,
        buildBarcodeScanCallback: () -> Unit,
        barcodeValue: String
    ): AlertDialog {

        var isValidBarcode = false
        try {
            UUID.fromString(barcodeValue)
            isValidBarcode = true
        } catch (exception: IllegalArgumentException) {
            //handle the case where string is not valid UUID
        }

        val factory = LayoutInflater.from(context)
        val borderBoxResponseAttentionDialogView =
            factory.inflate(R.layout.base_dialog_fragment, null)
        val borderBoxResponseAttentionDialog: AlertDialog = AlertDialog.Builder(context).create()
        borderBoxResponseAttentionDialog.setView(borderBoxResponseAttentionDialogView)
        borderBoxResponseAttentionDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        borderBoxResponseAttentionDialogView.findViewById<Button>(R.id.dismiss_base_dialog_button)
            ?.setOnClickListener {
                progressbarDismiss.invoke()
                buildBarcodeScanCallback.invoke()
                borderBoxResponseAttentionDialog.dismiss()
            }
        borderBoxResponseAttentionDialogView.findViewById<Button>(R.id.confirm_base_dialog_button)
            ?.setOnClickListener {
                borderBoxResponseAttentionDialog.dismiss()
                onBarcodeBoxValueSubjectInvoke()
            }
        borderBoxResponseAttentionDialogView.findViewById<Button>(R.id.confirm_base_dialog_button)
            ?.isEnabled = isValidBarcode
        borderBoxResponseAttentionDialogView.findViewById<Button>(R.id.confirm_base_dialog_button)
            .background.alpha = if(isValidBarcode) 255 else 125

        borderBoxResponseAttentionDialogView.findViewById<TextView>(
            R.id.base_dialog_barcode_number
        )?.text = if(isValidBarcode) barcodeValue else "NON-VALID [$barcodeValue]"
        borderBoxResponseAttentionDialog.show()

        return borderBoxResponseAttentionDialog
    }

    override fun buildResponseDialog(
        context: Context,
        fragmentManager: FragmentManager,
        fragmentActivity: FragmentActivity
    ) {
        val fragmentTransaction = fragmentManager.beginTransaction()
        val barcodeDialogFragment =
            BarcodeDialogFragment.newInstance(context, fragmentActivity)
        fragmentTransaction.setCustomAnimations(
            R.anim.enter_from_right,
            R.anim.exit_to_left,
            R.anim.enter_from_left,
            R.anim.exit_to_right
        )
        barcodeDialogFragment.show(fragmentTransaction, BARCODE_DIALOG_FRAGMENT_TAG)
    }

    override fun buildAndShowErrorBarcodeScanDialog(
        context: Context,
        fragmentManager: FragmentManager,
        activity: FragmentActivity,
        message: String
    ) {
        val factory = LayoutInflater.from(context)
        val accidentReportDialogView = factory
            .inflate(R.layout.dialog_success_motion, null)
        val accidentReportDialog = AlertDialog.Builder(context).create()
        accidentReportDialog.setView(accidentReportDialogView)
        accidentReportDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        accidentReportDialogView.findViewById<Button>(R.id.result_export_button)
            ?.setOnClickListener { accidentReportDialog.dismiss() }
        accidentReportDialogView.findViewById<TextView>(R.id.result_export_text_view)
            .text = message
        accidentReportDialogView.findViewById<ImageView>(R.id.attention_container_view)
            .visibility = VISIBLE
        accidentReportDialog.show()
    }

    class BarcodeDialogFragment(
        private val contextParent: Context,
        private val fragmentActivity: FragmentActivity
    ) : DialogFragment() {

        companion object {
            fun newInstance(
                context: Context,
                fragmentActivity: FragmentActivity
            ) = BarcodeDialogFragment(context, fragmentActivity)
        }

        override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
            val view: View = fragmentActivity.layoutInflater.inflate(
                R.layout.dialog_success_barcode_retrived,
                LinearLayout(fragmentActivity),
                false
            )

            // Build dialog
            val builder = Dialog(contextParent)
            builder.requestWindowFeature(Window.FEATURE_NO_TITLE)
            builder.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
            builder.setContentView(view)

            view.result_barcode_box_dialog_button.setOnClickListener {
                Observable.just(startTransitionToRight())
                    .delay(300, TimeUnit.MILLISECONDS)
                    .map { dismiss() }
                    .subscribe()
            }

            return builder
        }

        override fun onStart() {
            super.onStart()
            val dialog = dialog
            if (dialog != null) {
                dialog.window!!.setLayout(
                    ViewGroup.LayoutParams.MATCH_PARENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT
                )
                dialog.window!!.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
                startTransitionFromLeft()
            }
        }

        private fun startTransitionFromLeft() {
            (dialog?.window?.decorView as ViewGroup)
                .getChildAt(0).startAnimation(
                    AnimationUtils.loadAnimation(
                        context, android.R.anim.slide_in_left
                    )
                )
        }

        private fun startTransitionToRight() {
            (dialog?.window?.decorView as ViewGroup)
                .getChildAt(0).startAnimation(
                    AnimationUtils.loadAnimation(
                        context, android.R.anim.slide_out_right
                    )
                )
        }
    }
}