package ru.daotech.agroinvest.dialog.builder

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.AnimatedVectorDrawable
import android.graphics.drawable.ColorDrawable
import android.graphics.drawable.Drawable
import android.view.LayoutInflater
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.vectordrawable.graphics.drawable.AnimatedVectorDrawableCompat
import com.jakewharton.rxbinding2.view.clicks
import io.reactivex.Single
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.dialog_task_info.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.dialog.BaseDialogView
import ru.daotech.agroinvest.dialog.DialogBuilderView
import ru.daotech.agroinvest.hide
import ru.daotech.agroinvest.show
import ru.daotech.agroinvest.utils.DateUtil

open class DetailedDialogBuilder :
    DialogBuilderView.DetailedDialogView {

    lateinit var detailedDialogViewModel: DetailedDialogViewModel

    override fun buildTaskCancelAttentionDialog(
        onTaskCancelAccept: PublishSubject<Unit>,
        context: Context,
        showCanceledProgressbar: () -> Unit
    ): AlertDialog {
        val factory = LayoutInflater.from(context)
        val taskCancelAttentionDialogView = factory
            .inflate(R.layout.dialog_fragment_attention_cancel_task, null)
        val taskCancelAttentionDialog = AlertDialog.Builder(context).create()
        taskCancelAttentionDialog.setView(taskCancelAttentionDialogView)
        taskCancelAttentionDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        taskCancelAttentionDialogView.findViewById<Button>(R.id.task_cancel_cancel_attention_button)
            ?.setOnClickListener { taskCancelAttentionDialog.dismiss() }
        taskCancelAttentionDialogView.findViewById<Button>(R.id.task_cancel_confirm_attention_button)
            .clicks()
            .doOnNext {
                Single.just(showCanceledProgressbar())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe()
            }
            .map {
                taskCancelAttentionDialog.dismiss()
                onTaskCancelAccept.onNext(Unit)
            }
            .subscribe()

        return taskCancelAttentionDialog
    }

    override fun buildTaskFinishAttentionDialog(
        context: Context,
        showFinishedProgressbar: () -> Unit,
        hideKeyboard: () -> Unit,
        getCurrentNfcMark: () -> String?,
        checkFragmentInitiateBuilding: () -> Unit
    ): AlertDialog {
        val factory = LayoutInflater.from(context)
        val taskFinishAttentionDialogView = factory
            .inflate(R.layout.dialog_fragment_attention_finish_task, null)
        val taskFinishAttentionDialog = AlertDialog.Builder(context).create()
        taskFinishAttentionDialog.setView(taskFinishAttentionDialogView)
        taskFinishAttentionDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        taskFinishAttentionDialogView.findViewById<Button>(R.id.task_finish_cancel_attention_button)
            ?.setOnClickListener { taskFinishAttentionDialog.dismiss() }
        taskFinishAttentionDialogView.findViewById<Button>(R.id.task_finish_confirm_attention_button)
            .clicks()
            .doOnNext {
                hideKeyboard()
                showFinishedProgressbar()
            }
            .map {
                checkFragmentInitiateBuilding()
                taskFinishAttentionDialog.dismiss()
            }
            .subscribe()

        return taskFinishAttentionDialog
    }

    override fun buildTaskStartAttentionDialog(
        getCurrentNfcMark: () -> String?,
        removeNfcCodeFromArgs: () -> Unit,
        onTaskStartClickSubject: PublishSubject<String>,
        context: Context
    ): AlertDialog {
        val factory = LayoutInflater.from(context)
        val taskStartAttentionDialogView = factory
            .inflate(R.layout.dialog_fragment_attention_start_task, null)
        val taskStartAttentionDialog = AlertDialog.Builder(context).create()
        taskStartAttentionDialog.setView(taskStartAttentionDialogView)
        taskStartAttentionDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        taskStartAttentionDialogView.findViewById<Button>(R.id.task_finish_start_attention_button)
            ?.setOnClickListener { taskStartAttentionDialog.dismiss() }
        taskStartAttentionDialogView.findViewById<Button>(R.id.task_start_confirm_attention_button)
            .clicks()
            .map {
                taskStartAttentionDialog.dismiss()
                onTaskStartClickSubject.onNext(getCurrentNfcMark()!!)

                // Deleting nfc code from arguments
                // the task dialog finish would not show again
                removeNfcCodeFromArgs()
            }
            .subscribe()

        taskStartAttentionDialog.show()

        return taskStartAttentionDialog
    }

    @SuppressLint("SetTextI18n", "InflateParams")
    override fun buildTaskInfoDialog(
        context: Context,
        taskWithProperties: TaskWithProperties
    ): AlertDialog {
        val factory = LayoutInflater.from(context)
        val taskFinishAttentionDialogView = factory
            .inflate(R.layout.dialog_task_info, null)
        val taskInfoDialog = AlertDialog.Builder(context).create()
        taskInfoDialog.setView(taskFinishAttentionDialogView)
        taskInfoDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        taskFinishAttentionDialogView?.let { dialog ->
            with(dialog) {
                task_info_card_container
                    .clicks()
                    .map { taskInfoDialog.dismiss() }
                    .subscribe()

                task_info_number_text_view.text = taskWithProperties.name

                task_info_rating.rating = taskWithProperties.rating

                task_info_date.text =
                    DateUtil.getViewFormatDateStringFromDateString(taskWithProperties.startAt) +
                            " ${context.getString(R.string.skipping_character)} " +
                            DateUtil.getViewFormatDateStringFromDateString(taskWithProperties.finishAt)

                if (taskWithProperties.position == null
                    && taskWithProperties.position?.name.isNullOrBlank()
                ) {
                    task_info_position_title.hide()
                    task_info_position.hide()
                } else {
                    task_info_position.text = taskWithProperties.position?.name
                    task_info_position_title.show()
                    task_info_position.show()
                }

                if (taskWithProperties.department == null) {
                    task_info_department_title.hide()
                    task_info_department.hide()
                } else {
                    task_info_department.text = taskWithProperties.department?.fullName
                    task_info_department_title.show()
                    task_info_department.show()
                }

                if (taskWithProperties.section == null) {
                    task_info_section.hide()
                    task_info_section_title.hide()
                } else {
                    task_info_section.text = taskWithProperties.section?.fullName
                    task_info_section.show()
                    task_info_section_title.show()
                }

                if (taskWithProperties.description.isBlank()) {
                    task_info_description.hide()
                    task_info_description_title.hide()
                } else {
                    task_info_description.text = taskWithProperties.description
                    task_info_description_title.show()
                    task_info_description.show()
                }
            }
        }

        return taskInfoDialog
    }


    //FIXME: The method repeat the same function from aboutFragment of dialogBuilder. DRY it.
    override fun buildResponseAttentionDialog(
        context: Context,
        dismissResponseDialog: PublishSubject<Unit>
    ): AlertDialog {
        val factory = LayoutInflater.from(context)
        val responseDialogView = factory
            .inflate(R.layout.dialog_success_motion, null)
        val responseDialog = AlertDialog.Builder(context).create()
        responseDialog.setView(responseDialogView)
        responseDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        responseDialogView.findViewById<Button>(R.id.result_export_button)
            ?.setOnClickListener {
                dismissResponseDialog.onNext(Unit)
                responseDialog.dismiss()
            }
        val doneIcon =
            responseDialogView.findViewById<ImageView>(R.id.done_container_view)
        val failedIcon =
            responseDialogView.findViewById<ImageView>(R.id.failed_container_view)

        // Build view current model state
        detailedDialogViewModel = DetailedDialogViewModel(
            doneIcon,
            failedIcon,
            responseDialogView,
            responseDialog,
            context
        )

        return responseDialog
    }

    override fun showResultDialog(isResultSuccess: Boolean) {
        if (isResultSuccess) {
            detailedDialogViewModel.doneIcon.visibility = View.VISIBLE
            detailedDialogViewModel.failedIcon.visibility = View.INVISIBLE
            detailedDialogViewModel.dialogView
                .findViewById<TextView>(R.id.result_export_text_view)
                .text =
                detailedDialogViewModel.context.getString(R.string.success_export_result_text_view)

            val doneDrawable = detailedDialogViewModel.doneIcon.drawable
            drawAnimation(doneDrawable)
        } else {
            detailedDialogViewModel.doneIcon.visibility = View.INVISIBLE
            detailedDialogViewModel.failedIcon.visibility = View.VISIBLE
            detailedDialogViewModel.dialogView
                .findViewById<TextView>(R.id.result_export_text_view)
                .text =
                detailedDialogViewModel.context.getString(R.string.failed_export_result_text_view)

            val doneDrawable = detailedDialogViewModel.failedIcon.drawable
            drawAnimation(doneDrawable)
        }
        detailedDialogViewModel.dialog.show()
    }

    private fun drawAnimation(doneDrawable: Drawable) {
        if (doneDrawable is AnimatedVectorDrawableCompat) {
            doneDrawable.start()
        } else if (doneDrawable is AnimatedVectorDrawable) {
            doneDrawable.start()
        }
    }
}

data class DetailedDialogViewModel(
    val doneIcon: ImageView,
    val failedIcon: ImageView,
    override val dialogView: View,
    override val dialog: AlertDialog,
    override val context: Context
) : BaseDialogView()