package ru.daotech.agroinvest.dialog.builder

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.net.Uri
import android.text.Html
import android.text.Spannable
import android.text.SpannableString
import android.text.style.ForegroundColorSpan
import android.view.LayoutInflater
import android.view.View.VISIBLE
import android.widget.*
import androidx.appcompat.app.AlertDialog
import com.jakewharton.rxbinding2.view.clicks
import kotlinx.android.synthetic.main.dialog_hint_info.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.adapters.*
import ru.daotech.agroinvest.database.entities.TaskPlantImpl
import ru.daotech.agroinvest.dialog.DialogBuilderView
import ru.daotech.agroinvest.service.jsonmodels.PlantPayload

open class HeaderDialogBuilder : DialogBuilderView.HeaderDialogView {

    @SuppressLint("InflateParams")
    override fun buildAndShowErrorDialog(context: Context, message: String) {
        val factory = LayoutInflater.from(context)
        val accidentReportDialogView = factory
            .inflate(R.layout.dialog_success_motion, null)
        val accidentReportDialog = AlertDialog.Builder(context).create()
        accidentReportDialog.setView(accidentReportDialogView)
        accidentReportDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        accidentReportDialogView.findViewById<Button>(R.id.result_export_button)
            ?.setOnClickListener { accidentReportDialog.dismiss() }
        accidentReportDialogView.findViewById<TextView>(R.id.result_export_text_view)
            .text = message
        accidentReportDialogView.findViewById<ImageView>(R.id.attention_container_view)
            .visibility = VISIBLE
        accidentReportDialog.show()
    }

    @SuppressLint("SetTextI18n")
    override fun buildAndShowAttentionDialog(
        propertyList: List<Any>,
        signalToSubmitForm: () -> Unit,
        plant: PlantPayload,
        context: Context
    ): AlertDialog {
        val factory = LayoutInflater.from(context)
        val taskAttentionDialogView = factory
            .inflate(R.layout.dialog_fragment_attention_cancel_task, null)
        val taskAttentionDialog = AlertDialog.Builder(context).create()
        taskAttentionDialog.setView(taskAttentionDialogView)
        taskAttentionDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        taskAttentionDialogView.findViewById<TextView>(R.id.dialog_title_view).text
            context.getString(R.string.accident_group_dialog_title)
        taskAttentionDialogView.findViewById<Button>(R.id.task_cancel_cancel_attention_button)
            ?.setOnClickListener {
                taskAttentionDialog.dismiss()
            }

        val groupName = propertyList.filterIsInstance<GroupModelView>().first().property.name
        val taskAttentionTextTitleView =
            taskAttentionDialogView.findViewById<TextView>(R.id.header_group_plant_title)
        taskAttentionTextTitleView.visibility = VISIBLE

        val spannable = Html.fromHtml(
            """Вы сняли метрики на 
                |<b color="black">${plant.name}</b> для 
                |<b>${groupName}</b>.""".trimMargin()
        )

        taskAttentionTextTitleView.text = spannable

        val taskAttentionListView =
            taskAttentionDialogView.findViewById<ListView>(R.id.group_type_dialog_list)
        taskAttentionListView.visibility = VISIBLE

        val receivedPropertyList = propertyList.mapIndexed { index, it ->
            when (it) {
                is TextAreaModelView -> null
                is NumericFieldModelView ->
                    return@mapIndexed """${it.property.name}: ${it.property.value}"""
                is ImageBinaryModelView -> null
                is HeaderModelView -> null
                is SimpleTaskPropertyModel -> null
                is GroupModelView -> null
                else -> throw TypeCastException()
            }
        }.filterNotNull()

        val arrayAdapter: ArrayAdapter<String> = ArrayAdapter(
            context,
            R.layout.item_list_view_group_type,
            R.id.group_type_view,
            receivedPropertyList
        )

        taskAttentionListView.adapter = arrayAdapter

        taskAttentionDialogView.findViewById<Button>(R.id.task_cancel_confirm_attention_button)
            .clicks()
            .map {
                taskAttentionDialog.dismiss()
                signalToSubmitForm.invoke()
            }
            .subscribe()

        taskAttentionDialog.show()

        return taskAttentionDialog
    }

    @SuppressLint("InflateParams")
    fun buildAndShowFullscreenImageDialog(context: Context, uri: Uri?) {
        val factory = LayoutInflater.from(context)
        val fullscreenDialogView = factory
            .inflate(R.layout.dialog_fragment_fullscreen_image_view, null)
        val fullscreenDialog = AlertDialog.Builder(context).create()
        fullscreenDialog.setView(fullscreenDialogView)
        fullscreenDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        when (uri != null) {
            true -> {
                fullscreenDialogView.findViewById<ImageView>(R.id.fullscreen_image_view)
                    .setImageURI(uri)
            }
            false -> {
                fullscreenDialogView.findViewById<ImageView>(R.id.fullscreen_image_view)
                    .setImageBitmap(uri)
            }
        }
        fullscreenDialog.show()
    }

    fun buildAndShowHintDialog(context: Context, message: String) {
        val factory = LayoutInflater.from(context)
        val hintDialogView = factory
            .inflate(R.layout.dialog_hint_info, null)
        val hintDialog = AlertDialog.Builder(context).create()
        hintDialog.setView(hintDialogView)
        hintDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        hintDialog.window?.attributes?.windowAnimations = R.style.SlidingDialogAnimation

        hintDialogView?.let { dialog ->
            with(dialog) {
                task_hint_card_container
                    .clicks()
                    .map { hintDialog.dismiss() }
                    .subscribe()
            }
        }

        hintDialogView.findViewById<TextView>(R.id.hint_description_text)
            .text = message
        hintDialog.show()
    }
}