package ru.daotech.agroinvest.dialog.builder

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.view.LayoutInflater
import android.view.View
import android.widget.Button
import android.widget.CheckBox
import android.widget.LinearLayout
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import com.jakewharton.rxbinding2.view.clicks
import io.reactivex.subjects.PublishSubject
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.dialog.DialogBuilderView
import ru.daotech.agroinvest.utils.DateUtil
import ru.daotech.agroinvest.utils.TROLLEY_TIED_DELAY
import javax.inject.Inject

class TrolleyDialogBuilder : DialogBuilderView.TrolleyDialogView {

    lateinit var appStateManager: AppStateManager

    private fun checkTrolleyUntiedLowTime(
        trolleyTiedAttentionCheckBox: LinearLayout,
        trolleyAttentionDialogView: View,
        trolleyNumber: String,
        trolleyAttentionDialog: AlertDialog,
        onTrolleyUntiedClick: PublishSubject<String>,
        errorMessageSubject: PublishSubject<String>
    ) {
        val timeDiff = DateUtil.getSecondsBetweenTwoDates(
            DateUtil.getCurrentDateTimestampWithDelta(
                appStateManager.getServerTimestamp()
            ),
            appStateManager.getCurrentTrolleyTiedTimeMark()!!
        )
        var isCheckConsent = false
        if (timeDiff.toInt() < TROLLEY_TIED_DELAY) {
            trolleyTiedAttentionCheckBox.visibility = View.VISIBLE
            trolleyAttentionDialogView.findViewById<CheckBox>(R.id.untied_trolley_low_time_checkbox)
                ?.setOnCheckedChangeListener { buttonView, isChecked ->
                    isCheckConsent = isChecked
                }

            trolleyAttentionDialogView.findViewById<Button>(R.id.confirm_trolley_button)
                .clicks()
                .map {
                    if (isCheckConsent) {
                        trolleyAttentionDialog.dismiss()
                        onTrolleyUntiedClick.onNext(trolleyNumber)
                    } else {
                        errorMessageSubject.onNext("Необходимо дать согласие на отвязку.")
                    }
                }
                .subscribe()
        } else {
            trolleyTiedAttentionCheckBox.visibility = View.GONE
        }
    }

    @SuppressLint("InflateParams")
    override fun buildTrolleyAttentionDialog(
        context: Context,
        dialogMessage: String,
        trolleyNumber: String,
        isTakeNewTrolley: Boolean,
        onTrolleyTakeClick: PublishSubject<String>,
        onTrolleyUntiedClick: PublishSubject<String>,
        errorMessageSubject: PublishSubject<String>,
        appStateManager: AppStateManager
    ): AlertDialog {
        this.appStateManager = appStateManager

        val factory = LayoutInflater.from(context)
        val trolleyAttentionDialogView = factory
            .inflate(R.layout.dialog_fragment_trolley_attention, null)
        val trolleyAttentionDialog = AlertDialog.Builder(context).create()
        trolleyAttentionDialog.setView(trolleyAttentionDialogView)
        trolleyAttentionDialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        val trolleyTiedAttentionCheckBox =
            trolleyAttentionDialogView.findViewById<LinearLayout>(
                R.id.untied_trolley_low_time_checkbox_container
            )
        trolleyAttentionDialogView.findViewById<TextView>(R.id.trolley_tied_attention_number)
            .text =
            trolleyNumber
        trolleyAttentionDialogView.findViewById<TextView>(R.id.trolley_tied_attention_message)
            .text =
            dialogMessage
        trolleyAttentionDialogView.findViewById<Button>(R.id.cancel_trolley_button)
            ?.setOnClickListener { trolleyAttentionDialog.dismiss() }
        trolleyAttentionDialogView.findViewById<Button>(R.id.confirm_trolley_button)
            .clicks()
            .map {
                trolleyAttentionDialog.dismiss()
                if (isTakeNewTrolley) {
                    onTrolleyTakeClick.onNext(trolleyNumber)
                } else {
                    onTrolleyUntiedClick.onNext(trolleyNumber)
                }
            }
            .subscribe()

        if (!isTakeNewTrolley) {
            checkTrolleyUntiedLowTime(
                trolleyTiedAttentionCheckBox,
                trolleyAttentionDialogView,
                trolleyNumber,
                trolleyAttentionDialog,
                onTrolleyUntiedClick,
                errorMessageSubject
            )
        }

        if (!trolleyAttentionDialogView.isShown) {
            trolleyAttentionDialog.show()
        }

        return trolleyAttentionDialog
    }
}