package ru.daotech.agroinvest.fragments.barcode

import com.hannesdorfmann.mosby3.mvi.MviBasePresenter
import io.reactivex.disposables.Disposable
import timber.log.Timber
import javax.inject.Inject

class BarcodeScannerPresenter @Inject constructor(private val onBarcodeRetrieved: ((String) -> Unit)?) :
    MviBasePresenter<BarcodeScannerView, BarcodeScannerViewState>() {

    lateinit var disposable: Disposable

    override fun bindIntents() {
        val onBarcodeRetrievedIntent = intent(BarcodeScannerView::onBarcodeRetrieved)
            .map { retrievedBarcode ->
                BarcodeScannerViewState.Success(retrievedBarcode) as BarcodeScannerViewState
            }
            .onErrorReturn { error ->
                Timber.e(error)
                BarcodeScannerViewState.Error(error.localizedMessage)
            }
            .share()

        disposable = onBarcodeRetrievedIntent.filter { state ->
            state is BarcodeScannerViewState.Success
        }.map { state ->
            (state as BarcodeScannerViewState.Success).retrievedBarcode
        }.subscribe(
            { barcode ->
                onBarcodeRetrieved?.invoke(barcode)
            },
            { error ->
                Timber.e(error, error.localizedMessage)
            }
        )
    }

    override fun unbindIntents() {
        disposable.dispose()
        super.unbindIntents()
    }

}