package ru.daotech.agroinvest.fragments.base

import android.annotation.SuppressLint
import android.content.Context
import androidx.annotation.ColorRes
import androidx.core.content.ContextCompat
import androidx.core.content.ContextCompat.getColor
import com.google.android.material.snackbar.Snackbar
import com.hannesdorfmann.mosby3.mvi.MviFragment
import com.hannesdorfmann.mosby3.mvi.MviPresenter
import dagger.android.support.AndroidSupportInjection
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.interfaces.MviView
import javax.inject.Inject

/**
 * BaseMviFragment requires three generic classes:
 * [ViewState] - any class
 * [View] - must extends [MviView] with same [ViewState] generic class
 * [Presenter] - must extends [MviPresenter] with same [View] and [ViewState] generic classes
 */
abstract class BaseMviFragment<ViewState, View : MviView<ViewState>,
        Presenter : MviPresenter<View, ViewState>> : MviFragment<View, Presenter>() {

    protected val snackbarDismiss: PublishSubject<Unit> = PublishSubject.create()

    protected lateinit var snackbar: Snackbar

    @Inject
    lateinit var presenter: Presenter

    override fun onAttach(context: Context) {
        AndroidSupportInjection.inject(this)
        super.onAttach(context)
    }

    override fun createPresenter(): Presenter {
        return presenter
    }

    @SuppressLint("CheckResult")
    protected fun showSnackbar(message: String, @ColorRes snackbarBackgroundColor: Int? = null) {
        snackbar = Snackbar.make(view!!, message, Snackbar.LENGTH_INDEFINITE)
        snackbar
            .setAction("OK") {
                snackbarDismiss.onNext(Unit)
            }
            .setActionTextColor(getColor(requireContext(), R.color.colorWhite))
        val backgroundColor = snackbarBackgroundColor ?: getColor(
            requireContext(),
            R.color.accentSnackbarBackground
        )
        snackbar.view.setBackgroundColor(backgroundColor)
        snackbar.addCallback(object : Snackbar.Callback() {
            @SuppressLint("SwitchIntDef")
            override fun onDismissed(transientBottomBar: Snackbar?, event: Int) {
                when (event) {
                    DISMISS_EVENT_SWIPE -> {
                        snackbarDismiss.onNext(Unit)
                    }
                    DISMISS_EVENT_TIMEOUT -> {
                        snackbarDismiss.onNext(Unit)
                    }
                    DISMISS_EVENT_ACTION -> {
                        snackbarDismiss.onNext(Unit)
                    }
                    else -> {
                        super.onDismissed(transientBottomBar, event)
                    }
                }
            }
        })
        snackbar.show()
    }

    protected fun dismissSnackbar() {
        if (::snackbar.isInitialized) {
            snackbar.dismiss()
        }
    }
}