package ru.daotech.agroinvest.fragments.detailed_task

import android.content.Context
import android.net.ConnectivityManager
import android.net.Uri
import android.os.Bundle
import android.view.*
import android.view.View.GONE
import android.view.View.VISIBLE
import android.view.inputmethod.InputMethodManager
import android.widget.RelativeLayout
import androidx.appcompat.app.AlertDialog
import androidx.drawerlayout.widget.DrawerLayout
import androidx.fragment.app.Fragment
import com.jakewharton.rxbinding2.view.clicks
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.schedulers.Schedulers
import io.reactivex.subjects.BehaviorSubject
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.dialog_fragment_group_proprty.*
import kotlinx.android.synthetic.main.fragment_detailed_task.*
import kotlinx.android.synthetic.main.fragment_detailed_task.view.*
import kotlinx.android.synthetic.main.sub_toolbar_trolley_tied.view.*
import ru.daotech.agroinvest.*
import ru.daotech.agroinvest.components.NavigationFragmentComponentImpl
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.coordinators.NFC_CODE_DETAILED_ARGS
import ru.daotech.agroinvest.database.entities.TaskProperty
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.dialog.builder.DetailedDialogBuilder
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.DetailedTaskProcessingTab
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.group_properties_container.GroupPropertyFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.header_field_container.HeaderFieldFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.timer_container.TimerMviFragment
import ru.daotech.agroinvest.fragments.pages.accident.AccidentReportFragment
import ru.daotech.agroinvest.fragments.trolley_tied.TrolleyTiedFragment
import ru.daotech.agroinvest.models.InternetConnectivityState
import ru.daotech.agroinvest.service.jsonmodels.TaskPropertyPayload
import ru.daotech.agroinvest.utils.KEY_INITIATE_NFC_VALUE_ARGUMENT
import ru.daotech.agroinvest.utils.TASK_IN_PROGRESS_FRAGMENT_TAG
import ru.daotech.agroinvest.utils.TIMER_FRAGMENT_TAG
import ru.daotech.agroinvest.utils.ViewUtil
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

class DetailedTaskFragment :
    BaseMviFragment<DetailedTaskViewState, DetailedTaskView, DetailedTaskPresenter>(),
    DetailedTaskView {

    @Inject
    lateinit var appStateManager: AppStateManager

    @Inject
    lateinit var detailedDialogBuilder: DetailedDialogBuilder

    @Inject
    lateinit var navigationComponent: NavigationFragmentComponentImpl

    private var disposable = CompositeDisposable()
    private var errorDisposable = CompositeDisposable()

    private lateinit var taskCancelAttentionDialog: AlertDialog
    private lateinit var taskFinishAttentionDialog: AlertDialog
    private lateinit var taskStartAttentionDialog: AlertDialog
    private lateinit var taskInfoDialog: AlertDialog

    private val onDismissResponseDialog: PublishSubject<Unit> = PublishSubject.create()
    private val onDetailedTaskLoadFromDb: PublishSubject<String> = PublishSubject.create()
    private val onTaskCancelAccept: PublishSubject<Unit> = PublishSubject.create()
    private val onGoToMyTasksSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onGoToBarcodeBoxFragmentSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onShowAttentionCheck: PublishSubject<Unit> = PublishSubject.create()
    private val onDismissTaskStates: PublishSubject<Unit> = PublishSubject.create()
    private val onErrorDismiss: PublishSubject<Unit> = PublishSubject.create()
    private val onTaskStartClickSubject: PublishSubject<String> = PublishSubject.create()
    private val onCurrentTrolleyTiedSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onTaskFinishClickSubject: PublishSubject<Pair<String, String>> =
        PublishSubject.create()
    private val onErrorMessageSubject: PublishSubject<String> =
        PublishSubject.create()
    private val onTaskFinishWithPropsClickSubject: PublishSubject<Triple<String, List<TaskPropertyPayload?>, List<Uri?>>> =
        PublishSubject.create()
    private val onSendGroupPropertySubject: PublishSubject<Pair<List<TaskPropertyPayload?>, List<Uri>>> =
        PublishSubject.create()
    private val onPropertyRetrieveSubject: BehaviorSubject<Pair<List<TaskPropertyPayload?>, List<Uri>>> =
        BehaviorSubject.create()
    private val onBuildGroupPropertyWithoutMeasurements: PublishSubject<Pair<String, String>> =
        PublishSubject.create()
    val onGroupPropertyRetrieveSubject: BehaviorSubject<Pair<List<TaskPropertyPayload?>, List<Uri>>> =
        BehaviorSubject.create()

    // Observing internet connectivity state
    private val internetConnectivityStateSubject: BehaviorSubject<InternetConnectivityState> =
        BehaviorSubject.create()

    private lateinit var fullTaskInfo: TaskWithProperties
    private var currentNfcMark: String? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)

        taskCancelAttentionDialog = detailedDialogBuilder.buildTaskCancelAttentionDialog(
            onTaskCancelAccept,
            requireContext()
        ) { showCanceledProgressbar() }

        taskFinishAttentionDialog = detailedDialogBuilder.buildTaskFinishAttentionDialog(
            requireContext(),
            { showFinishedProgressbar() },
            { this.hideKeyboard() },
            { getCurrentNfcMark() },
            { checkFragmentInitiateBuilding() }
        )

        retainInstance = true
        this.activity?.drawer_layout?.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED)
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        super.onPrepareOptionsMenu(menu)

        (menu.findItem(R.id.menu_search) as MenuItem).isVisible = false

        val menuItemTaskInfo = (menu.findItem(R.id.menu_task_info) as MenuItem)

        activity?.let { activity ->
            ViewUtil.manageInternetConnectState(activity, menu)
            ViewUtil.manageSocketConnectState(activity, menu)
        }

        menuItemTaskInfo.setVisible(true)
        menuItemTaskInfo
            .clicks()
            .doOnNext { this.hideKeyboard() }
            .doOnError { Timber.e(it) }
            .map { taskInfoDialog.show() }
            .subscribe()

        val menuItemTaskAccidentReport = (menu.findItem(R.id.menu_task_accident_report)
                as MenuItem)
        menuItemTaskAccidentReport.setVisible(true)
        menuItemTaskAccidentReport
            .clicks()
            .doOnNext { this.hideKeyboard() }
            .doOnError { Timber.e(it) }
            .map {
                navigationComponent.buildAndShowAccidentFragment(
                    fragmentManager,
                    fullTaskInfo
                )
            }
            .subscribe()
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        val view = inflater.inflate(
            R.layout.fragment_detailed_task,
            container,
            false
        )

        view.start_button.clicks()
            .doOnNext { this.hideKeyboard() }
            .doOnError { Timber.e(it) }
            .map { buildTaskStartAttention() }
            .subscribe()

        // Handle onBack device button
        handleOnBackPressed()

        // Handle onBack toolbar button
        handleOnBackPressedToolbar()

        internetConnectivityStateSubject.onNext(
            InternetConnectivityState(
                requireContext().getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
            )
        )

        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        detailedDialogBuilder.buildResponseAttentionDialog(
            requireContext(),
            onDismissResponseDialog
        )

        headerPropertyRetrieveDisposable()
        groupPropertyRetrieveDisposable()
        groupPropertyBuildByBarcodeDisposable()
        errorBuildDisposable()
    }

    private fun errorBuildDisposable() {
        if (!errorDisposable.isDisposed) {
            errorDisposable.clear()
        }

        errorDisposable.add(
            onErrorMessageSubject
                .throttleFirst(5, TimeUnit.SECONDS)
                .map { errorMessage -> showSnackbar(errorMessage) }
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnError { Timber.e(it.localizedMessage) }
                .subscribe()
        )
    }

    override fun onHiddenChanged(hidden: Boolean) {
        if (hidden) {
            handleToolbarControllers()
        } else {
            handleOnBackPressedToolbar()
        }
        super.onHiddenChanged(hidden)
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)

        arguments?.let { bundle ->
            val args = DetailedTaskFragmentArgs.fromBundle(bundle)
            args.detailedTaskModel.let { detailedTaskId ->
                onDetailedTaskLoadFromDb.onNext(detailedTaskId)
            }
        }
    }

    override fun onResume() {
        if (childFragmentManager.findFragmentByTag(TASK_IN_PROGRESS_FRAGMENT_TAG)
                    !is DetailedTaskProcessingTab
            && childFragmentManager.findFragmentByTag(TIMER_FRAGMENT_TAG) !is TimerMviFragment
        ) {
            arguments?.let { bundle ->
                val args = DetailedTaskFragmentArgs.fromBundle(bundle)
                args.nfcValue.let { nfcMark ->
                    currentNfcMark = nfcMark.toString()
                }
                args.detailedTaskModel.let { detailedTaskId ->
                    onDetailedTaskLoadFromDb.onNext(detailedTaskId)
                }
            }
        }
        super.onResume()
    }

    override fun loadingIntent(): Observable<String> = onDetailedTaskLoadFromDb

    override fun taskCancelAcceptClick(): Observable<Unit> = onTaskCancelAccept

    override fun onDismissResponseDialogIntent(): Observable<Unit> = onDismissResponseDialog

    override fun showAttentionCheck(): Observable<Unit> = onShowAttentionCheck

    override fun snackbarDismiss(): Observable<Unit> = snackbarDismiss

    override fun onErrorDismiss(): Observable<Unit> = onErrorDismiss

    override fun taskFinishWithPropsAcceptClick(): Observable<Triple<String, List<TaskPropertyPayload?>, List<Uri?>>> =
        onTaskFinishWithPropsClickSubject

    override fun onSendGroupPropertyClick(): Observable<Pair<List<TaskPropertyPayload?>, List<Uri>>> =
        onSendGroupPropertySubject

    override fun taskFinishAcceptClick(): Observable<Pair<String, String>> =
        onTaskFinishClickSubject

    override fun startButtonClick(): Observable<String> = onTaskStartClickSubject

    override fun goToMyTasksIntent(): Observable<Unit> = onGoToMyTasksSubject

    override fun dismissTaskStates(): Observable<Unit> = onDismissTaskStates

    override fun onCurrentTrolleyTiedIntent(): Observable<Unit> = onCurrentTrolleyTiedSubject

    override fun onGoToBarcodeBoxFragmentIntent(): Observable<Unit> =
        onGoToBarcodeBoxFragmentSubject

    override fun internetConnectivityStateIntent(): Observable<InternetConnectivityState> =
        internetConnectivityStateSubject

    override fun render(state: DetailedTaskViewState) {
        manageErrors(state)

        with(state) {
            if (isLoading) {
                (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.tasks_loading)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
                progressbar.show()
                start_label.hide()
                start_button.isEnabled = false
                finish_button.isEnabled = false
            } else {
                progressbar.hide()
                start_label.show()
                start_button.isEnabled = true
                finish_button.isEnabled = true

                if (state.nfcMode != null && state.nfcMode) {
                    start_button.visibility = GONE
                    finish_button.visibility = GONE
                    task_processing_button_container.visibility = GONE
                } else {
                    start_button.visibility = VISIBLE
                    finish_button.visibility = VISIBLE
                    task_processing_button_container.visibility = VISIBLE
                }
            }

            taskWithProperties?.let { taskWithProperties ->
                fullTaskInfo = taskWithProperties
                (activity as ContainerActivity).activity_container_custom_toolbar?.title = taskWithProperties.name
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle =
                    "${BuildConfig.VERSION_NAME} ${BuildConfig.BUILD_TYPE}"

                taskInfoDialog = detailedDialogBuilder.buildTaskInfoDialog(
                    requireContext(),
                    taskWithProperties
                )

                if (taskWithProperties.isHarvesting != null
                    && taskWithProperties.isHarvesting == true
                ) {
                    showSubToolbar()
                }

                if (!taskWithProperties.startedAt.isNullOrBlank()
                    && taskWithProperties.finishedAt.isNullOrBlank()
                    || isInProgressTaskState
                ) {
                    // Check is necessary for correct reopen a current fragment.
                    // If current location is not TimerFragment,
                    // we have to open this fragment, else - do nothing
                    if (childFragmentManager
                            .findFragmentByTag(TIMER_FRAGMENT_TAG) !is TimerMviFragment
                    ) {
                        navigationComponent.buildAndShowTimerFragment(
                            view!!,
                            requireContext(),
                            childFragmentManager,
                            taskWithProperties,
                            onGoToBarcodeBoxFragmentSubject
                        )
                    }

                    view?.task_properties_row_number?.text =
                        requireContext().getString(R.string.raw_title_prefix) +
                                taskWithProperties.rowNumber?.toString()
                    view?.task_properties_job_type?.text = taskWithProperties.section?.name

                    showTaskInProgressView()

                } else {
                    if (isPreLaunchTaskState) {

                        navigationComponent.buildAndShowDetailedTaskInProgressFragment(
                            view!!,
                            requireContext(),
                            childFragmentManager,
                            taskWithProperties,
                            onDismissTaskStates
                        ) { showTaskSimpleView() }

                        arguments?.let { bundle ->
                            val args = DetailedTaskFragmentArgs.fromBundle(bundle)
                            args.nfcValue?.let {
                                if (it.isNotBlank()) {
                                    buildTaskStartAttention()
                                }
                            }
                        }
                    }
                }

                if (isGroupPropertyUploadSuccess) {
                    detailedDialogBuilder.showResultDialog(true)
                }
            }
        }
    }

    private fun showSubToolbar() {
        val includeToolbarContainer =
            this.view?.findViewById<RelativeLayout>(R.id.sub_toolbar_trolley_tied_detailed)

        includeToolbarContainer?.visibility = VISIBLE

        this.view?.current_trolley_value!!.text =
            when (appStateManager.getCurrentTrolleyNumber().isNotBlank()) {
                true -> {
                    includeToolbarContainer!!.setBackgroundResource(R.color.colorAccent)
                    view?.remove_trolley_button?.visibility = VISIBLE
                    view?.add_trolley_button?.visibility = GONE
                    appStateManager.getCurrentTrolleyNumber()
                }
                else -> {
                    includeToolbarContainer!!.setBackgroundResource(R.color.colorAppAttentionRed)
                    view?.remove_trolley_button?.visibility = GONE
                    view?.add_trolley_button?.visibility = VISIBLE
                    getString(R.string.no_trolley_assigned_title)
                }
            }

        includeToolbarContainer.setOnClickListener {
            onCurrentTrolleyTiedSubject.onNext(Unit)
        }
    }

    private fun getCurrentNfcMark(): String? {
        return currentNfcMark
    }

    private fun handleToolbarControllers() {
        (activity as ContainerActivity).activity_container_custom_toolbar.setNavigationOnClickListener {
            this.hideKeyboard()
            fragmentManager!!.beginTransaction().apply {
                fragmentManager!!.fragments.forEach {
                    if (it is AccidentReportFragment) {
                        this.hide(it)
                        this.remove(it)
                    }
                    if (it is TrolleyTiedFragment) {
                        this.hide(it)
                        this.remove(it)
                    }
                    if (it is DetailedTaskFragment && !it.isVisible) {
                        this.show(it)
                    }
                }
                commit()
            }
        }
    }

    private fun Fragment.hideKeyboard() {
        try {
            if (context != null) {
                val imm = context?.getSystemService(Context.INPUT_METHOD_SERVICE)!!
                        as InputMethodManager

                if (this.view != null) {
                    imm.hideSoftInputFromWindow(this.view?.windowToken, 0)
                }
            }
        } catch (ex: Exception) {
            Timber.e(ex, "SOFT INPUT: hideKeyboard error")
        }
    }

    private fun buildTaskStartAttention() {
        taskStartAttentionDialog = detailedDialogBuilder.buildTaskStartAttentionDialog(
            { getCurrentNfcMark() },
            { arguments?.remove(NFC_CODE_DETAILED_ARGS) },
            onTaskStartClickSubject,
            requireContext()
        )
    }

    fun initiateTaskFinish(nfcTaskRowMark: String) {
        if (!::taskFinishAttentionDialog.isInitialized || !taskFinishAttentionDialog.isShowing) {
            currentNfcMark = nfcTaskRowMark
            taskFinishAttentionDialog.show()
        }
    }

    fun initiateTaskStart(nfcTaskRowMark: String) {
        if (!::taskStartAttentionDialog.isInitialized || !taskStartAttentionDialog.isShowing) {
            currentNfcMark = nfcTaskRowMark
            buildTaskStartAttention()
        }
    }

    fun showTrolleyFragment(nfcMessageArgument: String?) {
        navigationComponent.buildAndShowTrolleyFragment(
            childFragmentManager,
            nfcMessageArgument
        )
    }

    private fun showTaskSimpleView() {
        detailed_task_in_progress_buttons_container.visibility = GONE
        start_button.visibility = VISIBLE
        detailed_task_in_job_type_container.visibility = VISIBLE
        detailed_task_in_progress_container.visibility = GONE
        task_in_progress_stage_title.visibility = GONE
    }

    private fun showTaskInProgressView() {
        detailed_task_in_progress_buttons_container.visibility = VISIBLE
        detailed_task_in_job_type_container.visibility = VISIBLE
        start_button.visibility = GONE
        task_in_progress_stage_title.visibility = GONE

        view?.finish_button?.setOnClickListener {
            // Clear focus need to initiate saving properties in fields
            properties_container.clearFocus()
            finish_button.requestFocus()
            taskFinishAttentionDialog.show()
        }

        view?.cancel_task_button?.setOnClickListener {
            // Clear focus need to initiate saving properties in fields
            properties_container.clearFocus()
            cancel_task_button.requestFocus()
            taskCancelAttentionDialog.show()
        }
    }

    // On view handling errors
    private fun manageErrors(state: DetailedTaskViewState) {
        with(state) {
            when {
                errorMessage != null -> {
                    hideFinishedProgressbar()
                    onErrorMessageSubject.onNext(errorMessage)
                }
                else -> {
                    dismissSnackbar()
                }
            }
        }
    }

    private fun showFinishedProgressbar() {
        finish_task_button_progressbar?.let {
            finish_task_button_progressbar.visibility = VISIBLE
            finish_task_button_label.hide()
            finish_button.isEnabled = false
        }
    }

    private fun hideFinishedProgressbar() {
        finish_task_button_progressbar?.let {
            finish_task_button_progressbar.visibility = GONE
            finish_task_button_label.show()
            finish_button.isEnabled = true
        }
    }

    private fun showCanceledProgressbar() {
        cancel_task_button_progressbar?.let {
            cancel_task_button_progressbar.visibility = VISIBLE
            cancel_task_button_label.hide()
            cancel_task_button.isEnabled = false
        }
    }

    private fun handleOnBackPressedToolbar() {
        (activity as ContainerActivity).activity_container_custom_toolbar.setNavigationOnClickListener {
            onBackMarkTask(false)
        }
    }

    private fun handleOnBackPressed() {
        this.hideKeyboard()

        this.activity?.drawer_layout?.setOnKeyListener { _, keyCode, event ->
            if ((keyCode == KeyEvent.KEYCODE_BACK)
                && event.action == KeyEvent.ACTION_DOWN
            ) {
                properties_container?.clearFocus()
                cancel_task_button?.requestFocus()
                onBackMarkTask(true)
            }
            false
        }
    }

    private fun onBackMarkTask(isOnBackCalled: Boolean) {
        properties_container?.clearFocus()
        cancel_task_button?.requestFocus()
        if (this.isVisible) {
            this.hideKeyboard()
            if (!isOnBackCalled) {
                this.activity?.onBackPressed()
            }
        }
    }

    private fun checkFragmentInitiateBuilding() {
        // Required checking for interrupt java.lang.IllegalStateException: Fragment has not been attached yet
        // This happens when the fragment is no longer attached to its parent activity/fragment.
        // So a simple isAdded check should resolve this issue.
        if (!isAdded) return;

        when ((childFragmentManager
            .findFragmentById(R.id.sub_header_field_fragment_container) != null)
                && (childFragmentManager
            .findFragmentById(R.id.sub_header_field_fragment_container)
                as HeaderFieldFragment).isVisible) {
            true -> {
                // Get data from HeaderFields fragment if exist
                val childHeaderFieldFragment = childFragmentManager
                    .findFragmentById(R.id.sub_header_field_fragment_container) as HeaderFieldFragment

                childHeaderFieldFragment.signalToFormSubmit(onPropertyRetrieveSubject)

                headerPropertyRetrieveDisposable()
                groupPropertyRetrieveDisposable()
                groupPropertyBuildByBarcodeDisposable()
            }
            false -> {
                var taskIdFromScan: String? = null
                arguments?.let { bundle ->
                    val args = DetailedTaskFragmentArgs.fromBundle(bundle)
                    args.nfcValue.let { nfcMark ->
                        currentNfcMark = nfcMark.toString()
                    }
                    args.detailedTaskModel.let { detailedTaskId ->
                        taskIdFromScan = detailedTaskId
                    }
                }

                if (taskIdFromScan != null && currentNfcMark != null) {
                    onTaskFinishClickSubject.onNext(Pair(currentNfcMark!!, taskIdFromScan!!))
                }
            }
        }

    }

    private fun headerPropertyRetrieveDisposable() {
        disposable.add(
            onPropertyRetrieveSubject
                .map { (propertyList, uriList) ->
                    when (propertyList.isNullOrEmpty()) {
                        true -> {
                            hideFinishedProgressbar()
                        }
                        false -> {
                            onTaskFinishWithPropsClickSubject.onNext(
                                Triple(
                                    getCurrentNfcMark()!!,
                                    propertyList,
                                    uriList
                                )
                            )
                        }
                    }
                }
                .subscribe()
        )
    }

    private fun groupPropertyRetrieveDisposable() {
        disposable.add(
            onGroupPropertyRetrieveSubject
                .doOnNext { (groupPropertyList, uriList) ->
                    childFragmentManager.fragments
                        .forEach { fragment ->
                            if (fragment is GroupPropertyFragment) {
                                if (!groupPropertyList.isNullOrEmpty()) {
                                    fragment.send_group_property_progressbar.visibility = GONE
                                    fragment.dismiss()
                                } else {
                                    fragment.hideButtonProgressbar()
                                }
                            }
                        }
                }
                .map { (groupPropertyList, uriList) ->
                    when (groupPropertyList.isNullOrEmpty()) {
                        true -> {
                            hideFinishedProgressbar()
                        }
                        false -> {
                            onSendGroupPropertySubject.onNext(
                                Pair(
                                    groupPropertyList,
                                    uriList
                                )
                            )
                        }
                    }
                }
                .subscribe()
        )
    }

    private fun groupPropertyBuildByBarcodeDisposable() {
        disposable.add(
            onBuildGroupPropertyWithoutMeasurements
                .observeOn(AndroidSchedulers.mainThread())
                .subscribeOn(AndroidSchedulers.mainThread())
                .doOnNext { (taskId, barcodeValue) ->
                    navigationComponent.buildGroupPropertyFragmentByBarcode(
                        childFragmentManager,
                        taskId,
                        barcodeValue
                    )
                }
                .subscribe()
        )
    }

    override fun onDestroy() {
        super.onDestroy()

        disposable.clear()
        arguments?.remove(KEY_INITIATE_NFC_VALUE_ARGUMENT)
    }

    fun loadGroupPropertyForm(groupType: TaskProperty?) {
        groupType?.let {
            onBuildGroupPropertyWithoutMeasurements.onNext(
                Pair(
                    groupType.taskId,
                    groupType.id
                )
            )
        }
    }

    companion object {
        const val KEY_TASK_ID = "taskId"
    }
}
