package ru.daotech.agroinvest.fragments.detailed_task

import ru.daotech.agroinvest.database.relation.TaskWithProperties

sealed class DetailedTaskPartialViewState {
    abstract fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState

    object Loading : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(isLoading = true)
        }
    }

    object GoingToMyTasks : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(errorMessage = null)
        }
    }


    object GoingToBarcodeScan : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(errorMessage = null)
        }
    }

    object GoingToTrolleyTiedPage : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState
        }
    }

    object EmptyState: DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState
        }
    }

    object ClearError : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(errorMessage = null)
        }
    }

    object SuccessGroupPropertyUpload : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(
                errorMessage = null,
                isGroupPropertyUploadSuccess = true
            )
        }
    }

    object DismissResponseDialogState : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(
                errorMessage = null,
                isGroupPropertyUploadSuccess = false
            )
        }
    }

    object DismissTaskStarted : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(
                isTaskStartedSuccess = false,
                taskWithProperties = null
            )
        }
    }

    object DismissTaskStates : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(
                isTaskStartedSuccess = false,
                isPreLaunchTaskState = false,
                isInProgressTaskState = false
            )
        }
    }

    object DismissSnackbarErrorStates : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(
                errorMessage = null,
                isLoading = false
            )
        }
    }

    data class SuccessLoadTasks(val taskWithProperties: TaskWithProperties) :
        DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(
                taskWithProperties = taskWithProperties,
                isPreLaunchTaskState = true
            )
        }
    }

    data class Error(val errorMessage: String) : DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(
                errorMessage = errorMessage,
                isLoading = false
            )
        }
    }

    data class SuccessStart(val taskWithProperties: TaskWithProperties) :
        DetailedTaskPartialViewState() {
        override fun reduce(previousState: DetailedTaskViewState): DetailedTaskViewState {
            return previousState.copy(
                taskWithProperties = taskWithProperties,
                isInProgressTaskState = true,
                isLoading = false,
                errorMessage = null
            )
        }
    }
}