package ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.jakewharton.rxbinding2.view.clicks
import io.reactivex.Observable
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.functions.BiFunction
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.fragment_detailed_processing_container.view.*
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.fragments.detailed_task.DetailedTaskFragment
import ru.daotech.agroinvest.utils.RAW_NFC_PREFIX
import ru.daotech.agroinvest.utils.ServiceUtil
import ru.daotech.agroinvest.utils.TROLLEY_NFC_PREFIX
import timber.log.Timber

class DetailedTaskProcessingTab :
    BaseMviFragment<
            DetailedTaskProcessingViewState,
            DetailedTaskProcessingTabView,
            DetailedTaskProcessingPresenter>(),
    DetailedTaskProcessingTabView {

    private val onErrorShown: PublishSubject<Unit> = PublishSubject.create()
    private val onFragmentChange: PublishSubject<Boolean> = PublishSubject.create()
    private val nfcScanSubject: PublishSubject<String> = PublishSubject.create()
    private val onCurrentTrolleySubject: PublishSubject<String> = PublishSubject.create()
    private val errorWrongNfcTypeSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onCurrentRawTaskStartSubject: PublishSubject<String> = PublishSubject.create()

    private var disposable = CompositeDisposable()

    companion object {
        fun newInstance(): DetailedTaskProcessingTab {
            return DetailedTaskProcessingTab()
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        retainInstance = true
        EventBus.getDefault().register(this)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(
            R.layout.fragment_detailed_processing_container,
            container,
            false
        )
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        disposable.add(Observable.combineLatest(
            view.nfc_scan_button.clicks(),
            nfcScanSubject,
            BiFunction<Unit, String, String> { _, nfcMessage ->
                nfcMessage
            }
        )
            .doOnError { error -> Timber.e(error) }
            .subscribe()
        )

        disposable.add(onCurrentRawTaskStartSubject
            .doOnNext { nfcTaskRowMark ->
                (this.parentFragment as DetailedTaskFragment).initiateTaskStart(nfcTaskRowMark)
            }
            .doOnError { Timber.e(it) }
            .subscribe()
        )
    }

    override fun onDestroy() {
        super.onDestroy()
        EventBus.getDefault().unregister(this)
        disposable.dispose()
    }

    override fun errorShown(): Observable<Unit> = onErrorShown

    override fun fragmentChange(): Observable<Boolean> = onFragmentChange

    override fun onCurrentTrolleyIntent(): Observable<String> = onCurrentTrolleySubject

    override fun errorWrongNfcTypeIntent(): Observable<Unit> = errorWrongNfcTypeSubject

    override fun render(state: DetailedTaskProcessingViewState) {

    }

    @Subscribe(threadMode = ThreadMode.MAIN_ORDERED)
    fun onNfcScanned(event: NfcScanned) {
        val nfcMessageValueArray = event.message.split('.')
        when {
            nfcMessageValueArray[0].toLowerCase() == TROLLEY_NFC_PREFIX -> {
                if((parentFragment as DetailedTaskFragment).isVisible) {
                    (parentFragment as DetailedTaskFragment).showTrolleyFragment(
                        ServiceUtil.cutOffExcessFromNfcMessage(event.message)
                    )
                }
            }
            nfcMessageValueArray[0].toLowerCase() == RAW_NFC_PREFIX -> {
                onCurrentRawTaskStartSubject.onNext(
                    ServiceUtil.cutOffExcessFromNfcMessage(event.message)
                )
            }
            else -> {
                errorWrongNfcTypeSubject.onNext(Unit)
            }
        }
    }
}

data class NfcScanned(val message: String)

