package ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.choose_group_container

import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.view.*
import android.view.View.GONE
import android.view.View.VISIBLE
import android.widget.ArrayAdapter
import android.widget.ListView
import androidx.core.content.ContextCompat
import androidx.drawerlayout.widget.DrawerLayout
import com.google.android.material.snackbar.Snackbar
import com.hannesdorfmann.mosby3.mvi.MviDialogFragment
import dagger.android.support.AndroidSupportInjection
import io.reactivex.Observable
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.dialog_fragment_choose_group_property.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.asListOfType
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.database.entities.TaskPlant
import ru.daotech.agroinvest.database.entities.TaskPlantImpl
import ru.daotech.agroinvest.database.entities.TaskProperty
import ru.daotech.agroinvest.fragments.detailed_task.DetailedTaskFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.group_properties_container.GroupPropertyFragment
import ru.daotech.agroinvest.utils.KEY_BARCODE_VALUE_GROUP_PROPERTY
import ru.daotech.agroinvest.utils.KEY_TASK_ID_GROUP_PROPERTY
import javax.inject.Inject

class ChooseGroupFragment :
    MviDialogFragment<ChooseGroupView, ChooseGroupPresenter>(), ChooseGroupView {

    companion object {
        fun newInstance(): ChooseGroupFragment {
            return ChooseGroupFragment()
        }
    }

    @Inject
    lateinit var accidentReportPresenter: ChooseGroupPresenter

    protected lateinit var snackbar: Snackbar

    private var disposable = CompositeDisposable()

    private val onErrorShownSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onDismissSnackbarSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onSendResultReport: PublishSubject<TaskProperty> = PublishSubject.create()
    private val onSaveCurrentTaskPlant: PublishSubject<TaskPlantImpl> = PublishSubject.create()

    private lateinit var groupTypeListView: ListView

    override fun onAttach(context: Context) {
        AndroidSupportInjection.inject(this)
        super.onAttach(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        (this.activity as ContainerActivity).drawer_layout
            ?.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED)

        setHasOptionsMenu(true)
        retainInstance = true

        (this.activity as ContainerActivity).actionBar?.setDisplayShowHomeEnabled(true)
        (this.activity as ContainerActivity).actionBar?.setDisplayHomeAsUpEnabled(true)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        val view = inflater.inflate(
            R.layout.dialog_fragment_choose_group_property,
            container,
            false
        )

        (activity as? AppBarController)?.apply {
            setupAppBarWithNavController()
            showAppBar()
        }

        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        dialog?.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
    }

    private fun setListViewHeightBasedOnChildren(listView: ListView) {
        val listAdapter = listView.adapter
            ?: return

        var totalHeight = 0
        var i = 0
        while (i < listAdapter.count) {
            val listItem = listAdapter.getView(i, null, listView)
            listItem.measure(0, 0)
            totalHeight += listItem.measuredHeight
            i++
        }

        val params = listView.layoutParams as ViewGroup.LayoutParams
        params.height = totalHeight + (listView.dividerHeight * (listAdapter.count - 1))
        listView.layoutParams = params
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        super.onPrepareOptionsMenu(menu)
        (menu.findItem(R.id.menu_search) as MenuItem).setVisible(false)
        (menu.findItem(R.id.menu_task_accident_report) as MenuItem).setVisible(false)
    }

    override fun render(state: ChooseGroupViewState) {
        manageErrors(state)

        if (state is ChooseGroupViewState.EmptyPlants) {
            empty_plants_placeholder.visibility = VISIBLE
            group_type_list.visibility = GONE
        }

        if (state is ChooseGroupViewState.GroupTypeLoaded) {
            empty_plants_placeholder.visibility = GONE
            group_type_list.visibility = VISIBLE

            buildListViewWithGroup(state.plantList, state.groupTypeList)
        }

        if (state is ChooseGroupViewState.ChoiceDoneState) {
            fragmentManager?.fragments?.forEach { fragment ->
                if (fragment !is GroupPropertyFragment) {
                    (parentFragment as DetailedTaskFragment).loadGroupPropertyForm(state.groupType)
                    dialog?.dismiss()
                    dismiss()
                    return
                }
            }
        }
    }

    private fun buildListViewWithGroup(vararg propertyList: List<Any>?) {
        if (propertyList[0].isNullOrEmpty()) {
            return
        }

        lateinit var objectiveGroupTitleList: List<String>

        val listOfGroupType =
            with(propertyList[0]!!) {
                when {
                    this.asListOfType<TaskPlant>() -> {
                        choose_group_title.text =
                            getString(R.string.plants_choose_group_dialog_title)
                        val resultPropertyTypeList = this as List<TaskPlant>
                        objectiveGroupTitleList = resultPropertyTypeList
                            .map { it.name }.toMutableList()
                        resultPropertyTypeList
                    }
                    this.asListOfType<TaskProperty>() -> {
                        choose_group_title.text =
                            getString(R.string.measurements_choose_group_dialog_title)
                        val resultPropertyTypeList = this as List<TaskProperty>
                        objectiveGroupTitleList = resultPropertyTypeList
                            .map { it.name }.toMutableList()
                        resultPropertyTypeList
                    }
                    else -> throw TypeCastException()
                }
            }

        val arrayAdapter: ArrayAdapter<String> = ArrayAdapter(
            requireContext(),
            R.layout.item_list_view_group_type,
            R.id.group_type_view,
            objectiveGroupTitleList
        )

        groupTypeListView = group_type_list
        groupTypeListView.adapter = arrayAdapter

        groupTypeListView.setOnItemClickListener { _, _, _, _ ->
            val selectedItemPosition = groupTypeListView.checkedItemPosition
            if (selectedItemPosition >= 0) {
                listOfGroupType[selectedItemPosition].apply {
                    if (this is TaskPlantImpl) {
                        // Open next choosing dialog
                        buildListViewWithGroup(propertyList[1])
                        onSaveCurrentTaskPlant.onNext(
                            propertyList[0]!![selectedItemPosition] as TaskPlantImpl
                        )
                    } else if (this is TaskProperty) {
                        onSendResultReport.onNext(this)
                    }
                }

            }
        }

        setListViewHeightBasedOnChildren(groupTypeListView)
    }

    private fun manageErrors(state: ChooseGroupViewState) {
        when (state) {
            is Error -> {
                showSnackbar(state.localizedMessage)
            }
            else -> {
            }
        }
    }

    override fun onDestroy() {
        disposable.clear()
        super.onDestroy()
    }

    override fun loadingIntent(): Observable<String> = Observable.just(Unit)
        .map { arguments?.getString(KEY_TASK_ID_GROUP_PROPERTY) }

    override fun onErrorShowIntent(): Observable<Unit> = onErrorShownSubject

    override fun dismissSnackbar(): Observable<Unit> = onDismissSnackbarSubject

    override fun onSaveCurrentTaskPlantIntent(): Observable<TaskPlantImpl> = onSaveCurrentTaskPlant

    override fun loadGroupPropertyFormIntent(): Observable<TaskProperty> = onSendResultReport

    override fun createPresenter(): ChooseGroupPresenter {
        return accidentReportPresenter
    }

    override fun onResume() {
        if (arguments?.size()!! > 0) {
            arguments?.apply {
                getString(KEY_BARCODE_VALUE_GROUP_PROPERTY)
            }
        }
        super.onResume()
    }

    private fun showSnackbar(message: String) {
        snackbar = Snackbar.make(
            requireActivity().findViewById(android.R.id.content)!!,
            message,
            Snackbar.LENGTH_SHORT
        )
        snackbar.view.setBackgroundColor(
            ContextCompat.getColor(
                requireContext(),
                R.color.accentSnackbarBackground
            )
        )
        snackbar.show()
    }
}