package ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.choose_group_container

import com.hannesdorfmann.mosby3.mvi.MviBasePresenter
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.business.Repository
import ru.daotech.agroinvest.utils.TASK_PROPERTY_GROUP_TYPE
import timber.log.Timber
import java.util.concurrent.TimeUnit

class ChooseGroupPresenter(
    private val appStateManager: AppStateManager,
    private val repository: Repository
) : MviBasePresenter<ChooseGroupView, ChooseGroupViewState>() {

    private lateinit var disposable: Disposable

    override fun bindIntents() {
        val loadingIntent = intent(ChooseGroupView::loadingIntent)
            .flatMap { taskId ->
                repository.getTaskWithProperties(taskId)
            }
            .map { taskWithProperties ->
                when (taskWithProperties.plants.isNullOrEmpty()) {
                    true -> {
                        ChooseGroupViewState.EmptyPlants
                    }
                    false -> {
                        ChooseGroupViewState.GroupTypeLoaded(
                            taskWithProperties.properties
                                .filter {
                                    it.type == TASK_PROPERTY_GROUP_TYPE
                                },
                            taskWithProperties.plants
                        )
                    }
                }
            }
            .cast(ChooseGroupViewState::class.java)

        val errorMessageIntent = intent(
            ChooseGroupView::onErrorShowIntent
        )
            .map { ChooseGroupViewState.Error }

        val onSaveCurrentTaskPlantIntent = intent(
            ChooseGroupView::onSaveCurrentTaskPlantIntent
        )
            .map { appStateManager.saveCurrentTaskPlant(it) }
            .map { ChooseGroupViewState.EmptyState }

        val snackbarDismissIntent = intent(
            ChooseGroupView::dismissSnackbar
        )
            .map { ChooseGroupViewState.EmptyState }

        val loadGroupPropertyFormIntent = intent(
            ChooseGroupView::loadGroupPropertyFormIntent
        )
            .map { ChooseGroupViewState.ChoiceDoneState(it) }

        val allIntentList = listOf(
            loadingIntent,
            errorMessageIntent,
            snackbarDismissIntent,
            loadGroupPropertyFormIntent,
            onSaveCurrentTaskPlantIntent
        )

        val allIntent = Observable.merge(
            allIntentList
        )
            .onErrorResumeNext { error: Throwable ->
                errorProcessing(error)
            }
            .observeOn(AndroidSchedulers.mainThread())
            .share()

        subscribeViewState(allIntent, ChooseGroupView::render)
    }

    private fun errorProcessing(
        error: Throwable
    ): Observable<ChooseGroupViewState> {
        Timber.e(error)
        return Observable.timer(3, TimeUnit.SECONDS)
            .map { Unit }
            .take(1)
            .map { ChooseGroupViewState.Error }
            .cast(ChooseGroupViewState::class.java)
    }

    override fun unbindIntents() {
        if (::disposable.isInitialized) {
            disposable.dispose()
        }
    }
}