package ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.group_properties_container

import android.app.Dialog
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.net.Uri
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.View.GONE
import android.view.View.VISIBLE
import android.view.ViewGroup
import androidx.fragment.app.DialogFragment
import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.android.material.bottomsheet.BottomSheetBehavior.*
import com.jakewharton.rxbinding2.view.clicks
import dagger.android.support.AndroidSupportInjection
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.subjects.BehaviorSubject
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.dialog_fragment_group_proprty.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.dialog.builder.GroupDialogBuilder
import ru.daotech.agroinvest.fragments.detailed_task.DetailedTaskFragment
import ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.header_field_container.HeaderFieldFragment
import ru.daotech.agroinvest.service.jsonmodels.TaskPropertyPayload
import ru.daotech.agroinvest.utils.HEADER_FIELD_FRAGMENT_TAG
import ru.daotech.agroinvest.utils.KEY_BARCODE_VALUE_GROUP_PROPERTY
import ru.daotech.agroinvest.utils.KEY_TASK_ID_GROUP_PROPERTY
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

class GroupPropertyFragment : DialogFragment() {

    @Inject
    lateinit var groupDialogBuilder: GroupDialogBuilder

    companion object {
        fun newInstance(): GroupPropertyFragment {
            return GroupPropertyFragment()
        }
    }

    override fun onAttach(context: Context) {
        AndroidSupportInjection.inject(this)
        super.onAttach(context)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.dialog_fragment_group_proprty, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        dialog?.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        group_property_general_container.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))

        val retrieveSubject =
            (parentFragment as DetailedTaskFragment).onGroupPropertyRetrieveSubject

        send_group_property_button.clicks()
            .throttleFirst(4, TimeUnit.SECONDS)
            .doOnNext {
//                groupDialogBuilder.buildBottomSheetAttentionDialog(
//                    { send_group_property_button.requestFocus() },
//                    { showButtonProgressbar() },
//                    { signalToFormSubmit(retrieveSubject) },
//                    retrieveSubject,
//                    context!!
//                )
                send_group_property_button.requestFocus()
                signalToFormSubmit(retrieveSubject)
            }
            .subscribe()

        arguments?.getString(KEY_TASK_ID_GROUP_PROPERTY)?.let { taskId ->
            arguments?.getString(KEY_BARCODE_VALUE_GROUP_PROPERTY)?.let { barcodeValue ->
                buildFormSubFragment(taskId, barcodeValue)
                return
            }
            buildFormSubFragment(taskId, null)
        }
    }

    override fun onStart() {
        super.onStart()
        val dialog = dialog
        if (dialog != null) {
            val width = ViewGroup.LayoutParams.MATCH_PARENT
            val height = ViewGroup.LayoutParams.MATCH_PARENT
            dialog.window?.setLayout(width, height)
        }
    }

    private fun buildFormSubFragment(taskId: String, barcodeValue: String?) {
        val fragmentManager = childFragmentManager
        val fragmentTransaction = fragmentManager.beginTransaction()

        sub_group_property_fragment_container.visibility = VISIBLE

        val headerFieldFragment = HeaderFieldFragment.newInstance()
        val arguments = Bundle()
        fragmentTransaction.setCustomAnimations(
            R.anim.enter_from_right,
            R.anim.exit_to_left,
            R.anim.enter_from_left,
            R.anim.exit_to_right
        )
        arguments.putString(KEY_TASK_ID_GROUP_PROPERTY, taskId)
        barcodeValue?.let {
            arguments.putString(KEY_BARCODE_VALUE_GROUP_PROPERTY, barcodeValue)
        }
        headerFieldFragment.arguments = arguments
        headerFieldFragment.retainInstance = true
        fragmentTransaction.replace(
            R.id.sub_group_property_fragment_container,
            headerFieldFragment,
            HEADER_FIELD_FRAGMENT_TAG
        )
        fragmentTransaction.commit()
    }

    private fun signalToFormSubmit(
        onSendGroupPropertySubject: BehaviorSubject<Pair<List<TaskPropertyPayload?>, List<Uri>>>
    ) {
        val fragment: HeaderFieldFragment =
            childFragmentManager.fragments[0] as HeaderFieldFragment
        fragment.signalToFormSubmit(onSendGroupPropertySubject)
    }

    private fun showButtonProgressbar() {
        send_group_property_label.visibility = GONE
        send_group_property_progressbar.visibility = VISIBLE
        send_group_property_button.isEnabled = false
    }

    fun hideButtonProgressbar() {
        send_group_property_label.visibility = VISIBLE
        send_group_property_progressbar.visibility = GONE
        send_group_property_button.isEnabled = true
    }

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val bottomSheetDialog = super.onCreateDialog(savedInstanceState)
        bottomSheetDialog.setOnShowListener { dialogInterface ->
            //            val sheetDialog = dialogInterface as? BottomSheetDialog

            val bottomSheetBehavior = from(design_bottom_sheet_container)
//            bottomSheetBehavior.skipCollapsed = false
            bottomSheetBehavior.isFitToContents = false
            bottomSheetBehavior.state = STATE_EXPANDED
            bottomSheetBehavior.setBottomSheetCallback(object :
                BottomSheetBehavior.BottomSheetCallback() {
                override fun onStateChanged(bottomSheet: View, newState: Int) {
                    // React to state change
                    when (newState) {
                        STATE_HIDDEN -> {
                            bottomSheetBehavior.state = STATE_HIDDEN
                        }
                        STATE_EXPANDED -> {
                            bottomSheetBehavior.state = STATE_EXPANDED
                            close_dialog_bottom_sheet.visibility = GONE
                        }
                        STATE_HALF_EXPANDED -> {
                            bottomSheetBehavior.state = STATE_HALF_EXPANDED

                            close_dialog_bottom_sheet.animate()
                                .alpha(0.0f)
                                .withEndAction {
                                    close_dialog_bottom_sheet.visibility = GONE
                                }
                                .duration = 400
                        }
                        STATE_COLLAPSED -> {
                            bottomSheetBehavior.state = STATE_COLLAPSED

                            close_dialog_bottom_sheet.visibility = VISIBLE

                            close_dialog_bottom_sheet.animate()
                                .alpha(1.0f).duration = 500

                            close_dialog_bottom_sheet.clicks()
                                .doOnNext { bottomSheetClose() }
                                .doOnError { Timber.e(it.localizedMessage) }
                                .subscribe()

                            Timber.i("STATE_COLLAPSED")
                        }
                        STATE_DRAGGING -> Timber.i("STATE_DRAGGING")
                        STATE_SETTLING -> Timber.i("STATE_SETTLING")
                    }
                }

                override fun onSlide(bottomSheet: View, slideOffset: Float) {
                    // React on dragging events
                }
            })

        }
        return bottomSheetDialog
    }

    private fun bottomSheetClose() {
        onDestroyView()
    }
}