package ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.header_field_container

import android.os.Bundle
import com.hannesdorfmann.mosby3.mvi.MviBasePresenter
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.business.Repository
import ru.daotech.agroinvest.utils.KEY_TASK_ID_GROUP_PROPERTY
import ru.daotech.agroinvest.utils.KEY_TASK_ID_HEADER_FIELD
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

class HeaderFieldPresenter @Inject constructor(
    private val repository: Repository,
    private val appStateManager: AppStateManager
) : MviBasePresenter<HeaderFieldView, HeaderFieldViewState>() {

    override fun bindIntents() {

        val loadingIntent = intent(HeaderFieldView::loadingIntent)
            .flatMap { arguments ->
                buildDynamicForm(arguments)
            }
            .onErrorReturn {
                Timber.e(it)
                HeaderFieldViewState.Error
            }
            .startWith(HeaderFieldViewState.Loading)


        val loadingGroupPropertyIntent = intent(HeaderFieldView::loadingGroupPropertiesIntent)
            .flatMap { taskId ->
                repository.getTaskWithProperties(taskId)
            }
            .map { taskWithProperties ->
                HeaderFieldViewState.BuildGroupPropertyForm(taskWithProperties) as HeaderFieldViewState
            }
            .onErrorReturn {
                Timber.e(it)
                HeaderFieldViewState.Error
            }
            .startWith(HeaderFieldViewState.Loading)

        val dismissSnackbarIntent = intent(HeaderFieldView::dismissSnackbar)
            .map {
                HeaderFieldViewState.DismissSnackbar as HeaderFieldViewState
            }

        val allIntent = Observable.merge(
            loadingIntent,
            loadingGroupPropertyIntent,
            dismissSnackbarIntent
        )
            .onErrorResumeNext { error: Throwable ->
                errorProcessing(error)
            }
            .observeOn(AndroidSchedulers.mainThread())
            .share()

        subscribeViewState(allIntent, HeaderFieldView::render)
    }

    private fun buildDynamicForm(arguments: Bundle): Observable<HeaderFieldViewState> {
        return if (arguments.getString(KEY_TASK_ID_HEADER_FIELD).isNullOrBlank()) {
            repository.getTaskWithProperties(
                arguments.getString(
                    KEY_TASK_ID_GROUP_PROPERTY
                )
                    ?.let { taskId ->
                        return@let taskId
                    }!!
            )
                .map { taskWithProperties ->
                    HeaderFieldViewState.BuildGroupPropertyForm(taskWithProperties)
                            as HeaderFieldViewState
                }
        } else {
            repository.getTaskWithProperties(
                arguments.getString(
                    KEY_TASK_ID_HEADER_FIELD
                )
                    ?.let { taskId ->
                        return@let taskId
                    }!!
            )
                .map { taskWithProperties ->
                    HeaderFieldViewState.BuildForm(taskWithProperties)
                            as HeaderFieldViewState
                }
        }
    }

    private fun errorProcessing(
        error: Throwable
    ): Observable<HeaderFieldViewState> {
        Timber.e(error)
        return Observable.timer(3, TimeUnit.SECONDS)
            .map { Unit }
            .take(1)
            .map {
                HeaderFieldViewState.Error as HeaderFieldViewState
            }
            .startWith(HeaderFieldViewState.Error)
    }
}