package ru.daotech.agroinvest.fragments.detailed_task.detailed_task_in_progress.timer_container

sealed class TimerContainerPartialViewState {
    abstract fun reduce(previousState: TimerContainerViewState): TimerContainerViewState

    object TaskStarted : TimerContainerPartialViewState() {
        override fun reduce(previousState: TimerContainerViewState): TimerContainerViewState {
            return previousState.copy(
                isStarted = true,
                errorMessage = null
            )
        }
    }

    object TaskErrorWithoutMessage : TimerContainerPartialViewState() {
        override fun reduce(previousState: TimerContainerViewState): TimerContainerViewState {
            return previousState.copy(errorMessage = null)
        }
    }

    object DismissTimerChangedState : TimerContainerPartialViewState() {
        override fun reduce(previousState: TimerContainerViewState): TimerContainerViewState {
            return previousState.copy(
                isStateChanged = false,
                errorMessage = null
            )
        }
    }

    object EmptyState : TimerContainerPartialViewState() {
        override fun reduce(previousState: TimerContainerViewState): TimerContainerViewState {
            return previousState
        }
    }

    data class FragmentStateSaving(val taskId: String, val taskInterval: Long) : TimerContainerPartialViewState() {
        override fun reduce(previousState: TimerContainerViewState): TimerContainerViewState {
            return previousState.copy(
                currentTaskId = taskId,
                currentTaskInterval = taskInterval,
                errorMessage = null
            )
        }
    }

    data class TaskErrorWithMessage(val errorMessage: String) :
        TimerContainerPartialViewState() {
        override fun reduce(previousState: TimerContainerViewState): TimerContainerViewState {
            return previousState.copy(
                errorMessage = errorMessage,
                isTimerRunState = false,
                isStateChanged = false
            )
        }
    }

    data class DisablePauseForThisTaskState(val errorMessage: String) :
        TimerContainerPartialViewState() {
        override fun reduce(previousState: TimerContainerViewState): TimerContainerViewState {
            return previousState.copy(
                errorMessage = errorMessage,
                isTimerRunState = false,
                isStateChanged = true
            )
        }
    }

    data class FixTaskTimerState(val isTimerRunState: Boolean) :
        TimerContainerPartialViewState() {
        override fun reduce(previousState: TimerContainerViewState): TimerContainerViewState {
            return previousState.copy(
                isTimerRunState = isTimerRunState,
                isStateChanged = true,
                errorMessage = null
            )
        }
    }

    data class GoToTrolleyTied(val nfcTrolleyMark: String?) :
        TimerContainerPartialViewState() {
        override fun reduce(previousState: TimerContainerViewState): TimerContainerViewState {
            return previousState.copy(
                nfcTrolleyMark = nfcTrolleyMark,
                errorMessage = null
            )
        }
    }
}