package ru.daotech.agroinvest.fragments.earnings

import android.os.Bundle
import android.text.Html
import android.view.*
import io.reactivex.Observable
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_earnings.*
import kotlinx.android.synthetic.main.fragment_welcome.department_text
import kotlinx.android.synthetic.main.fragment_welcome.personal_number_text
import kotlinx.android.synthetic.main.fragment_welcome.position_text
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.fragments.ratings.EarningsView
import ru.daotech.agroinvest.utils.ViewUtil
import java.text.DecimalFormat


class EarningsFragment : BaseMviFragment<EarningsViewState, EarningsView, EarningsPresenter>(),
    EarningsView {

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        (activity as? AppBarController)?.apply {
            setupAppBarWithNavController()
            showAppBar()
        }
        val view = inflater.inflate(R.layout.fragment_earnings, container, false)

        return view
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)
        retainInstance = true
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        super.onPrepareOptionsMenu(menu)
        (menu.findItem(R.id.menu_search) as MenuItem).isVisible = false

        activity?.let { activity ->
            ViewUtil.manageInternetConnectState(activity, menu)
            ViewUtil.manageSocketConnectState(activity, menu)
        }
    }

    override fun loadingIntent(): Observable<Unit> = Observable.just(Unit)

    override fun snackbarDismiss(): Observable<Unit> = snackbarDismiss

    override fun render(state: EarningsViewState) {
        manageErrors(state)

        with(state) {
            if (isLoading) {
                (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.tasks_loading)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
            }

            worker?.let { worker ->
                (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.earnings_title)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle =
                    getString(R.string.menu_in_progress)

                update_app_version_title.text = worker.name
                personal_number_text.text =
                    getString(R.string.personal_number, worker.personalNumber)
                position_text.text = worker.positionName

                worker.department?.let {
                    department_text.text = it.fullName
                }

                val monthEarned: String = DecimalFormat("#,###")
                    .format(worker.earnings!!["monthEarned"]?.toInt())
                earned_this_month_amount.text = Html.fromHtml(
                    "<font color='#666666'>\u20BD</font> " +
                            monthEarned +
                            "<small><font color='#666666'>.00</font></small>"
                )

                val monthForecast: String = DecimalFormat("#,###")
                    .format(worker.earnings["monthForecast"]?.toInt())
                will_earn_this_month_amount.text =
                    Html.fromHtml("<font color='#666666'>\u20BD</font> $monthForecast<small><font color='#666666'>.00</font></small>")

                tasks_today.text = tasksToday.toString()
                tasks_yesterday.text = tasksYesterday.toString()
            }
        }
    }

    // On view handling errors
    private fun manageErrors(state: EarningsViewState) {
        with(state) {
            when {
                errorMessage != null -> {
                    showSnackbar(errorMessage)
                }
                else -> {
                    dismissSnackbar()
                }
            }
        }
    }
}