package ru.daotech.agroinvest.fragments.earnings

import ru.daotech.agroinvest.models.Worker

sealed class EarningsPartialViewState {
    abstract fun reduce(previousState: EarningsViewState): EarningsViewState

    object Dummy : EarningsPartialViewState() {
        override fun reduce(previousState: EarningsViewState): EarningsViewState {
            return previousState.copy()
        }
    }

    object Loading : EarningsPartialViewState() {
        override fun reduce(previousState: EarningsViewState): EarningsViewState {
            return previousState.copy(
                isLoading = true,
                errorMessage = null
            )
        }
    }

    data class SuccessLoadEarnings(val worker: Worker, val tasksToday: Int, val tasksYesterday: Int) : EarningsPartialViewState() {
        override fun reduce(previousState: EarningsViewState): EarningsViewState {
            return previousState.copy(
                isLoading = false,
                tasksToday = tasksToday,
                tasksYesterday = tasksYesterday,
                worker = worker
            )
        }
    }

    object ClearError : EarningsPartialViewState() {
        override fun reduce(previousState: EarningsViewState): EarningsViewState {
            return previousState.copy(
                errorMessage = null
            )
        }
    }

    data class Error(val errorMessage: String) : EarningsPartialViewState() {
        override fun reduce(previousState: EarningsViewState): EarningsViewState {
            return previousState.copy(errorMessage = errorMessage)
        }
    }
}