package ru.daotech.agroinvest.fragments.login

import android.Manifest
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Environment
import android.os.Handler
import android.view.*
import android.view.View.GONE
import android.view.View.VISIBLE
import android.view.inputmethod.InputMethodManager
import android.webkit.MimeTypeMap
import android.widget.ArrayAdapter
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.core.content.FileProvider
import androidx.core.os.postDelayed
import androidx.fragment.app.Fragment
import com.google.android.material.snackbar.Snackbar
import com.hannesdorfmann.mosby3.mvi.MviFragment
import com.jakewharton.rxbinding2.view.clicks
import dagger.android.support.AndroidSupportInjection
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_login.*
import kotlinx.android.synthetic.main.fragment_login.view.*
import ru.daotech.agroinvest.BuildConfig
import ru.daotech.agroinvest.BuildConfig.isSocketIOEnable
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.database.entities.RecentUser
import ru.daotech.agroinvest.dialog.builder.LoginDialogBuilder
import ru.daotech.agroinvest.fragments.pages.update.UpdateAppFragment
import ru.daotech.agroinvest.hide
import ru.daotech.agroinvest.show
import ru.daotech.agroinvest.utils.EXTERNAL_READ_PERMISSION_CODE
import ru.daotech.agroinvest.utils.EXTERNAL_WRITE_PERMISSION_CODE
import ru.daotech.agroinvest.utils.GPS_PERMISSION_CODE
import ru.daotech.agroinvest.utils.ViewUtil
import timber.log.Timber
import java.io.File
import java.util.concurrent.TimeUnit
import javax.inject.Inject

const val SIZE_OF_BARCODE = 4

class LoginFragment : MviFragment<LoginView, LoginPresenter>(), LoginView {

    @Inject
    lateinit var loginPresenter: LoginPresenter

    @Inject
    lateinit var loginDialogBuilder: LoginDialogBuilder

    private var disposable = CompositeDisposable()

    private var isArgumentChecked: Boolean = false
    private lateinit var downloadFolder: String

    private val onBarcodeRetrieve: PublishSubject<String> = PublishSubject.create()
    private val onLoginInitiate: PublishSubject<Unit> = PublishSubject.create()
    private val onErrorShown: PublishSubject<Unit> = PublishSubject.create()
    private val onUpdateAppInitiateSubject: PublishSubject<String> = PublishSubject.create()
    private val onRecentUsersRetrieve: PublishSubject<Unit> = PublishSubject.create()

    override fun onAttach(context: Context) {
        AndroidSupportInjection.inject(this)
        super.onAttach(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        downloadFolder =
            Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS)
                .absolutePath
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        (activity as? AppBarController)?.apply {
            showAppBar()
        }

        val view = inflater.inflate(R.layout.fragment_login, container, false)

        with(view) {

            // TODO: Temporary comment logic
//            barcode_input.addTextChangedListener(object : TextWatcher {
//                override fun afterTextChanged(s: Editable) {}
//
//                override fun beforeTextChanged(
//                    s: CharSequence,
//                    start: Int,
//                    count: Int,
//                    after: Int
//                ) {
//                }
//
//                override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
//                    when (s.length) {
//                        SIZE_OF_BARCODE -> {
//                            onLoginInitiate.onNext(Unit)
//                        }
//                    }
//                }
//            })

            proceed_button.clicks()
                .doOnNext { onLoginInitiate.onNext(Unit) }
                .subscribe()

            login_fragment_upgrade_app_button.clicks()
                .debounce(1000, TimeUnit.MILLISECONDS)
                .doOnNext {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                        onUpdateAppInitiateSubject.onNext(downloadFolder)
                    } else {
                        if (ContextCompat.checkSelfPermission(
                                requireContext(),
                                Manifest.permission.WRITE_EXTERNAL_STORAGE
                            )
                            != PackageManager.PERMISSION_GRANTED
                        ) {

                            // Permission is not granted
                            // Should we show an explanation?
                            if (shouldShowRequestPermissionRationale(
                                    Manifest.permission.WRITE_EXTERNAL_STORAGE
                                )
                            ) {
                                // Show an explanation to the user *asynchronously* -- don't block
                                // this thread waiting for the user's response! After the user
                                // sees the explanation, try again to request the permission.
                            } else {
                                // No explanation needed, we can request the permission.
                                requestPermissions(
                                    arrayOf(
                                        Manifest.permission.WRITE_EXTERNAL_STORAGE,
                                        Manifest.permission.READ_EXTERNAL_STORAGE
                                    ),
                                    UpdateAppFragment.REQUEST_PERMISSIONS_CODE_WRITE_STORAGE
                                )

                                // MY_PERMISSIONS_REQUEST_READ_CONTACTS is an
                                // app-defined int constant. The callback method gets the
                                // result of the request.
                            }
                        } else {
                            // Permission has already been granted
                            onUpdateAppInitiateSubject.onNext(downloadFolder)
                        }
                    }
                }
                .subscribe()

            disposable.add(onLoginInitiate
                .doOnNext {
                    hideKeyboard()
                    barcode_input.isEnabled = false
                }
                .map { barcode_input.text.toString() }
                .subscribe(
                    { barcode ->
                        if (barcode.isEmpty()) {
                            val handler = Handler()
                            barcode_input.error = getString(R.string.barcode_cannot_empty)
                            handler.postDelayed(1500) { barcode_input.error = null }

                            enableBarcodeField()
                        } else {
                            onBarcodeRetrieve.onNext(barcode)
                        }
                    },
                    { error ->
                        enableBarcodeField()
                        disposable.clear()
                        Timber.e(error, error.localizedMessage)
                    }
                )
            )
        }

        // Handle onBack device button
        handleOnBackPressed()

        return view
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        super.onPrepareOptionsMenu(menu)
        (menu.findItem(R.id.menu_search) as MenuItem).isVisible = false

        activity?.let { activity ->
            ViewUtil.manageInternetConnectState(activity, menu)
            ViewUtil.manageSocketConnectState(activity, menu)
        }
    }

    override fun onResume() {
        if (!isArgumentChecked) {
            arguments?.let { bundle ->
                val args = LoginFragmentArgs.fromBundle(bundle)
                args.retrievedBarcode?.let { barcode ->
                    barcode_input.setText(barcode)
                    onBarcodeRetrieve.onNext(barcode)
                }
            }
            isArgumentChecked = true
        }
        super.onResume()
    }

    override fun barcodeRetrieved(): Observable<String> = onBarcodeRetrieve

    override fun onUpdateAppInitiateIntent(): Observable<String> = onUpdateAppInitiateSubject

    override fun scanButtonClick(): Observable<Unit> = barcode_scan_fab.clicks()

    override fun errorShown(): Observable<Unit> = onErrorShown

    override fun loadingIntent(): Observable<Unit> = Observable.just(Unit)
        .doOnNext { checkWriteReadGalleryPermissions() }
        .doOnNext { checkGPSMarks() }

    override fun loadRecentUsersIntent(): Observable<Unit> = Observable.just(Unit)

    override fun recentUsersRetrieved(): Observable<Unit> = onRecentUsersRetrieve

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>, grantResults: IntArray
    ) {
        when (requestCode) {
            UpdateAppFragment.REQUEST_PERMISSIONS_CODE_WRITE_STORAGE -> {
                // If request is cancelled, the result arrays are empty.
                if ((grantResults.isNotEmpty() && grantResults[0]
                            == PackageManager.PERMISSION_GRANTED)
                ) {
                    onUpdateAppInitiateSubject.onNext(downloadFolder)
                    // permission was granted, yay! Do the
                    // contacts-related task you need to do.
                } else {
                    Timber.d(
                        "${javaClass.name}: Permissions denied! Request them before file uploading."
                    )
                    // permission denied, boo! Disable the
                    // functionality that depends on this permission.
                }
                return
            }
            // Add other 'when' lines to check for other
            // permissions this app might request.
            else -> {
                // Ignore all other requests.
            }
        }
    }

    private fun checkWriteReadGalleryPermissions() {
        if (
            (ContextCompat.checkSelfPermission(
                requireContext(),
                android.Manifest.permission.READ_EXTERNAL_STORAGE
            ) == PackageManager.PERMISSION_DENIED)
        )
            ActivityCompat.requestPermissions(
                requireActivity(),
                arrayOf(android.Manifest.permission.READ_EXTERNAL_STORAGE),
                EXTERNAL_READ_PERMISSION_CODE
            )
        if (
            (ContextCompat.checkSelfPermission(
                requireContext(),
                android.Manifest.permission.WRITE_EXTERNAL_STORAGE
            ) == PackageManager.PERMISSION_DENIED)
        )
            ActivityCompat.requestPermissions(
                requireActivity(),
                arrayOf(android.Manifest.permission.WRITE_EXTERNAL_STORAGE),
                EXTERNAL_WRITE_PERMISSION_CODE
            )
    }

    private fun checkGPSMarks() {
        if (
            (ContextCompat.checkSelfPermission(
                requireContext(),
                android.Manifest.permission.ACCESS_COARSE_LOCATION
            ) == PackageManager.PERMISSION_DENIED)
        )
            ActivityCompat.requestPermissions(
                requireActivity(),
                arrayOf(android.Manifest.permission.ACCESS_COARSE_LOCATION),
                GPS_PERMISSION_CODE
            )
    }

    override fun createPresenter(): LoginPresenter {
        return loginPresenter
    }

    private fun enableBarcodeField() {
        barcode_input.clearComposingText()
//        barcode_input.text = null
        view?.barcode_input?.isEnabled = true
        barcode_input.requestFocus()
    }

    private fun Fragment.hideKeyboard() {
        val imm = context?.getSystemService(Context.INPUT_METHOD_SERVICE)
                as InputMethodManager
        imm.hideSoftInputFromWindow(this.view?.windowToken, 0)
    }

    override fun render(state: LoginViewState) {
        manageErrors(state)

        (activity as? AppBarController)?.hideOnBackAppBarItem()
        (activity as ContainerActivity).activity_container_custom_toolbar?.title = null
        (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null

        state.apply {

            if (isLoginDisable) {
                proceed_button.hide()
                barcode_input.isEnabled = false
            } else {
                proceed_button.show()
                barcode_input.isEnabled = true
            }

            appFile?.let { initiateUpdateAppInstallation(it) }

            if (isUpdateAppEnable) {
                login_fragment_upgrade_app_button.show()

                progressButtonLabel?.let {
                    login_fragment_update_app_progress_label.text = it
                }
            } else {
                login_fragment_upgrade_app_button.hide()
            }

            if (isLoading) {
                progressbar.show()
                proceed_label.hide()
                proceed_button.isEnabled = false

                view?.barcode_input?.isEnabled = false
            } else {
                progressbar.hide()
                proceed_label.show()
                proceed_button.isEnabled = true

                enableBarcodeField()

                loginDialogBuilder.hideLoginQueueDialog()
            }

            if (isUpdateAppLoading) {
                login_fragment_upgrade_app_button_progressbar.show()
                login_fragment_upgrade_app_button_label.hide()
                login_fragment_upgrade_app_button.isEnabled = false
            } else {
                login_fragment_upgrade_app_button_progressbar.hide()
                login_fragment_upgrade_app_button_label.show()
                login_fragment_upgrade_app_button.isEnabled = true
            }

            if (isSuccess) {
                if (isSocketIOEnable != null && isSocketIOEnable) {
                    (activity as ContainerActivity).startSocketService()
                }
                loginDialogBuilder.hideLoginQueueDialog()
            }

            if (!appInfoValue.isNullOrEmpty()) {
                login_fragment_app_info_text_view.visibility = VISIBLE
                login_fragment_app_info_text_view.text = appInfoValue
            } else {
                login_fragment_app_info_text_view.visibility = GONE
            }

            queueResultMessage?.let { message ->
                loginDialogBuilder.buildLoginQueueDialog(
                    requireContext(),
                    message
                )
            }

            if (recentUsers.isNotEmpty()) {
                setupRecentUsersMenu(recentUsers)
            }
        }
    }

    private fun setupRecentUsersMenu(users: List<RecentUser>) {
        val recentBarcode = users[0].personalNumber
        val items = users.map { "${it.personalNumber} ${it.workerName}" }
        val adapter = ArrayAdapter<String>(
            requireContext(),
            android.R.layout.simple_dropdown_item_1line,
            items
        )

        barcode_input.apply {
            setText(recentBarcode)
            setSelection(recentBarcode.length)
            setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.menu_down, 0)
            setAdapter(adapter)

            setOnItemClickListener { parent, _, position, _ ->
                val selected = parent.getItemAtPosition(position) as String
                val pos: Int = items.indexOf(selected)
                val barcode = users[pos].personalNumber
                barcode_input.apply {
                    setText(barcode)
                    setSelection(barcode.length)
                }
            }

            setOnTouchListener(View.OnTouchListener { v, event ->
                val DRAWABLE_RIGHT_CODE = 2

                if (event.action == MotionEvent.ACTION_UP) {
                    if (event.rawX
                        >= (barcode_input
                            .right - barcode_input
                            .compoundDrawables[DRAWABLE_RIGHT_CODE]
                            .bounds.width())
                    ) {
                        barcode_input.showDropDown()
                        v.performClick()
                        return@OnTouchListener true
                    }
                }
                return@OnTouchListener false
            })
        }

        onRecentUsersRetrieve.onNext(Unit)
    }

    private fun initiateUpdateAppInstallation(appFile: File) {
        Observable.just(Unit)
            .take(1)
            .subscribeOn(AndroidSchedulers.mainThread())
            .map {
                val intent = Intent()
                intent.flags = Intent.FLAG_ACTIVITY_CLEAR_TOP

                val uri: Uri?
                var mime: String? =
                    "application/vnd.android.package-archive" // Default Value

                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                    intent.action = Intent.ACTION_INSTALL_PACKAGE

                    uri = FileProvider.getUriForFile(
                        requireContext(),
                        BuildConfig.APPLICATION_ID + ".fileprovider",
                        appFile
                    )

                    if (uri != null) {
                        mime = requireContext().contentResolver.getType(uri)
                    }
                } else {
                    intent.action = Intent.ACTION_VIEW
                    intent.putExtra(Intent.EXTRA_NOT_UNKNOWN_SOURCE, true)
                    intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK

                    appFile.setReadable(true)
                    uri = Uri.fromFile(appFile)

                    mime =
                        MimeTypeMap.getSingleton()
                            .getMimeTypeFromExtension(appFile.extension)
                }

                intent.setDataAndType(uri, mime)
                intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)

                startActivity(intent)
            }
            .onErrorReturn {
                showLocalSnackbar(
                    ContextCompat.getColor(requireContext(), android.R.color.black),
                    getString(R.string.upgrade_app_error_message)
                )
                return@onErrorReturn
            }
            .doOnError { Timber.e(it.localizedMessage) }
            .subscribe()
    }

    private fun manageErrors(state: LoginViewState) {
        state.errorMessage?.let { errorMessage ->
            val snackbarBackgroundColor = if (state.isFailedAttempt && state.failedAttempts > 2) {
                ContextCompat.getColor(requireContext(), R.color.accentSnackbarBackground)
            } else {
                ContextCompat.getColor(requireContext(), android.R.color.black)
            }
            showLocalSnackbar(snackbarBackgroundColor, errorMessage)

            // Hide login queue dialog if it is exist
            loginDialogBuilder.hideLoginQueueDialog()
        }
    }

    private fun showLocalSnackbar(color: Int, message: String) {
        val snackbar = Snackbar.make(requireView(), message, Snackbar.LENGTH_SHORT)
        snackbar.view.setBackgroundColor(color)
        snackbar.addCallback(object : Snackbar.Callback() {
            override fun onShown(sb: Snackbar?) {
                super.onShown(sb)
                onErrorShown.onNext(Unit)
            }
        })
        snackbar.show()
    }

    private fun handleOnBackPressed() {
        this.hideKeyboard()

        requireActivity().drawer_layout?.setOnKeyListener { _, keyCode, event ->
            if ((keyCode == KeyEvent.KEYCODE_BACK)
                && event.action == KeyEvent.ACTION_DOWN
            ) {
                if (this.isVisible) {
                    this.hideKeyboard()
                }
            }
            false
        }
    }

    override fun onDestroyView() {
        disposable.dispose()
        super.onDestroyView()
    }
}