package ru.daotech.agroinvest.fragments.pages.about

import com.hannesdorfmann.mosby3.mvi.MviBasePresenter
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.business.Repository
import timber.log.Timber
import java.util.concurrent.TimeUnit

class AboutPagePresenter(
    private val appStateManager: AppStateManager,
    private val repository: Repository
) :
    MviBasePresenter<AboutPageView, AboutPageViewState>() {

    private lateinit var disposable: Disposable

    override fun bindIntents() {
        val loadingIntent = intent(AboutPageView::loadingIntent)
            .onErrorReturn {
                Timber.e(it)
                AboutPageViewState.Error
            }
            .map { AboutPageViewState.EmptyState }

        val uploadFile = intent(AboutPageView::fileUpload)
            .flatMap { files -> repository.uploadDatabaseFileDump(files) }
            .map { AboutPageViewState.UploadSuccess as AboutPageViewState }
            .onErrorReturn {
                Timber.e(it)
                AboutPageViewState.UploadFailed
            }
            .startWith(AboutPageViewState.Loading)

        val allIntent = Observable.merge(
            loadingIntent,
            uploadFile
        )
            .onErrorResumeNext { error: Throwable ->
                errorProcessing(error)
            }
            .observeOn(AndroidSchedulers.mainThread())
            .share()

        subscribeViewState(allIntent, AboutPageView::render)
    }

    private fun errorProcessing(
        error: Throwable
    ): Observable<AboutPageViewState> {
        Timber.e(error)
        return Observable.timer(3, TimeUnit.SECONDS)
            .map { Unit }
            .take(1)
            .map {
                AboutPageViewState.Error as AboutPageViewState
            }
            .startWith(AboutPageViewState.Error)
    }

    override fun unbindIntents() {
    }
}