package ru.daotech.agroinvest.fragments.pages.update

import ru.daotech.agroinvest.models.AppInfo
import java.io.File

sealed class UpdateAppPartialViewState {
    abstract fun reduce(previousState: UpdateAppViewState): UpdateAppViewState

    data class ShowAppInfo(val appInfo: AppInfo) : UpdateAppPartialViewState() {
        override fun reduce(previousState: UpdateAppViewState): UpdateAppViewState {
            return previousState.copy(appInfo = appInfo, errorMessage = null)
        }
    }

    data class ShowUpdateButton(val appFile: File) : UpdateAppPartialViewState() {
        override fun reduce(previousState: UpdateAppViewState): UpdateAppViewState {
            return previousState.copy(appFile = appFile, errorMessage = null)
        }
    }

    data class Error(val errorMessage: String) : UpdateAppPartialViewState() {
        override fun reduce(previousState: UpdateAppViewState): UpdateAppViewState {
            return previousState.copy(errorMessage = errorMessage)
        }
    }

    data class ShowDownloadProgressValue(val progressValue: Int) : UpdateAppPartialViewState() {
        override fun reduce(previousState: UpdateAppViewState): UpdateAppViewState {
            return previousState.copy(progress = progressValue, errorMessage = null)
        }
    }

    object ShowNextScreen : UpdateAppPartialViewState() {
        override fun reduce(previousState: UpdateAppViewState): UpdateAppViewState {
            return previousState
        }
    }

    object ClearError : UpdateAppPartialViewState() {
        override fun reduce(previousState: UpdateAppViewState): UpdateAppViewState {
            return previousState.copy(errorMessage = null)
        }
    }
}