package ru.daotech.agroinvest.fragments.ratings

import android.os.Bundle
import android.view.*
import androidx.recyclerview.widget.LinearLayoutManager
import io.reactivex.Observable
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_ratings.*
import kotlinx.android.synthetic.main.fragment_ratings.view.*
import ru.daotech.agroinvest.*
import ru.daotech.agroinvest.adapters.RatingAdapter
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.utils.ViewUtil
import javax.inject.Inject

class RatingsFragment : BaseMviFragment<RatingsViewState, RatingsView, RatingsPresenter>(),
    RatingsView {

    @Inject
    lateinit var appStateManager: AppStateManager

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)
        retainInstance = true
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        (activity as? AppBarController)?.apply {
            setupAppBarWithNavController()
            showAppBar()
        }
        val view = inflater.inflate(R.layout.fragment_ratings, container, false)
        view.recycler_ratings.apply {
            layoutManager = LinearLayoutManager(context)
            adapter = RatingAdapter()
        }
        return view
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        super.onPrepareOptionsMenu(menu)
        (menu.findItem(R.id.menu_search) as MenuItem).isVisible = false

        activity?.let { activity ->
            ViewUtil.manageInternetConnectState(activity, menu)
            ViewUtil.manageSocketConnectState(activity, menu)
        }
    }

    override fun loadingIntent(): Observable<Unit> = Observable.just(Unit)

    override fun snackbarDismiss(): Observable<Unit> = snackbarDismiss

    override fun render(state: RatingsViewState) {
        manageErrors(state)

        with(state) {
            if (isLoading) {
                (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.tasks_loading)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
                ratings_load_progressbar.show()
            } else {
                ratings_load_progressbar.hide()
            }

            ratingsList?.let { ratings ->
                (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.ratings)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle =
                    "${BuildConfig.VERSION_NAME} ${BuildConfig.BUILD_TYPE}"
                if (ratings.isEmpty()) {
                    fragment_ratings_empty_placeholder.show()
                    recycler_ratings.hide()
                } else {
                    fragment_ratings_empty_placeholder.hide()
                    recycler_ratings.show()
                    (recycler_ratings.adapter as RatingAdapter).ratingList = ratings
                }
            }
        }
    }

    // On view handling errors
    private fun manageErrors(state: RatingsViewState) {
        with(state) {
            when {
                errorMessage != null -> {
                    showSnackbar(errorMessage)
                }
                else -> {
                    dismissSnackbar()
                }
            }
        }
    }
}