package ru.daotech.agroinvest.fragments.ratings

import ru.daotech.agroinvest.models.Rating

sealed class RatingsPartialViewState {
    abstract fun reduce(previousState: RatingsViewState): RatingsViewState

    object Loading : RatingsPartialViewState() {
        override fun reduce(previousState: RatingsViewState): RatingsViewState {
            return previousState.copy(
                isLoading = true,
                errorMessage = null
            )
        }
    }

    data class SuccessLoadRatings(val ratingsList: List<Rating>) : RatingsPartialViewState() {
        override fun reduce(previousState: RatingsViewState): RatingsViewState {
            return previousState.copy(
                isLoading = false,
                ratingsList = ratingsList
            )
        }
    }

    object ClearError : RatingsPartialViewState() {
        override fun reduce(previousState: RatingsViewState): RatingsViewState {
            return previousState.copy(
                errorMessage = null
            )
        }
    }

    data class Error(val errorMessage: String) : RatingsPartialViewState() {
        override fun reduce(previousState: RatingsViewState): RatingsViewState {
            return previousState.copy(errorMessage = errorMessage)
        }
    }

    data class GoToDetailedItem(val ratingId: String) : RatingsPartialViewState() {
        override fun reduce(previousState: RatingsViewState): RatingsViewState {
            return previousState
        }
    }
}