package ru.daotech.agroinvest.fragments.sections.detailed

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.LinearLayoutManager
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_sections.*
import kotlinx.android.synthetic.main.fragment_sections.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.adapters.SectionAdapter
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.database.entities.Section
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.hide
import ru.daotech.agroinvest.show

class DetailedSectionFragment :
    BaseMviFragment<DetailedSectionViewState, DetailedSectionView, DetailedSectionPresenter>(),
    DetailedSectionView {

    private val sectionClicks: PublishSubject<Section> = PublishSubject.create()
    private val onDetailedSectionLoadFromDb: PublishSubject<String> = PublishSubject.create()

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        (activity as? AppBarController)?.apply {
            setupAppBarWithNavController()
            showAppBar()
        }
        val view = inflater.inflate(R.layout.fragment_sections, container, false)
        view.recycler_sections.apply {
            layoutManager = LinearLayoutManager(context)
            adapter = SectionAdapter(sectionClicks)
        }
        return view
    }

    override fun onResume() {
        arguments?.let { bundle ->
            val args = DetailedSectionFragmentArgs.fromBundle(bundle)
            args.detailedSectionModel.let { detailedSectionId ->
                onDetailedSectionLoadFromDb.onNext(detailedSectionId)
            }
        }
        super.onResume()
    }

    override fun onSectionClick(): Observable<Section> = sectionClicks

    override fun loadingIntent(): Observable<String> = onDetailedSectionLoadFromDb

    override fun snackbarDismiss(): Observable<Unit> = snackbarDismiss

    override fun render(state: DetailedSectionViewState) {
        manageErrors(state)

        with(state) {
            if (isLoading) {
                (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.tasks_loading)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
                sections_load_progressbar.show()
            } else {
                sections_load_progressbar.hide()
            }

            parentSection?.let {
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = parentSection.name
            }

            sectionsList?.let { sections ->
                (activity as ContainerActivity).activity_container_custom_toolbar?.title =
                    getString(R.string.sections_fragment_title)
                if (sections.isEmpty()) {
                    fragment_sections_empty_placeholder.show()
                    recycler_sections.hide()
                } else {
                    fragment_sections_empty_placeholder.hide()
                    recycler_sections.show()
                    (recycler_sections.adapter as SectionAdapter).sectionList = sections
                }
            }
        }
    }

    // On view handling errors
    private fun manageErrors(state: DetailedSectionViewState) {
        with(state) {
            when {
                errorMessage != null -> {
                    showSnackbar(errorMessage)
                }
                else -> {
                    dismissSnackbar()
                }
            }
        }
    }
}