package ru.daotech.agroinvest.fragments.sections.detailed

import ru.daotech.agroinvest.database.entities.Section

sealed class DetailedSectionPartialViewState {
    abstract fun reduce(previousState: DetailedSectionViewState): DetailedSectionViewState

    object Loading : DetailedSectionPartialViewState() {
        override fun reduce(previousState: DetailedSectionViewState): DetailedSectionViewState {
            return previousState.copy(
                isLoading = true,
                errorMessage = null
            )
        }
    }

    data class SuccessLoadDetailedSection(val sectionsList: List<Section>) : DetailedSectionPartialViewState() {
        override fun reduce(previousState: DetailedSectionViewState): DetailedSectionViewState {
            return previousState.copy(
                isLoading = false,
                sectionsList = sectionsList
            )
        }
    }

    data class SuccessLoadParentSection(val section: Section) : DetailedSectionPartialViewState() {
        override fun reduce(previousState: DetailedSectionViewState): DetailedSectionViewState {
            return previousState.copy(
                parentSection = section
            )
        }
    }

    object ClearError : DetailedSectionPartialViewState() {
        override fun reduce(previousState: DetailedSectionViewState): DetailedSectionViewState {
            return previousState.copy(errorMessage = null)
        }
    }

    data class Error(val errorMessage: String) : DetailedSectionPartialViewState() {
        override fun reduce(previousState: DetailedSectionViewState): DetailedSectionViewState {
            return previousState.copy(errorMessage = errorMessage)
        }
    }

    data class GoToSectionTasks(val sectionId: String) : DetailedSectionPartialViewState() {
        override fun reduce(previousState: DetailedSectionViewState): DetailedSectionViewState {
            return previousState
        }
    }
}