package ru.daotech.agroinvest.fragments.sections.root

import android.os.Bundle
import android.view.*
import androidx.recyclerview.widget.LinearLayoutManager
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_sections.*
import kotlinx.android.synthetic.main.fragment_sections.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.adapters.SectionAdapter
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.database.entities.Section
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.hide
import ru.daotech.agroinvest.show
import ru.daotech.agroinvest.utils.ViewUtil

class SectionsFragment : BaseMviFragment<SectionsViewState, SectionsView, SectionsPresenter>(),
    SectionsView {

    private val sectionClicks: PublishSubject<Section> = PublishSubject.create()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)
        retainInstance = true
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        super.onPrepareOptionsMenu(menu)
        (menu.findItem(R.id.menu_search) as MenuItem).isVisible = false

        activity?.let { activity ->
            ViewUtil.manageInternetConnectState(activity, menu)
            ViewUtil.manageSocketConnectState(activity, menu)
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        (activity as? AppBarController)?.apply {
            setupAppBarWithNavController()
            showAppBar()
        }
        val view = inflater.inflate(R.layout.fragment_sections, container, false)
        view.recycler_sections.apply {
            layoutManager = LinearLayoutManager(context)
            adapter = SectionAdapter(sectionClicks)
        }
        return view
    }

    override fun onSectionClick(): Observable<Section> = sectionClicks

    override fun loadingIntent(): Observable<Unit> = Observable.just(Unit)

    override fun snackbarDismiss(): Observable<Unit> = snackbarDismiss

    override fun render(state: SectionsViewState) {

        if (state.isLoading) {
            (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.tasks_loading)
            (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
            sections_load_progressbar.show()
        } else {
            sections_load_progressbar.hide()
        }

        manageErrors(state)

        state.sectionsList?.let { sections ->
            (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.sections_fragment_title)
            (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
            if (sections.isEmpty()) {
                fragment_sections_empty_placeholder.show()
                recycler_sections.hide()
            } else {
                fragment_sections_empty_placeholder.hide()
                recycler_sections.show()
                (recycler_sections.adapter as SectionAdapter).sectionList = sections
            }
        }
    }

    // On view handling errors
    private fun manageErrors(state: SectionsViewState) {
        with(state) {
            when {
                errorMessage != null -> {
                    showSnackbar(errorMessage)
                }
                else -> {
                    dismissSnackbar()
                }
            }
        }
    }
}