package ru.daotech.agroinvest.fragments.sections.root

import ru.daotech.agroinvest.database.entities.Section

sealed class SectionsPartialViewState {
    abstract fun reduce(previousState: SectionsViewState): SectionsViewState

    object Loading : SectionsPartialViewState() {
        override fun reduce(previousState: SectionsViewState): SectionsViewState {
            return previousState.copy(
                isLoading = true,
                errorMessage = null
            )
        }
    }

    data class SuccessLoadSections(val sectionsList: List<Section>) : SectionsPartialViewState() {
        override fun reduce(previousState: SectionsViewState): SectionsViewState {
            return previousState.copy(
                isLoading = false,
                sectionsList = sectionsList
            )
        }
    }

    object ClearError : SectionsPartialViewState() {
        override fun reduce(previousState: SectionsViewState): SectionsViewState {
            return previousState.copy(errorMessage = null)
        }
    }

    data class Error(val errorMessage: String) : SectionsPartialViewState() {
        override fun reduce(previousState: SectionsViewState): SectionsViewState {
            return previousState.copy(errorMessage = errorMessage)
        }
    }

    data class GoToDetailedItem(val sectionId: String) : SectionsPartialViewState() {
        override fun reduce(previousState: SectionsViewState): SectionsViewState {
            return previousState
        }
    }
}