package ru.daotech.agroinvest.fragments.splash

import com.hannesdorfmann.mosby3.mvi.MviBasePresenter
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import ru.daotech.agroinvest.AppStateManager
import timber.log.Timber
import java.util.concurrent.TimeUnit

class SplashPresenter(
    private val appStateManager: AppStateManager,
    private var onSplashFinished: ((Boolean) -> Unit)?
) :
    MviBasePresenter<SplashView, SplashViewState>() {

    private val disposable = CompositeDisposable()

    override fun bindIntents() {
        val loadingIntent = intent(SplashView::loadingIntent)
            .delay(3, TimeUnit.SECONDS)
            .flatMap { appStateManager.isWorkerSet() }
            .map { currentWorkerExists ->
                SplashViewState.Success(false) as SplashViewState
            }
            .onErrorReturn {
                Timber.e(it)
                SplashViewState.Error
            }
            .observeOn(AndroidSchedulers.mainThread())
            .share()

        disposable.addAll(loadingIntent
            .filter { state ->
                state is SplashViewState.Success
            }
            .map { state ->
                state as SplashViewState.Success
            }
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe(
                { state ->
                    onSplashFinished?.invoke(state.isAuthenticated)
                },
                { error ->
                    Timber.e(error, error.localizedMessage)
                }
            )
        )

        subscribeViewState(loadingIntent, SplashView::render)
    }

    override fun unbindIntents() {
        disposable.clear()
        onSplashFinished = null
    }
}