package ru.daotech.agroinvest.fragments.support_notification

import android.annotation.SuppressLint
import android.content.Context
import android.os.Bundle
import android.view.*
import android.view.inputmethod.InputMethodManager
import androidx.drawerlayout.widget.DrawerLayout
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.hannesdorfmann.mosby3.mvi.MviFragment
import com.jakewharton.rxbinding2.view.clicks
import dagger.android.support.AndroidSupportInjection
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_support_notify.*
import kotlinx.android.synthetic.main.fragment_support_notify.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.adapters.SupportNotificationAdapter
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.hide
import ru.daotech.agroinvest.show
import ru.daotech.agroinvest.utils.ViewUtil
import javax.inject.Inject

class SupportNotificationFragment :
    MviFragment<SupportNotificationView, SupportNotificationPresenter>(),
    SupportNotificationView {

    @Inject
    lateinit var supportNotificationPresenter: SupportNotificationPresenter

    private val onGoToAccidentSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onSendUserSupportMessage: PublishSubject<String> = PublishSubject.create()

    override fun onAttach(context: Context) {
        AndroidSupportInjection.inject(this)
        super.onAttach(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)

        this.requireActivity().drawer_layout?.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED)
        (this.activity as ContainerActivity).actionBar?.setDisplayShowHomeEnabled(true)
        (this.activity as ContainerActivity).actionBar?.setDisplayHomeAsUpEnabled(true)

        retainInstance = true
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        super.onPrepareOptionsMenu(menu)
        (menu.findItem(R.id.menu_search) as MenuItem).isVisible = false

        activity?.let { activity ->
            ViewUtil.manageInternetConnectState(activity, menu)
            ViewUtil.manageSocketConnectState(activity, menu)
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        (activity as? AppBarController)?.apply {
            setupAppBarWithNavController()
            showAppBar()
        }

        val view = inflater.inflate(
            R.layout.fragment_support_notify,
            container,
            false
        )

        view.recycler_support_notifications.apply {
            layoutManager = LinearLayoutManager(context)
            adapter = SupportNotificationAdapter()
        }
        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.support_chat_form)
        (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
        (this.activity as ContainerActivity).showToggle()

        support_go_to_accident_report.clicks()
            .doOnNext {
                hideKeyboard()
                onGoToAccidentSubject.onNext(Unit)
            }
            .subscribe()

        with(recycler_support_notifications.adapter as SupportNotificationAdapter) {
            registerAdapterDataObserver(object : RecyclerView.AdapterDataObserver() {
                override fun onItemRangeInserted(positionStart: Int, itemCount: Int) {
                    super.onItemRangeInserted(positionStart, itemCount)
                    recycler_support_notifications.smoothScrollToPosition(this@with.itemCount)
                }
            })
        }

        handleDrawableMessageEditView()
    }

    @SuppressLint("ClickableViewAccessibility")
    private fun handleDrawableMessageEditView() {
        support_message_input_field_view.setOnTouchListener(View.OnTouchListener { _, event ->
            val DRAWABLE_RIGHT_CODE = 2

            if (event.action == MotionEvent.ACTION_UP) {
                if (event.rawX
                    >= (support_message_input_field_view
                        .right - support_message_input_field_view
                        .compoundDrawables[DRAWABLE_RIGHT_CODE]
                        .bounds.width())
                ) {
                    if (!support_message_input_field_view.text.isNullOrBlank()) {
                        onSendUserSupportMessage.onNext(
                            support_message_input_field_view.text.toString()
                        )
                        support_message_input_field_view.text.clear()
                    }
                    return@OnTouchListener true
                }
            }
            return@OnTouchListener false
        })
    }

    override fun loadingIntent(): Observable<Unit> = Observable.just(Unit)

    override fun onGoToAccidentIntent(): Observable<Unit> = onGoToAccidentSubject

    override fun onSendUserSupportMessageIntent(): Observable<String> = onSendUserSupportMessage

    override fun render(state: SupportNotificationViewState) {

        if (state is SupportNotificationViewState.Loading) {
            (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.tasks_loading)
            support_notifications_load_progressbar.show()
        } else {
            support_notifications_load_progressbar.hide()
        }

        manageErrors(state)

        if (state is SupportNotificationViewState.LoadedSupportMessages) {
            state.supportMessagesList.let { supportMessageList ->
                (activity as ContainerActivity).activity_container_custom_toolbar?.title =
                    getString(R.string.support_chat_form)

                if (supportMessageList.isEmpty()) {
                    fragment_support_notification_placeholder.show()
                    recycler_support_notifications.hide()
                    support_go_to_accident_report.hide()
                } else {
                    fragment_support_notification_placeholder.hide()
                    recycler_support_notifications.show()
                    support_go_to_accident_report.show()
                    with(recycler_support_notifications.adapter as SupportNotificationAdapter) {
                        supportNotificationList = supportMessageList
                    }
                }
            }
        }
    }

    // On view handling errors
    private fun manageErrors(state: SupportNotificationViewState) {
        when (state) {
            is Error -> {
                // TODO: Need to show snackbar
            }
            else -> {
            }
        }
    }

    private fun Fragment.hideKeyboard() {
        val imm = context?.getSystemService(Context.INPUT_METHOD_SERVICE)
                as InputMethodManager
        imm.hideSoftInputFromWindow(this.view?.windowToken, 0)
    }

    override fun createPresenter(): SupportNotificationPresenter {
        return supportNotificationPresenter
    }
}