package ru.daotech.agroinvest.fragments.tasks.collectionHistory

import android.os.Bundle
import android.view.*
import androidx.recyclerview.widget.LinearLayoutManager
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_tasks.*
import kotlinx.android.synthetic.main.fragment_tasks.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.adapters.CollectionHistoryAdapter
import ru.daotech.agroinvest.adapters.CollectionHistoryModelView
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.database.entities.TrolleyInfoImpl
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.hide
import ru.daotech.agroinvest.show
import ru.daotech.agroinvest.utils.DateUtil
import ru.daotech.agroinvest.utils.ViewUtil

class CollectionHistoryFragment :
    BaseMviFragment<CollectionHistoryViewState, CollectionHistoryView, CollectionHistoryPresenter>(),
    CollectionHistoryView {

    private val onRefreshCollectionHistoryListSubject: PublishSubject<Unit> =
        PublishSubject.create()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        (activity as? AppBarController)?.apply {
            setupAppBarWithNavController()
            showAppBar()
        }

        val view = inflater.inflate(R.layout.fragment_tasks, container, false)

        view.swipeRefreshLayout.setOnRefreshListener {
            swipeRefreshLayout.isRefreshing = false
            onRefreshCollectionHistoryListSubject.onNext(Unit)
        }

        view.recycler_tasks.apply {
            layoutManager = LinearLayoutManager(context)
            adapter = CollectionHistoryAdapter()
        }

        return view
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        super.onPrepareOptionsMenu(menu)
        (menu.findItem(R.id.menu_search) as MenuItem).isVisible = false

        activity?.let { activity ->
            ViewUtil.manageInternetConnectState(activity, menu)
            ViewUtil.manageSocketConnectState(activity, menu)
        }
    }

    override fun loadingIntent(): Observable<Unit> = Observable.just(Unit)

    override fun snackbarDismiss(): Observable<Unit> = snackbarDismiss

    override fun collectionHistoryListSignalIntent(): Observable<Unit> =
        onRefreshCollectionHistoryListSubject

    override fun render(state: CollectionHistoryViewState) {
        with(state) {
            if (isLoading) {
                (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.tasks_loading)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
                tasks_load_progressbar.show()
            } else {
                tasks_load_progressbar.hide()
            }

            trolleyInfoList?.let { trolleyInfoList ->
                if (errorMessage != null) {
                    showSnackbar(errorMessage, R.color.colorAccent)
                } else {
                    dismissSnackbar()
                }

                if (errorMessage != null) {
                    showSnackbar(errorMessage, R.color.colorGreyAppText)
                } else {
                    dismissSnackbar()
                }

                (activity as ContainerActivity).activity_container_custom_toolbar?.title =
                    getString(R.string.collection_history)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
                if (trolleyInfoList.isEmpty()) {
                    fragment_tasks_empty_placeholder.show()
                    recycler_tasks.hide()
                } else {
                    fragment_tasks_empty_placeholder.hide()
                    recycler_tasks.show()

                    val trolleyUniqueRegisteredDateList = trolleyInfoList
                        .map { DateUtil.getCollectionHistoryDateFormatDateString(it.registeredAt) }
                        .distinct()

                    val collectionHistoryViewModelList = ArrayList<CollectionHistoryModelView>()

                    trolleyUniqueRegisteredDateList.forEach { date ->
                        val trolleyViewModelList = ArrayList<TrolleyInfoImpl>()
                        trolleyInfoList
                            .filter { DateUtil.getCollectionHistoryDateFormatDateString(it.registeredAt) == date }
                            .forEach { trolleyInfo  ->
                                trolleyViewModelList.add(trolleyInfo)
                            }
                        collectionHistoryViewModelList.add(
                            CollectionHistoryModelView(
                                trolleyViewModelList
                                    .filter { it.type == "finish" }
                                    .reversed(),
                                date
                            )
                        )
                    }

                    (recycler_tasks.adapter as CollectionHistoryAdapter).collectionHistoryList =
                        collectionHistoryViewModelList
                            .reversed()
                }
            }
        }
    }
}