package ru.daotech.agroinvest.fragments.tasks.collectionHistory

import ru.daotech.agroinvest.database.entities.TrolleyInfoImpl

sealed class CollectionHistoryPartialViewState {

    abstract fun reduce(previousState: CollectionHistoryViewState): CollectionHistoryViewState

    object Loading : CollectionHistoryPartialViewState() {
        override fun reduce(previousState: CollectionHistoryViewState): CollectionHistoryViewState {
            return previousState.copy(
                isLoading = true,
                errorMessage = null
            )
        }
    }

    object LoadingDismiss : CollectionHistoryPartialViewState() {
        override fun reduce(previousState: CollectionHistoryViewState): CollectionHistoryViewState {
            return previousState.copy(
                isLoading = false,
                errorMessage = null
            )
        }
    }

    object UploadDataSuccess : CollectionHistoryPartialViewState() {
        override fun reduce(previousState: CollectionHistoryViewState): CollectionHistoryViewState {
            return previousState
        }
    }

    object ClearError : CollectionHistoryPartialViewState() {
        override fun reduce(previousState: CollectionHistoryViewState): CollectionHistoryViewState {
            return previousState.copy(errorMessage = null)
        }
    }

    data class SuccessLoadCollectionHistory(
        val trolleyInfoList: List<TrolleyInfoImpl>,
        val serverTimestamp: Int?
    ) : CollectionHistoryPartialViewState() {
        override fun reduce(previousState: CollectionHistoryViewState): CollectionHistoryViewState {
            return previousState.copy(
                isLoading = false,
                trolleyInfoList = trolleyInfoList
            )
        }
    }

    data class Error(val errorMessage: String) : CollectionHistoryPartialViewState() {
        override fun reduce(previousState: CollectionHistoryViewState): CollectionHistoryViewState {
            return previousState.copy(errorMessage = errorMessage)
        }
    }

    data class GoToDetailedItem(val taskId: String) : CollectionHistoryPartialViewState() {
        override fun reduce(previousState: CollectionHistoryViewState): CollectionHistoryViewState {
            return previousState
        }
    }
}
