package ru.daotech.agroinvest.fragments.tasks.myTasks

import android.annotation.SuppressLint
import android.os.Bundle
import android.view.*
import android.view.View.GONE
import android.view.View.VISIBLE
import android.widget.RelativeLayout
import androidx.appcompat.app.AlertDialog
import androidx.core.view.isVisible
import androidx.recyclerview.widget.LinearLayoutManager
import com.jakewharton.rxbinding2.view.clicks
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_tasks.*
import kotlinx.android.synthetic.main.fragment_tasks.view.*
import kotlinx.android.synthetic.main.sub_toolbar_trolley_tied.view.*
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import ru.daotech.agroinvest.*
import ru.daotech.agroinvest.adapters.TaskAdapter
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.database.entities.Task
import ru.daotech.agroinvest.dialog.builder.TasksDialogBuilder
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.utils.RAW_NFC_PREFIX
import ru.daotech.agroinvest.utils.ServiceUtil
import ru.daotech.agroinvest.utils.TROLLEY_NFC_PREFIX
import ru.daotech.agroinvest.utils.ViewUtil
import timber.log.Timber
import javax.inject.Inject

class TasksFragment : BaseMviFragment<TasksViewState, TasksView, TasksPresenter>(), TasksView {

    @Inject
    lateinit var appStateManager: AppStateManager

    @Inject
    lateinit var dialogBuilder: TasksDialogBuilder

    private lateinit var taskAlreadyInProgressAttentionDialog: AlertDialog
    private lateinit var rawScannedNumber: String

    private val taskClicks: PublishSubject<Task> = PublishSubject.create()
    private val onRefreshTaskListSubject: PublishSubject<Unit> = PublishSubject.create()
    private val clearRatingMessageSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onOpenDetailedTaskById: PublishSubject<String> = PublishSubject.create()
    private val onClearPreferences: PublishSubject<Unit> = PublishSubject.create()
    private val onCurrentTrolleyTiedSubject: PublishSubject<Unit> = PublishSubject.create()
    private val onCurrentRawSubject: PublishSubject<String> = PublishSubject.create()
    private val onCurrentTrolleySubject: PublishSubject<String> = PublishSubject.create()
    private val goToDetailedTaskSubject: PublishSubject<String> = PublishSubject.create()
    private val goToBarcodeBoxSubject: PublishSubject<Unit> = PublishSubject.create()
    private val errorWrongNfcTypeSubject: PublishSubject<Unit> = PublishSubject.create()
    private val dismissTaskAttentionDialog: PublishSubject<Unit> = PublishSubject.create()
    private val dismissTaskAlreadyInProgressDialogSubject: PublishSubject<Unit> =
        PublishSubject.create()

    override fun onCreate(savedInstanceState: Bundle?) {
        EventBus.getDefault().register(this)
        setHasOptionsMenu(true)
        super.onCreate(savedInstanceState)
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        super.onPrepareOptionsMenu(menu)
        (menu.findItem(R.id.menu_search) as MenuItem).isVisible = true

        activity?.let { activity ->
            ViewUtil.manageInternetConnectState(activity, menu)
            ViewUtil.manageSocketConnectState(activity, menu)
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        (activity as? AppBarController)?.apply {
            setupAppBarWithNavController()
            showAppBar()
        }

        val view = inflater.inflate(R.layout.fragment_tasks, container, false)
        view.recycler_tasks.apply {
            layoutManager = LinearLayoutManager(context)
            adapter = TaskAdapter(taskClicks)
        }

        showSubToolbarContainer(view)

        return view
    }

    private fun showSubToolbarContainer(view: View) {
        val includeToolbarContainer =
            view.findViewById<RelativeLayout>(R.id.sub_toolbar_trolley_tied_tasks_container)

        includeToolbarContainer?.let {
            view.current_trolley_value.text =
                when (appStateManager.getCurrentTrolleyNumber().isNotBlank()) {
                    true -> {
                        includeToolbarContainer.setBackgroundResource(R.color.colorAccent)
                        view.remove_trolley_button.visibility = VISIBLE
                        view.add_trolley_button.visibility = GONE
                        appStateManager.getCurrentTrolleyNumber()
                    }
                    else -> {
                        includeToolbarContainer.setBackgroundResource(R.color.colorAppAttentionRed)
                        view.remove_trolley_button.visibility = GONE
                        view.add_trolley_button.visibility = VISIBLE
                        getString(R.string.no_trolley_assigned_title)
                    }
                }

            includeToolbarContainer
                .clicks()
                .doOnError { Timber.e(it) }
                .subscribe(onCurrentTrolleyTiedSubject)
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        this.view!!.swipeRefreshLayout.setOnRefreshListener {
            if (swipeRefreshLayout != null) {
                swipeRefreshLayout.isRefreshing = false
            }
            onRefreshTaskListSubject.onNext(Unit)
        }
    }

    override fun onTaskClick(): Observable<Task> = taskClicks

    override fun loadingIntent(): Observable<Unit> = Observable.just(Unit)

    override fun clearRatingMessageIntent(): Observable<Unit> = clearRatingMessageSubject

    override fun taskListSignalIntent(): Observable<Unit> = onRefreshTaskListSubject

    override fun snackbarDismiss(): Observable<Unit> = snackbarDismiss

    override fun openDetailedTaskByIdIntent(): Observable<String> = onOpenDetailedTaskById

    override fun clearPreferencesIntent(): Observable<Unit> = onClearPreferences

    override fun onCurrentRawIntent(): Observable<String> = onCurrentRawSubject

    override fun goToDetailedTaskIntent(): Observable<String> = goToDetailedTaskSubject

    override fun onCurrentTrolleyIntent(): Observable<String> = onCurrentTrolleySubject
    //    TODO: Some way to fix multiple event bus handling
    //     from other fragments after fragment transaction
    //        .doOnNext { EventBus.getDefault().unregister(this) }

    override fun onCurrentTrolleyTiedIntent(): Observable<Unit> = onCurrentTrolleyTiedSubject

    override fun errorWrongNfcTypeIntent(): Observable<Unit> = errorWrongNfcTypeSubject

    override fun goToBarcodeBoxIntent(): Observable<Unit> = goToBarcodeBoxSubject

    override fun dismissTaskAttentionDialogIntent(): Observable<Unit> = dismissTaskAttentionDialog

    override fun dismissTaskAlreadyInProgressDialogIntent(): Observable<Unit> =
        dismissTaskAlreadyInProgressDialogSubject

    override fun render(state: TasksViewState) {
        with(state) {
            if (isLoading) {
                (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.tasks_loading)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null

                tasks_load_progressbar.show()
            } else {
                tasks_load_progressbar.hide()
            }

            manageErrors(state)

            taskList?.let { tasks ->
                (activity as ContainerActivity).activity_container_custom_toolbar?.title =
                    """${getString(R.string.my_tasks_fragment_title)} (${tasks.size})"""
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle =
                    "${BuildConfig.VERSION_NAME} ${BuildConfig.BUILD_TYPE}"

                when (tasks.any { it.isHarvesting != null && it.isHarvesting == true }
                        || appStateManager.getCurrentTrolleyNumber().isNotBlank()) {
                    true -> {
                        sub_toolbar_trolley_tied_tasks_container.visibility = VISIBLE

                        if (appStateManager.getCurrentTrolleyNumber().isNotBlank()) {
                            // TODO: Temporary comment logic
//                            barcode_box_open_button.isVisible = true
                            barcode_box_open_button
                                .clicks()
                                .doOnNext { goToBarcodeBoxSubject.onNext(Unit) }
                                .subscribe()
                        }
                    }
                    else -> {
                        sub_toolbar_trolley_tied_tasks_container.visibility = GONE
                        barcode_box_open_button.isVisible = false

                        (activity as ContainerActivity).navigation_view?.let { currentMenu ->
                            currentMenu.menu.findItem(R.id.collectionHistoryFragment).isVisible =
                                false
                            currentMenu.menu.findItem(R.id.collectionHistoryFragment)
                                .setVisible(false)
                        }
                    }
                }

                if (tasks.isEmpty()) {
                    fragment_tasks_empty_placeholder.show()
                    recycler_tasks.hide()
                    if (isRowFilterDefined) {
                        fragment_tasks_empty_placeholder.text =
                            getString(R.string.task_list_is_empty_placeholder)
                    } else {
                        fragment_tasks_empty_placeholder.text =
                            getString(R.string.task_empty_list_placeholder)
                    }
                } else {
                    fragment_tasks_empty_placeholder.hide()
                    recycler_tasks.show()

                    (recycler_tasks.adapter as TaskAdapter).taskList = tasks

                    serverTimestamp?.let {
                        (recycler_tasks.adapter as TaskAdapter).serverTimestamp = serverTimestamp
                    }
                }
            }
        }
    }

    // On view handling errors
    private fun manageErrors(state: TasksViewState) {
        with(state) {
            when {
                errorMessage != null -> {
                    showSnackbar(errorMessage)
                }
                ratingMessage != null -> {
                    showSnackbar(ratingMessage)
                    clearRatingMessageSubject.onNext(Unit)
                }
                isTaskAlreadyInProgress -> {
                    taskAlreadyInProgressAttentionDialog =
                        dialogBuilder.buildTaskAlreadyInProgressAttentionDialog(
                            requireContext(),
                            dismissTaskAlreadyInProgressDialogSubject,
                            goToDetailedTaskSubject
                        ) { appStateManager.getCurrentRowNumber() }

                    dismissTaskAttentionDialog.onNext(Unit)
                }
                dismissSnackbar -> {
                    dismissSnackbar()
                }
            }
        }
    }

    override fun onResume() {
        if (!EventBus.getDefault().isRegistered(this)) {
            EventBus.getDefault().register(this)
        }
        arguments?.let { bundle ->
            try {
                val args = TasksFragmentArgs.fromBundle(bundle)
                args.taskId.let { taskId ->
                    onOpenDetailedTaskById.onNext(taskId!!)
                }
            } catch (ex: IllegalArgumentException) {
                Timber.i(ex)
            }
        }
        arguments?.clear()

//        Signal to clear preferences of DetailedTaskFragment.
//        This action is necessary for showing correct state after app hiding.
        onClearPreferences.onNext(Unit)

        super.onResume()
    }

    override fun onDestroy() {
        super.onDestroy()
        EventBus.getDefault().unregister(this)
    }

    @SuppressLint("DefaultLocale")
    @Subscribe(threadMode = ThreadMode.MAIN_ORDERED)
    fun onNfcAppScanned(event: NfcAppScanned) {
        val nfcMessageValueArray = event.message.split('.')
        when {
            nfcMessageValueArray[0].toLowerCase() == TROLLEY_NFC_PREFIX ->
                onCurrentTrolleySubject.onNext(ServiceUtil.cutOffExcessFromNfcMessage(event.message))
            nfcMessageValueArray[0].toLowerCase() == RAW_NFC_PREFIX -> {
                rawScannedNumber = ServiceUtil.cutOffExcessFromNfcMessage(event.message)
                onCurrentRawSubject.onNext(rawScannedNumber)
            }
            else -> {
                errorWrongNfcTypeSubject.onNext(Unit)
            }
        }
    }
}

data class NfcAppScanned(val message: String)
