package ru.daotech.agroinvest.fragments.tasks.myTasks

import ru.daotech.agroinvest.database.entities.Task

sealed class TasksPartialViewState {
    abstract fun reduce(previousState: TasksViewState): TasksViewState

    object Loading : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                isLoading = true,
                errorMessage = null,
                ratingMessage = null,
                dismissSnackbar = true
            )
        }
    }

    object LoadingDismiss : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                isLoading = false
            )
        }
    }

    data class SuccessLoadTasks(
        val taskList: List<Task>,
        val serverTimestamp: Int
    ) : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                isLoading = false,
                taskList = taskList,
                serverTimestamp = serverTimestamp,
                isRowFilterDefined = false
            )
        }
    }

    data class SuccessLoadTasksByFilter(
        val taskList: List<Task>,
        val serverTimestamp: Int
    ) : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                isLoading = false,
                taskList = taskList,
                serverTimestamp = serverTimestamp,
                isRowFilterDefined = true
            )
        }
    }

    object ClearError : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                errorMessage = null,
                ratingMessage = null,
                dismissSnackbar = true
            )
        }
    }

    object ClearLastTaskStatePreferences : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState
        }
    }

    object ShowErrorTaskAlreadyInProgressAlertAttention : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                isTaskAlreadyInProgress = true,
                dismissSnackbar = false,
                errorMessage = null
            )
        }
    }

    object DismissTaskAttentionDialogIntent : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                isTaskAlreadyInProgress = false,
                dismissSnackbar = false,
                errorMessage = null
            )
        }
    }

    object DismissTaskAlreadyInProgressDialog  : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                isTaskAlreadyInProgress = false,
                dismissSnackbar = false,
                errorMessage = null
            )
        }
    }

    object GoingToBarcodeBox : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState
        }
    }

    data class ShowRatingMessage(val ratingMessage: String?) : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                ratingMessage = ratingMessage,
                dismissSnackbar = false
            )
        }
    }

    data class GoToTrolleyTied(val nfcTrolleyMark: String?) : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                nfcTrolleyMark = nfcTrolleyMark
            )
        }
    }

    data class Error(val errorMessage: String) : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                errorMessage = errorMessage,
                dismissSnackbar = false
            )
        }
    }

    data class GoToDetailedItem(val taskId: String) : TasksPartialViewState() {
        override fun reduce(previousState: TasksViewState): TasksViewState {
            return previousState.copy(
                isTaskAlreadyInProgress = false,
                dismissSnackbar = false,
                errorMessage = null
            )
        }
    }
}