package ru.daotech.agroinvest.fragments.tasks.sectionTasks

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.LinearLayoutManager
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_tasks.*
import kotlinx.android.synthetic.main.fragment_tasks.view.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.adapters.TaskHistoryAdapter
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.database.entities.Task
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.hide
import ru.daotech.agroinvest.show

class SectionTasksFragment : BaseMviFragment<SectionTasksViewState, SectionTasksView, SectionTasksPresenter>(),
    SectionTasksView {

    private val taskClicks: PublishSubject<Task> = PublishSubject.create()
    private val onSectionTasksLoadFromDb: PublishSubject<String> = PublishSubject.create()

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        (activity as? AppBarController)?.apply {
            setupAppBarWithNavController()

            showAppBar()
        }

        val view = inflater.inflate(R.layout.fragment_tasks, container, false)

        view.swipeRefreshLayout.setOnRefreshListener {
            swipeRefreshLayout.isRefreshing = false
        }

        view.recycler_tasks.apply {
            layoutManager = LinearLayoutManager(context)

            adapter = TaskHistoryAdapter(showNames = true)
        }

        return view
    }

    override fun onResume() {
        arguments?.let { bundle ->
            val args = SectionTasksFragmentArgs.fromBundle(bundle)
            args.sectionTasksModel.let { sectionId ->
                onSectionTasksLoadFromDb.onNext(sectionId)
            }
        }
        super.onResume()
    }

    override fun onTaskClick(): Observable<Task> = taskClicks

    override fun loadingIntent(): Observable<String> = onSectionTasksLoadFromDb

    override fun snackbarDismiss(): Observable<Unit> = snackbarDismiss

    override fun render(state: SectionTasksViewState) {

        if (state.isLoading) {
            (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.tasks_loading)
            (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
            tasks_load_progressbar.show()
        } else {
            tasks_load_progressbar.hide()
        }

        manageErrors(state)

        state.parentSection?.let {
            (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = state.parentSection.name
        }

        state.taskWithPropertiesList?.let { taskWithProperties ->
            (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.sections)
            if (taskWithProperties.isEmpty()) {
                fragment_tasks_empty_placeholder.show()
                recycler_tasks.hide()
            } else {
                fragment_tasks_empty_placeholder.hide()
                recycler_tasks.show()
                (recycler_tasks.adapter as TaskHistoryAdapter).taskList = taskWithProperties
            }
        }
    }

    // On view handling errors
    private fun manageErrors(state: SectionTasksViewState) {
        with(state) {
            when {
                errorMessage != null -> {
                    showSnackbar(errorMessage)
                }
                else -> {
                    dismissSnackbar()
                }
            }
        }
    }
}
