package ru.daotech.agroinvest.fragments.tasks.sectionTasks

import ru.daotech.agroinvest.database.entities.Section
import ru.daotech.agroinvest.database.relation.TaskWithProperties

sealed class SectionTasksPartialViewState {
    abstract fun reduce(previousState: SectionTasksViewState): SectionTasksViewState

    object Loading : SectionTasksPartialViewState() {
        override fun reduce(previousState: SectionTasksViewState): SectionTasksViewState {
            return previousState.copy(
                isLoading = true,
                errorMessage = null
            )
        }
    }

    data class SuccessLoadTasks(val taskWithPropertiesList: List<TaskWithProperties>) : SectionTasksPartialViewState() {
        override fun reduce(previousState: SectionTasksViewState): SectionTasksViewState {
            return previousState.copy(
                isLoading = false,
                taskWithPropertiesList = taskWithPropertiesList
            )
        }
    }

    data class SuccessLoadParentSection(val section: Section) : SectionTasksPartialViewState() {
        override fun reduce(previousState: SectionTasksViewState): SectionTasksViewState {
            return previousState.copy(
                parentSection = section
            )
        }
    }

    object ClearError : SectionTasksPartialViewState() {
        override fun reduce(previousState: SectionTasksViewState): SectionTasksViewState {
            return previousState.copy(errorMessage = null)
        }
    }

    data class Error(val errorMessage: String) : SectionTasksPartialViewState() {
        override fun reduce(previousState: SectionTasksViewState): SectionTasksViewState {
            return previousState.copy(errorMessage = errorMessage)
        }
    }

    data class GoToDetailedItem(val taskId: String) : SectionTasksPartialViewState() {
        override fun reduce(previousState: SectionTasksViewState): SectionTasksViewState {
            return previousState
        }
    }
}