package ru.daotech.agroinvest.fragments.tasks.tasksHistory

import android.os.Bundle
import android.view.*
import androidx.annotation.ColorRes
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.android.material.snackbar.Snackbar
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_tasks.*
import kotlinx.android.synthetic.main.fragment_tasks.view.*
import ru.daotech.agroinvest.*
import ru.daotech.agroinvest.adapters.TaskHistoryAdapter
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.database.entities.Task
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.utils.ViewUtil
import javax.inject.Inject

class TasksHistoryFragment :
    BaseMviFragment<TasksHistoryViewState, TasksHistoryView, TasksHistoryPresenter>(),
    TasksHistoryView {

    @Inject
    lateinit var appStateManager: AppStateManager

    private val taskClicks: PublishSubject<Task> = PublishSubject.create()
    private val onSendNotSentTasksSubject: PublishSubject<Unit> = PublishSubject.create()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        (activity as? AppBarController)?.apply {
            setupAppBarWithNavController()
            showAppBar()
        }

        val view = inflater.inflate(R.layout.fragment_tasks, container, false)

        view.swipeRefreshLayout.setOnRefreshListener {
            swipeRefreshLayout.isRefreshing = false
            onSendNotSentTasksSubject.onNext(Unit)
        }

        view.recycler_tasks.apply {
            layoutManager = LinearLayoutManager(context)
            adapter = TaskHistoryAdapter(showNames = false)
        }

        return view
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        super.onPrepareOptionsMenu(menu)
        (menu.findItem(R.id.menu_search) as MenuItem).isVisible = false

        activity?.let { activity ->
            ViewUtil.manageInternetConnectState(activity, menu)
            ViewUtil.manageSocketConnectState(activity, menu)
        }
    }

    override fun onTaskClick(): Observable<Task> = taskClicks

    override fun loadingIntent(): Observable<Unit> = Observable.just(Unit)

    override fun snackbarDismiss(): Observable<Unit> = snackbarDismiss

    override fun onSendNotSentTasksIntent(): Observable<Unit> = onSendNotSentTasksSubject

    override fun render(state: TasksHistoryViewState) {
        with(state) {
            if (isLoading) {
                (activity as ContainerActivity).activity_container_custom_toolbar?.title = getString(R.string.tasks_loading)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle = null
                tasks_load_progressbar.show()
            } else {
                tasks_load_progressbar.hide()
            }

            taskWithPropertiesList?.let { taskWithPropertiesList ->
                if (errorMessage != null) {
                    showCurrentSnackbar(errorMessage, R.color.colorAccent, true)
                } else {
                    dismissSnackbar()
                }

                if (uploadDataMessage != null) {
                    showCurrentSnackbar(uploadDataMessage, R.color.colorGreyAppText, true)
                } else {
                    dismissSnackbar()
                }

                (activity as ContainerActivity).activity_container_custom_toolbar?.title =
                    getString(R.string.tasks_history_fragment_title)
                (activity as ContainerActivity).activity_container_custom_toolbar?.subtitle =
                    "${BuildConfig.VERSION_NAME} ${BuildConfig.BUILD_TYPE}"
                if (taskWithPropertiesList.isEmpty()) {
                    fragment_tasks_empty_placeholder.show()
                    recycler_tasks.hide()
                } else {
                    fragment_tasks_empty_placeholder.hide()
                    recycler_tasks.show()
                    (recycler_tasks.adapter as TaskHistoryAdapter).taskList = taskWithPropertiesList

                    serverTimestamp?.let {
                        (recycler_tasks.adapter as TaskHistoryAdapter).serverTimestamp = serverTimestamp
                    }
                }
            }
        }
    }

    private fun showCurrentSnackbar(
        message: String,
        @ColorRes snackbarBackgroundColor: Int = R.color.accentSnackbarBackground,
        withOkAction: Boolean = false
    ) {
        val snackbar: Snackbar = if (withOkAction) {
            Snackbar.make(
                view!!,
                message,
                Snackbar.LENGTH_INDEFINITE
            ).apply {
                setAction("ОК") {
                    this.dismiss()
                }
                setActionTextColor(
                    ContextCompat.getColor(
                        requireContext(),
                        android.R.color.white
                    )
                )

            }
        } else {
            Snackbar.make(requireView(), message, Snackbar.LENGTH_SHORT)
        }
        val backgroundColor =
            ContextCompat.getColor(requireContext(), snackbarBackgroundColor)

        snackbar.view.setBackgroundColor(backgroundColor)
        snackbar.show()
    }
}