package ru.daotech.agroinvest.fragments.tasks.tasksHistory

import ru.daotech.agroinvest.database.relation.TaskWithProperties

sealed class TasksHistoryPartialViewState {
    abstract fun reduce(previousState: TasksHistoryViewState): TasksHistoryViewState

    object Loading : TasksHistoryPartialViewState() {
        override fun reduce(previousState: TasksHistoryViewState): TasksHistoryViewState {
            return previousState.copy(
                isLoading = true,
                errorMessage = null
            )
        }
    }

    object UploadDataSuccess : TasksHistoryPartialViewState() {
        override fun reduce(previousState: TasksHistoryViewState): TasksHistoryViewState {
            return previousState
        }
    }

    data class SuccessLoadTasks(
        val taskWithPropertiesList: List<TaskWithProperties>,
        val serverTimestamp: Int?
    ) : TasksHistoryPartialViewState() {
        override fun reduce(previousState: TasksHistoryViewState): TasksHistoryViewState {
            return previousState.copy(
                isLoading = false,
                taskWithPropertiesList = taskWithPropertiesList,
                serverTimestamp = serverTimestamp
            )
        }
    }

    object ClearError : TasksHistoryPartialViewState() {
        override fun reduce(previousState: TasksHistoryViewState): TasksHistoryViewState {
            return previousState.copy(errorMessage = null)
        }
    }

    data class Error(val errorMessage: String) : TasksHistoryPartialViewState() {
        override fun reduce(previousState: TasksHistoryViewState): TasksHistoryViewState {
            return previousState.copy(errorMessage = errorMessage)
        }
    }

    data class GoToDetailedItem(val taskId: String) : TasksHistoryPartialViewState() {
        override fun reduce(previousState: TasksHistoryViewState): TasksHistoryViewState {
            return previousState
        }
    }
}