package ru.daotech.agroinvest.fragments.trolley_tied

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.View.*
import android.view.ViewGroup
import android.view.WindowManager
import androidx.appcompat.app.AlertDialog
import androidx.drawerlayout.widget.DrawerLayout
import com.jakewharton.rxbinding2.view.clicks
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.activity_container.*
import kotlinx.android.synthetic.main.fragment_trolley_tied.*
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.container.ContainerActivity
import ru.daotech.agroinvest.dialog.builder.TrolleyDialogBuilder
import ru.daotech.agroinvest.fragments.base.BaseMviFragment
import ru.daotech.agroinvest.hide
import ru.daotech.agroinvest.show
import ru.daotech.agroinvest.utils.RAW_NFC_PREFIX
import ru.daotech.agroinvest.utils.ServiceUtil
import ru.daotech.agroinvest.utils.TROLLEY_NFC_PREFIX
import timber.log.Timber
import java.util.*
import javax.inject.Inject

class TrolleyTiedFragment :
    BaseMviFragment<TrolleyTiedViewState, TrolleyTiedView, TrolleyTiedPresenter>(),
    TrolleyTiedView {

    @Inject
    lateinit var appStateManager: AppStateManager

    @Inject
    lateinit var dialogBuilder: TrolleyDialogBuilder

    private val onTrolleyTakeClick: PublishSubject<String> = PublishSubject.create()
    private val onTrolleyUntiedClick: PublishSubject<String> = PublishSubject.create()
    private val onTrolleyLoadingInfo: PublishSubject<Unit> = PublishSubject.create()
    private val errorMessageSubject: PublishSubject<String> = PublishSubject.create()
    private val nfcMessageScannedSubject: PublishSubject<String> = PublishSubject.create()

    private lateinit var trolleyAttentionDialog: AlertDialog

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        EventBus.getDefault().register(this)
        retainInstance = true

        this.activity?.drawer_layout?.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED)
        this.activity?.window!!.setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN)
        (this.activity as ContainerActivity).actionBar?.setDisplayShowHomeEnabled(true)
        (this.activity as ContainerActivity).actionBar?.setDisplayHomeAsUpEnabled(true)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        val view = inflater.inflate(
            R.layout.fragment_trolley_tied,
            container,
            false
        )

        // Handle onBack toolbar button
        handleOnBackPressedToolbar()

        return view
    }

    override fun loadingIntent(): Observable<Unit> = onTrolleyLoadingInfo

    override fun takeTrolleyButtonClick(): Observable<String> = onTrolleyTakeClick

    override fun untiedTrolleyButtonClick(): Observable<String> = onTrolleyUntiedClick

    override fun errorMessageIntent(): Observable<String> = errorMessageSubject

    override fun nfcMessageScannedIntent(): Observable<String> = nfcMessageScannedSubject

    override fun render(state: TrolleyTiedViewState) {
        manageErrors(state)

        with(state) {
            if (isLoading) {
                progressbar.show()
                trolley_take_label.hide()
                take_trolley_button.isEnabled = false
                untied_trolley_button.isEnabled = false
            } else {
                progressbar.hide()

                trolley_take_label.show()
                take_trolley_button.isEnabled = true
                untied_trolley_button.isEnabled = true

                trolley_tied_number.setText("")
                trolley_tied_number.isFocusable = true
                trolley_tied_number.isEnabled = true
                trolley_tied_number.isCursorVisible = true
                trolley_tied_number.isFocusableInTouchMode = true

                trolley_tied_current_number_value.text = when (state.trolleyNumber.isNotBlank()) {
                    true -> state.trolleyNumber
                    else -> getString(R.string.data_not_found_title)
                }

                if (state.nfcMode != null && state.nfcMode) {
                    untied_trolley_button.visibility = GONE
                    take_trolley_button.visibility = GONE
                    trolley_tied_number.visibility = GONE
                } else {
                    when (state.trolleyNumber.isNotBlank()) {
                        true -> {
                            untied_trolley_button.visibility = VISIBLE
                            take_trolley_button.visibility = INVISIBLE
                            trolley_tied_number.visibility = GONE
                        }
                        else -> {
                            untied_trolley_button.visibility = INVISIBLE
                            take_trolley_button.visibility = VISIBLE
                            trolley_tied_number.visibility = VISIBLE
                        }
                    }
                }

                if (state.nfcMessage.isNullOrBlank()) {
                    manageArguments(null)
                } else {
                    manageArguments(state.nfcMessage)
                }

                take_trolley_button.clicks()
                    .doOnError { Timber.e(it) }
                    .map {
                        when (!trolley_tied_number.text.isNullOrBlank()) {
                            true -> {
                                buildAttentionTrolleyDialog(
                                    getString(R.string.trolley_tied_take_attention_title),
                                    trolley_tied_number.text.toString(),
                                    true
                                )
                            }
                            else -> trolley_tied_number.error =
                                getString(R.string.empty_value_of_field_attention)
                        }
                    }
                    .subscribe()

                untied_trolley_button.clicks()
                    .doOnError { Timber.e(it) }
                    .map {
                        when (!state.trolleyNumber.isBlank()) {
                            true -> {
                                buildAttentionTrolleyDialog(
                                    getString(R.string.trolley_tied_untie_attention_title),
                                    state.trolleyNumber,
                                    false
                                )
                            }
                            else -> trolley_tied_number.error =
                                getString(R.string.empty_value_of_field_attention)
                        }
                    }
                    .subscribe()
            }
        }
    }

    // On view handling errors
    private fun manageErrors(state: TrolleyTiedViewState) {
        with(state) {
            when {
                errorMessage != null -> {
                    showSnackbar(errorMessage)
                }
                isTakeSuccess -> {
                    // Clear arguments for next dialog showing dismiss
                    arguments?.clear()

                    try {
                        // Auto redirect
                        activity?.onBackPressed()
                    } catch (ex: IllegalStateException) {
                        Timber.i(ex)
                    }
                }
                isUntiedSuccess -> {
                    // Clear arguments for next dialog showing dismiss
                    arguments?.clear()

                    try {
                        // Auto redirect
                        activity?.onBackPressed()
                    } catch (ex: IllegalStateException) {
                        Timber.i(ex)
                    }
                }
                else -> {
                    dismissSnackbar()
                }
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        EventBus.getDefault().unregister(this)
        arguments?.clear()

        if (::trolleyAttentionDialog.isInitialized) {
            trolleyAttentionDialog.dismiss()
        }

        val fragmentManager = requireFragmentManager()
        requireFragmentManager().beginTransaction().apply {
            requireFragmentManager().fragments.forEach {
                if (it is TrolleyTiedFragment) {
                    this.remove(it)
                }
            }
            try {
                commit()
            } catch (ex: IllegalStateException) {
                // Do nothing...
            }
        }
    }

    private fun buildAttentionTrolleyDialog(
        message: String,
        trolleyNumber: String,
        isTakeNewTrolley: Boolean
    ) {
        trolleyAttentionDialog = dialogBuilder.buildTrolleyAttentionDialog(
            requireContext(),
            message,
            trolleyNumber,
            isTakeNewTrolley,
            onTrolleyTakeClick,
            onTrolleyUntiedClick,
            errorMessageSubject,
            appStateManager
        )
    }

    private fun handleOnBackPressedToolbar() {
        (activity as ContainerActivity).activity_container_custom_toolbar.setNavigationOnClickListener {
            onBackMarkTask()
        }
    }

    private fun onBackMarkTask() {
        if (this.isVisible) {
            this.activity?.onBackPressed()
        }
    }

    private fun manageArguments(message: String?) {
        if (!::trolleyAttentionDialog.isInitialized
            || !trolleyAttentionDialog.isShowing
        ) {
            var argumentValue: String? = message
            arguments?.let { bundle ->
                if (argumentValue.isNullOrBlank() && !bundle.isEmpty) {
                    val args = TrolleyTiedFragmentArgs.fromBundle(bundle)
                    args.trolleyNumber?.let { argumentValue = it }
                }
            }
            if (!argumentValue.isNullOrBlank()) {
                if (appStateManager.getCurrentTrolleyNumber().isBlank()) {
                    trolley_tied_number?.setText(argumentValue)
                    buildAttentionTrolleyDialog(
                        getString(R.string.trolley_tied_take_attention_title),
                        argumentValue!!,
                        true
                    )
                } else {
                    buildAttentionTrolleyDialog(
                        getString(R.string.trolley_tied_untie_attention_title),
                        argumentValue!!,
                        false
                    )
                }
            }
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN_ORDERED)
    fun onNfcTrolleyTiedScanned(event: NfcTrolleyTiedScanned) {
        val nfcMessageValueArray = event.message.split('.')
        when {
            nfcMessageValueArray[0].toLowerCase(Locale.ROOT) == TROLLEY_NFC_PREFIX ->
                nfcMessageScannedSubject.onNext(ServiceUtil.cutOffExcessFromNfcMessage(event.message))
            nfcMessageValueArray[0].toLowerCase(Locale.ROOT) == RAW_NFC_PREFIX -> {
                errorMessageSubject.onNext(
                    getString(R.string.wrong_scan_trolley_type_attention_message)
                )
            }
            else -> {
                errorMessageSubject.onNext(
                    getString(R.string.unknown_scan_trolley_type_attention_message)
                )
            }
        }
    }
}

data class NfcTrolleyTiedScanned(val message: String)