package ru.daotech.agroinvest.fragments.trolley_tied

sealed class TrolleyTiedPartialViewState {
    abstract fun reduce(previousState: TrolleyTiedViewState): TrolleyTiedViewState

    object Loading : TrolleyTiedPartialViewState() {
        override fun reduce(previousState: TrolleyTiedViewState): TrolleyTiedViewState {
            return previousState.copy(isLoading = true)
        }
    }

    object EmptyState : TrolleyTiedPartialViewState() {
        override fun reduce(previousState: TrolleyTiedViewState): TrolleyTiedViewState {
            return previousState
        }
    }

    object SuccessTakeLoaded : TrolleyTiedPartialViewState() {
        override fun reduce(previousState: TrolleyTiedViewState): TrolleyTiedViewState {
            return previousState.copy(
                isLoading = false,
                isTakeSuccess = true,
                isUntiedSuccess = false,
                errorMessage = null
            )
        }
    }

    object SuccessUntiedLoaded : TrolleyTiedPartialViewState() {
        override fun reduce(previousState: TrolleyTiedViewState): TrolleyTiedViewState {
            return previousState.copy(
                isLoading = false,
                isTakeSuccess = false,
                isUntiedSuccess = true,
                errorMessage = null
            )
        }
    }

    object ClearError : TrolleyTiedPartialViewState() {
        override fun reduce(previousState: TrolleyTiedViewState): TrolleyTiedViewState {
            return previousState.copy(errorMessage = null)
        }
    }

    data class Error(val errorMessage: String) : TrolleyTiedPartialViewState() {
        override fun reduce(previousState: TrolleyTiedViewState): TrolleyTiedViewState {
            return previousState.copy(
                errorMessage = errorMessage,
                isLoading = false
            )
        }
    }

    data class NfcMessageScanned(val nfcMessageScanned: String) : TrolleyTiedPartialViewState() {
        override fun reduce(previousState: TrolleyTiedViewState): TrolleyTiedViewState {
            return previousState.copy(
                nfcMessage = nfcMessageScanned,
                isLoading = false
            )
        }
    }
}