package ru.daotech.agroinvest.fragments.welcome

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.google.android.material.snackbar.Snackbar
import com.hannesdorfmann.mosby3.mvi.MviFragment
import dagger.android.support.AndroidSupportInjection
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.fragment_welcome.*
import ru.daotech.agroinvest.R
import ru.daotech.agroinvest.container.AppBarController
import ru.daotech.agroinvest.container.ContainerActivity
import javax.inject.Inject

class WelcomeFragment : MviFragment<WelcomeView, WelcomePresenter>(), WelcomeView {
    @Inject
    lateinit var welcomePresenter: WelcomePresenter

    private val clearErrorSubject = PublishSubject.create<Unit>()

    override fun onAttach(context: Context) {
        AndroidSupportInjection.inject(this)
        super.onAttach(context)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_welcome, container, false)
    }

    override fun createPresenter(): WelcomePresenter = welcomePresenter

    override fun loadingIntent(): Observable<Unit> = Observable.just(Unit)

    override fun sendSignal(): Observable<Unit> = Observable.just(Unit)

    override fun clearErrorIntent(): Observable<Unit> = clearErrorSubject

    override fun render(state: WelcomeViewState) {
        (activity as? AppBarController)?.hideAppBar()
        state.worker?.let { worker ->
            update_app_version_title.text = worker.name
            personal_number_text.text = getString(R.string.personal_number, worker.personalNumber)
            position_text.text = worker.positionName

            worker.department?.let {
                department_text.text = it.fullName
            }

            // Start listen to unread support message count
            (requireActivity() as ContainerActivity).onListenUnreadMessageSubject.onNext(Unit)
        }

        state.errorMessage?.let { errorMessage ->
            val snackbar = Snackbar.make(view!!, errorMessage, Snackbar.LENGTH_LONG)
            snackbar.addCallback(object : Snackbar.Callback() {
                override fun onShown(sb: Snackbar?) {
                    super.onShown(sb)
                    clearErrorSubject.onNext(Unit)
                }
            })
            snackbar.show()
        }
    }
}