package ru.daotech.agroinvest.models

import android.net.ConnectivityManager
import com.beust.klaxon.Json
import com.google.gson.Gson
import ru.daotech.agroinvest.network.models.BaseMetaDeviceData

/**
 *
 * Strings with "null" value is necessary because
 * the Countly does not work with null or empty values into segments.
 */
data class InternetConnectionStateJson(
    @Json(name = "rowNumber")
    val rowNumber: String? = "null",
    @Json(name = "downSpeed")
    val downSpeed: Int? = 0,
    @Json(name = "upSpeed")
    val upSpeed: Int? = 0,
    @Json(name = "time")
    val time: String? = "null",
    @Json(name = "connectionState")
    val connectionState: Boolean? = null,
    @Json(name = "workerName")
    val workerName: String? = "null",
    @Json(name = "personalNumber")
    val personalNumber: String? = "null",
    @Json(name = "networkInfo")
    val networkInfo: String? = "null",
    @Json(name = "networkCapabilities")
    val networkCapabilities: String? = "null",
    @Json(name = "workerId")
    override val workerId: String? = "null",
    @Json(name = "sessionId")
    override val sessionId: String? = "null",
    @Json(name = "deviceTimeMark")
    override var deviceTimeMark: String? = "null",
    @Json(name = "serverTimeMark")
    override var serverTimeMark:  String? = "null",
    @Json(name = "deviceId")
    override var deviceId:  String? = "null",
    @Json(name = "deltaTimestamp")
    override var deltaTimestamp:  Int? = 0
) : MetaRequest(
    workerId = workerId,
    deviceId = deviceId,
    sessionId = sessionId,
    deviceTimeMark = deviceTimeMark,
    serverTimeMark = serverTimeMark,
    deltaTimestamp = deltaTimestamp
)

/**
 *
 * Replicate properties form ConnectivityManager of
 * current internet connection state
 */
data class InternetConnectivityState(
    val downSpeed: Int? = 0,
    val upSpeed: Int? = 0,
    val networkInfo: String? = "null",
    val networkCapabilities: String? = "null"
) {
    constructor(from: ConnectivityManager) : this(
        downSpeed = from.getNetworkCapabilities(from.activeNetwork)
            ?.linkDownstreamBandwidthKbps,
        upSpeed = from.getNetworkCapabilities(from.activeNetwork)
            ?.linkUpstreamBandwidthKbps,
        networkInfo = if (from.activeNetworkInfo != null) {
            from.activeNetworkInfo.toString()
        } else {
            "null"
        },
        networkCapabilities = if (from.activeNetworkInfo != null) {
            from.getNetworkCapabilities(from.activeNetwork).toString()
        } else {
            "null"
        }
    )
}

abstract class MetaRequest(
    override val workerId: String? = "null",
    override var deviceId: String? = "null",
    override val sessionId: String? = "null",
    override var deviceTimeMark: String? = "null",
    override var serverTimeMark: String? = "null",
    override var deltaTimestamp: Int? = 0
) : BaseMetaDeviceData() {
    override fun toString(): String {
        return Gson().toJson(this)
    }
}