package ru.daotech.agroinvest.models

import android.os.Parcelable
import com.beust.klaxon.Json
import kotlinx.android.parcel.Parcelize

@Parcelize
data class Worker(
    val id: String,
    val name: String,
    val personalNumber: String,
    val departmentId: String? = null,
    val department: Department? = null,
    val positionName: String,
    val positionId: String,
    val exchangers: List<QueueExchanger>,
    val ratings: List<Rating>,
    var isWorkerInfoSent: Boolean = true,
    val earnings: Map<String, Float>? = null,
    val nfcMode: Boolean? = null
) : Parcelable

@Parcelize
data class Department(
    val name: String,
    val structure: List<DepartmentLevel>
) : Parcelable {
    @Json(ignored = true)
    val fullName: String
        get() = structure
            .map { departmentLevel -> departmentLevel.name }
            .takeLast(2)
            .joinToString(separator = " ")
}

@Parcelize
data class DepartmentLevel(
    val level: Int,
    val id: String,
    val name: String
) : Parcelable

@Parcelize
data class QueueExchanger(
    val name: String,
    val bindingKey: String
) : Parcelable

@Parcelize
data class Rating(
    val id: String,
    val name: String,
    val rating: Float,
    val ratingInt: Int
) : Parcelable