package ru.daotech.agroinvest.network

import io.reactivex.Observable
import io.reactivex.Single
import okhttp3.MultipartBody
import okhttp3.ResponseBody
import retrofit2.Response
import retrofit2.http.*
import ru.daotech.agroinvest.network.models.CamundaResponse
import ru.daotech.agroinvest.network.models.FileUploadResponse
import ru.daotech.agroinvest.network.models.VariableRequest
import ru.daotech.agroinvest.service.jsonmodels.TaskPayload
import ru.daotech.agroinvest.service.jsonmodels.WorkerPayload

interface AgroinvestService {

    @POST("/engine-rest/process-definition/key/MobileRequests/start")
    fun camundaSignOutRequest(
        @Body json: VariableRequest
    ): Single<Response<Any>>

    @Headers("Content-Type: application/json")
    @POST("/engine-rest/process-definition/key/MobileRequests/start")
    fun signalToCamunda(
        @Body body: String
    ): Single<Response<CamundaResponse>>

    @Multipart
    @POST
    fun fileUploadRequest(
        @Url url: String,
        @Part filePart: List<MultipartBody.Part>
    ): Observable<FileUploadResponse>

    @Headers("Content-Type: application/json")
    @POST("/engine-rest/process-definition/key/MobileRequests/start")
    fun sendTaskAccidentReport(@Body json: VariableRequest): Observable<ResponseBody>

    @Headers("Content-Type: application/json")
    @GET("{signInCacheUrl}/{workerCode}")
    fun signInByCacheUrl(
        @Path(value = "signInCacheUrl", encoded = true) signInCacheUrl: String,
        @Path(value = "workerCode", encoded = true) workerCode: String
    ): Observable<String>

    @Headers("Content-Type: application/json")
    @GET("{tasksCacheUrl}/{workerId}")
    fun loadTasksDataByCacheUrl(
        @Path(value = "tasksCacheUrl", encoded = true) tasksCacheUrl: String,
        @Path(value = "workerId", encoded = true) workerId: String
    ): Observable<String>

    @Headers("Content-Type: application/json")
    @GET
    fun loadCurrentAppStatusByUrl(@Url appStatusUrl: String): Observable<String>
}