package ru.daotech.agroinvest.network

import io.reactivex.Observable
import io.reactivex.Single
import okhttp3.ResponseBody
import retrofit2.Response
import ru.daotech.agroinvest.network.models.CamundaResponse
import ru.daotech.agroinvest.network.models.FileUploadResponse
import ru.daotech.agroinvest.network.models.VariableRequest
import ru.daotech.agroinvest.service.events.Event
import ru.daotech.agroinvest.service.events.SignIn
import ru.daotech.agroinvest.service.jsonmodels.ServerDataPayload
import ru.daotech.agroinvest.service.jsonmodels.TaskPayload
import java.io.File

interface NetworkRepository {

    fun signOut(signOutRequest: VariableRequest, delay: Long): Single<Response<Any>>

    fun signalToCamunda(payload: String): Single<Response<CamundaResponse>>

    fun sendTaskAccidentReport(payload: VariableRequest): Observable<ResponseBody>

    fun fileUploadRequest(
        meta: VariableRequest,
        files: List<File>,
        filename: String
    ): Observable<FileUploadResponse>

    fun screenshotUploadRequest(
        meta: VariableRequest,
        file: File,
        filename: String
    ): Observable<FileUploadResponse>

    fun signInByCacheUrl(workerCode: String): Observable<SignIn>

    fun signInByCacheUrlWithAttempts(workerCode: String): Observable<Event>

    fun loadTasksDataByCacheUrl(workerId: String): Observable<List<TaskPayload>>

    fun loadCurrentAppStatusByUrl(): Observable<ServerDataPayload>
}