package ru.daotech.agroinvest.network.models

import ru.daotech.agroinvest.BuildConfig
import ru.daotech.agroinvest.database.entities.TaskInterval
import ru.daotech.agroinvest.database.entities.TaskProperty
import ru.daotech.agroinvest.models.ObjectRequestTypeEnum
import ru.daotech.agroinvest.utils.DeviceInfo

interface BaseRequest {
    val type: BaseVariable<String>
    val meta: BaseVariable<String>
    val payload: BaseVariable<String>
}

abstract class BasePayload

abstract class BaseTaskPayload {
    abstract val taskId: String
    abstract val properties: List<TaskProperty>
    abstract val intervals: List<TaskInterval>
}

abstract class BaseCamundaPayload {
    abstract val businessKey: String
    abstract val variables: BaseRequest
}

class BaseVariable<T>(
    val value: T,
    val type: String = ObjectRequestTypeEnum.STRING.objectRequestType,
    val valueInfo: JsonValueInfo? = null
)

data class JsonValueInfo(
    val objectTypeName: String = "java.util.HashMap<java.lang.Object,java.lang.Object>",
    val serializationDataFormat: String = "application/json"
)

abstract class BaseMetaDeviceData {
    abstract val workerId: String?
    abstract var deviceId: String?
    abstract val sessionId: String?
    abstract var deviceTimeMark: String?
    abstract var serverTimeMark: String?
    abstract var deltaTimestamp: Int?
    val osVersion: String = DeviceInfo.getDeviceOsVersion()
    val apiLevel: Int = DeviceInfo.getDeviceApiLevel()
    val device: String = DeviceInfo.getDeviceInfo()
    val model: String = DeviceInfo.getDeviceModel()
    val product: String = DeviceInfo.getDeviceProduct()
    val versionCode: Int = BuildConfig.VERSION_CODE
    val versionName: String = BuildConfig.VERSION_NAME
}
