package ru.daotech.agroinvest.network.models

import com.google.gson.Gson
import ru.daotech.agroinvest.database.entities.TaskPropertyImpl
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import java.io.Serializable

data class GroupPropertyRequest(
    override val type: BaseVariable<String>,
    override val meta: BaseVariable<String>,
    override val payload: BaseVariable<String>
) : Serializable, BaseRequest {

    data class PayloadRequest(
        var barcodeValue: String?,
        val taskId: String,
        var plantId: String?,
        val properties: List<PropertyRequest.PropertyPayloadRequest>
    ) : BasePayload() {

        constructor(from: TaskWithProperties, barcodeValue: String, plantId: String) : this(
            barcodeValue = barcodeValue,
            plantId = plantId,
            taskId = from.id,
            properties = (from.properties as List<*>).map { property ->
                PropertyRequest.PropertyPayloadRequest(
                    property as TaskPropertyImpl
                )
            }
        )

        override fun toString(): String {
            return Gson().toJson(this)
        }
    }

    data class CamundaRequest(
        override val businessKey: String,
        override val variables: SuperTaskRequest
    ) : BaseCamundaPayload() {

        override fun toString(): String {
            return Gson().toJson(this)
        }
    }

    data class MetaRequest(
        override val workerId: String?,
        override var deviceId: String?,
        override val sessionId: String,
        override var deviceTimeMark: String?,
        override var serverTimeMark: String?,
        override var deltaTimestamp: Int?
    ) : BaseMetaDeviceData() {
        override fun toString(): String {
            return Gson().toJson(this)
        }
    }
}