package ru.daotech.agroinvest.network.models

import com.google.gson.Gson
import ru.daotech.agroinvest.database.entities.ActionReport
import ru.daotech.agroinvest.models.RequestTypeEnum
import java.io.Serializable

data class LogReportRequest(
    override val type: BaseVariable<String> = BaseVariable(RequestTypeEnum.LOG_REPORT.requestType),
    override val meta: BaseVariable<String>,
    override val payload: BaseVariable<String>
) : Serializable, BaseRequest {


    data class PayloadRequestMessage(
        val code: String
    ) : BasePayload() {
        override fun toString(): String {
            return Gson().toJson(this)
        }
    }

    data class PayloadRequest(
        val actionId: Long,
        val createdAt : Long?,
        val updatedAt : Long?,
        val payload: String?,
        val isScanned: Boolean?,
        val isSent: Boolean?
    ) {
        constructor(from: ActionReport) : this(
            actionId = from.id,
            createdAt = from.createdAt,
            updatedAt = from.updatedAt,
            payload = from.payload,
            isScanned = from.isScanned,
            isSent = from.isSent
        )

        override fun toString(): String {
            return Gson().toJson(this)
        }
    }

    data class MetaRequest(
        override val workerId: String?,
        override var deviceId: String?,
        override val sessionId: String,
        override var deviceTimeMark: String?,
        override var serverTimeMark: String?,
        override var deltaTimestamp: Int?
    ) : BaseMetaDeviceData() {
        override fun toString(): String {
            return Gson().toJson(this)
        }
    }
}