package ru.daotech.agroinvest.network.models

import com.google.gson.Gson
import ru.daotech.agroinvest.database.entities.TaskPropertyImpl
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import java.io.Serializable

data class PropertyRequest(
    override val type: BaseVariable<String>,
    override val meta: BaseVariable<String>,
    override val payload: BaseVariable<String>
) : Serializable, BaseRequest {

    data class PayloadRequest(val properties: List<TaskPropertyImpl>): BasePayload() {

        constructor(from: TaskWithProperties) : this(
            properties = from.properties as List<TaskPropertyImpl>
        )

        override fun toString(): String {
            val propertyPayloadList = properties
                .map { property -> PropertyPayloadRequest(property) }
            return Gson().toJson(propertyPayloadList)
        }
    }

    data class PropertyPayloadRequest(
        val id: String,
        val value: String,
        val timestamp: String?,
        val parentId: String,
        val workerId: String?
    ) {

        constructor(from: TaskPropertyImpl) : this(
            id = from.id,
            value = from.value,
            timestamp = from.timestamp,
            parentId = from.parentId,
            workerId = from.workerId
        )

        override fun toString(): String {
            return Gson().toJson(this)
        }
    }

    data class MetaRequest(
        override val workerId: String?,
        override var deviceId: String?,
        override val sessionId: String,
        override var deviceTimeMark: String?,
        override var serverTimeMark: String?,
        override var deltaTimestamp: Int?
    ) : BaseMetaDeviceData() {
        override fun toString(): String {
            return Gson().toJson(this)
        }
    }
}
