package ru.daotech.agroinvest.network.models

import com.google.gson.Gson
import ru.daotech.agroinvest.database.entities.TaskInterval
import ru.daotech.agroinvest.database.entities.TaskIntervalImpl
import ru.daotech.agroinvest.database.entities.TaskPropertyImpl
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import java.io.Serializable

data class SuperTaskRequest(
    override val type: BaseVariable<String>,
    override val meta: BaseVariable<String>,
    override val payload: BaseVariable<String>
) : Serializable, BaseRequest {

    data class PayloadRequest(
        val taskId: String,
        val startedAt: String?,
        val finishedAt: String?,
        val isFinished: Boolean,
        val properties: List<PropertyRequest.PropertyPayloadRequest>,
        val intervals: List<TaskIntervalImpl>
    ) : BasePayload() {

        constructor(from: TaskWithProperties) : this(
            taskId = from.id,
            startedAt = from.startedAt,
            finishedAt = from.finishedAt,
            isFinished = from.isFinished,
            properties = (from.properties as List<TaskPropertyImpl>).map { property ->
                PropertyRequest.PropertyPayloadRequest(
                    property
                )
            },
            intervals = from.intervals as List<TaskIntervalImpl>
        )

        override fun toString(): String {
            return Gson().toJson(this)
        }
    }

    data class CamundaRequest(
        override val businessKey: String,
        override val variables: SuperTaskRequest
    ) : BaseCamundaPayload() {

        override fun toString(): String {
            return Gson().toJson(this)
        }
    }

    data class MetaRequest(
        override val workerId: String?,
        override var deviceId: String?,
        override val sessionId: String,
        override var deviceTimeMark: String?,
        override var serverTimeMark: String?,
        override var deltaTimestamp: Int?
    ) : BaseMetaDeviceData() {
        override fun toString(): String {
            return Gson().toJson(this)
        }
    }
}
