package ru.daotech.agroinvest.network.models

import com.google.gson.Gson
import ru.daotech.agroinvest.database.entities.TaskIntervalImpl
import ru.daotech.agroinvest.database.entities.TaskPropertyImpl
import ru.daotech.agroinvest.database.relation.TaskWithProperties
import ru.daotech.agroinvest.models.RequestTypeEnum
import java.io.Serializable

data class TaskReportRequest(
    override val type: BaseVariable<String> = BaseVariable(RequestTypeEnum.TASK_REPORT.requestType),
    override val meta: BaseVariable<String>,
    override val payload: BaseVariable<String>
) : Serializable, BaseRequest {


    data class PayloadRequestMessage(
        val message: String
    ) : BasePayload() {
        override fun toString(): String {
            return Gson().toJson(this)
        }
    }

    data class PayloadRequest(
        val message: String,
        override val taskId: String,
        val startedAt: String?,
        val finishedAt: String?,
        val isFinished: Boolean?,
        override val properties: List<TaskPropertyImpl>,
        override val intervals: List<TaskIntervalImpl>
    ) : BaseTaskPayload() {

        constructor(from: TaskWithProperties, message: String) : this(
            message = message,
            taskId = from.id,
            startedAt = from.startedAt,
            finishedAt = from.finishedAt,
            isFinished = from.isFinished,
            properties = from.properties as List<TaskPropertyImpl>,
            intervals = from.intervals as List<TaskIntervalImpl>
        )

        override fun toString(): String {
            return Gson().toJson(this)
        }
    }

    data class MetaRequest(
        override val workerId: String?,
        override var deviceId: String?,
        override val sessionId: String,
        override var deviceTimeMark: String?,
        override var serverTimeMark: String?,
        override var deltaTimestamp: Int?
    ) : BaseMetaDeviceData() {
        override fun toString(): String {
            return Gson().toJson(this)
        }
    }
}