package ru.daotech.agroinvest.service

import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.graphics.Color
import android.os.Build
import androidx.annotation.RequiresApi
import androidx.core.app.NotificationCompat
import androidx.core.content.ContextCompat
import ru.daotech.agroinvest.BuildConfig
import ru.daotech.agroinvest.R

object NotificationCreator {
    private const val notificationId = 1983

    private const val serviceChannelId = "ru.daotech.agroinvest"
    private const val serviceChannelName = "Agroinvest Service"

    private var notification: Notification? = null

    @RequiresApi(Build.VERSION_CODES.O)
    private fun createNotificationChannel(context: Context, channelId: String, channelName: String): String{
        val chan = NotificationChannel(channelId,
            channelName, NotificationManager.IMPORTANCE_NONE)

        chan.lightColor = Color.BLUE
        chan.lockscreenVisibility = Notification.VISIBILITY_PRIVATE

        val service = context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        service.createNotificationChannel(chan)

        return channelId
    }

    fun getNotification(context: Context?): Notification? {
        if (notification == null && context != null) {
            //val pendingIntent: PendingIntent =
            //    Intent(this, CamundaService::class.java).let { notificationIntent ->
            //        PendingIntent.getActivity(this, 0, notificationIntent, 0)
            //    }

            val channelId =
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                    createNotificationChannel(context = context, channelId = serviceChannelId, channelName = serviceChannelName)
                } else {
                    // If earlier version channel ID is not used
                    // https://developer.android.com/reference/android/support/v4/app/NotificationCompat.Builder.html#NotificationCompat.Builder(android.content.Context)
                    serviceChannelId
                }

            notification = NotificationCompat.Builder(context, channelId)
                .setContentTitle(context.resources.getString(R.string.app_name))
                .setContentText(BuildConfig.VERSION_NAME)
                .setSmallIcon(R.drawable.foreground_service_icon)
                .setColor(ContextCompat.getColor(context, R.color.colorPrimary))
                //.setContentIntent(pendingIntent)
                .setTicker(BuildConfig.VERSION_NAME)
                .setPriority(NotificationCompat.PRIORITY_MAX)
                .build()
        }

        return notification
    }

    fun getNotificationId(): Int {
        return notificationId
    }
}