package ru.daotech.agroinvest.service.couch;

import androidx.annotation.NonNull;

import com.couchbase.lite.Replicator;
import com.couchbase.lite.ReplicatorConfiguration;

public class ReplicatorHelper {

    private ReplicatorHelper() {
    }

    public enum ReplicationType {PUSH_AND_PULL, PUSH, PULL}

    public static void setReplicatorType(@NonNull ReplicatorConfiguration config, @NonNull ReplicationType type) {
        config.setReplicatorType(
                (type == ReplicationType.PUSH_AND_PULL)
                        ? ReplicatorConfiguration.ReplicatorType.PUSH_AND_PULL
                        : ((type == ReplicationType.PUSH)
                        ? ReplicatorConfiguration.ReplicatorType.PUSH
                        : ReplicatorConfiguration.ReplicatorType.PULL));
    }

    public static ReplicationType getReplicatorType(@NonNull Replicator replicator) {
        switch (replicator.getConfig().getReplicatorType()) {
            case PUSH_AND_PULL:
                return ReplicationType.PUSH_AND_PULL;
            case PUSH:
                return ReplicationType.PUSH;
            case PULL:
                return ReplicationType.PULL;
        }
        throw new IllegalArgumentException("illegal argument exception for replication type");
    }

    public static ReplicatorConfiguration.ReplicatorType getReplicatorTypeFor(Boolean pull, Boolean push) {
        if (pull && push) return ReplicatorConfiguration.ReplicatorType.PUSH_AND_PULL;
        if (pull) return ReplicatorConfiguration.ReplicatorType.PULL;
        if (push) return ReplicatorConfiguration.ReplicatorType.PUSH;
        return null;
    }
}
