package ru.daotech.agroinvest.service.events

import android.app.AlarmManager
import android.app.PendingIntent
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import io.reactivex.Observable
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import ru.daotech.agroinvest.AppStateManager
import ru.daotech.agroinvest.business.Repository
import timber.log.Timber
import java.util.*
import javax.inject.Inject

class LogoutReceiver : BroadcastReceiver() {

    @Inject lateinit var appStateManager: AppStateManager
    @Inject lateinit var repository: Repository

    private var workerDisposable: Disposable? = null

    override fun onReceive(context: Context?, intent: Intent?) {

        if (workerDisposable?.isDisposed == false) {
            workerDisposable?.dispose()
        }

        if (::appStateManager.isInitialized) {
            appStateManager
                .getCurrentWorker()
                .subscribeOn(Schedulers.io())
                .subscribe(
                    {
                        repository.signOut(it.id)
                    },
                    Timber::e
                )
                .let { workerDisposable = it }
        }
    }

    companion object {

        fun start(context: Context): Observable<Unit> = Observable.fromCallable { schedule(context) }

        fun cancel(context: Context): Observable<Unit> {
            val alarmManager = context.getSystemService(Context.ALARM_SERVICE) as AlarmManager
            return Observable.fromCallable { alarmManager.cancel(createPendingIntent(context))  }
        }

        private fun schedule(context: Context) {
            val alarmManager = context.getSystemService(Context.ALARM_SERVICE) as AlarmManager

            val calendar = Calendar.getInstance().apply {
                timeInMillis = System.currentTimeMillis()
                set(Calendar.HOUR_OF_DAY, 0)
                set(Calendar.MINUTE, 0)
            }

            val intent = createPendingIntent(context)

            alarmManager.set(
                AlarmManager.RTC_WAKEUP,
                calendar.timeInMillis,
                intent
            )
        }

        private fun createPendingIntent(context: Context): PendingIntent {
            val intent = Intent(context, LogoutReceiver::class.java)

            return PendingIntent.getBroadcast(context, 0, intent, 0)
        }
    }
}