package ru.daotech.agroinvest.utils

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.net.ConnectivityManager
import io.reactivex.subjects.BehaviorSubject
import io.reactivex.subjects.PublishSubject
import ru.daotech.agroinvest.models.InternetConnectivityState
import timber.log.Timber

class ConnectivityReceiver : BroadcastReceiver() {
    val connectivitySubject: BehaviorSubject<Pair<Boolean, InternetConnectivityState>>
            = BehaviorSubject.create()

    val connectivityViewSubject: PublishSubject<Boolean> = PublishSubject.create()

    var isConnected: Boolean = true

    private lateinit var internetConnectivityState: InternetConnectivityState

    override fun onReceive(context: Context, arg1: Intent) {
        isConnected = isConnectedOrConnecting(context)

        try {
            val connectivityManager = context
                .getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager

            internetConnectivityState = InternetConnectivityState(connectivityManager)
        } catch (ex: RuntimeException) {
            Timber.e(ex)
        }

        connectivityViewSubject.onNext(isConnected)
        connectivitySubject.onNext(Pair(isConnected, internetConnectivityState))
    }

    private fun isConnectedOrConnecting(context: Context): Boolean {
        val connMgr = context
            .getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager

        val networkInfo = connMgr.activeNetworkInfo

        return networkInfo != null
    }
}