package ru.daotech.agroinvest.utils

import com.github.debop.kodatimes.hours
import com.github.debop.kodatimes.seconds
import org.joda.time.*
import org.joda.time.format.DateTimeFormat
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.*
import kotlin.math.abs

const val SHOWED_DATE_TIME_FORMAT = "dd/MM/yyyy HH:mm:ss"
const val SHOWED_DATE_TIME_COLLECTION_HISTORY_FORMAT = "dd-MM-yyyy"
const val SHOWED_TIME_COLLECTION_HISTORY_FORMAT = "HH:mm:ss"
const val GENERAL_DATE_FORMAT = "yyyy-MM-dd'T'HH:mm:ssZZ"
const val REQUEST_DATE_FORMAT = "yyyy-MM-dd'T'HH-mm-ss"
const val DATE_TIME_VIEW_FORMAT = "dd/MM  HH:mm"
const val TIMER_TIME_VIEW_FORMAT = "HH:mm"
const val FULL_TIME_FORMAT_WITH_TIMEZONE = "yyyy-MM-dd'T'HH:mm:ssZ"
const val TIMER_MESSAGE_TIME_VIEW_FORMAT = "HH:mm:ss"

class DateUtil {
    companion object {

        fun getCurrentDateTimestampWithDelta(delta: Int): String {
            return getDateTimeWithDelta(DateTime.now(), delta)
        }

        fun isSessionToday(dateTime: DateTime): Boolean {
            val today = Interval(DateTime.now().withTimeAtStartOfDay(), Days.ONE)
            return today.contains(dateTime)
        }

        fun getDateTimeWithDelta(dateTime: DateTime, delta: Int): String {
            return dateTime
                .minusSeconds(delta)
                .toString(GENERAL_DATE_FORMAT)
        }

        fun getCurrentDateTimeWithDeltaForRequest(delta: Int): String {
            return DateTime.now()
                .minusSeconds(delta)
                .toString(REQUEST_DATE_FORMAT)
        }

        fun getCurrentDateTimestampWithDelta(dateString: String, delta: Int): Date {
            return DateTimeFormat
                .forPattern(GENERAL_DATE_FORMAT)
                .parseDateTime(dateString)
                .plusSeconds(delta)
                .toDate()
        }

        fun getDateTimeWithDelta(dateString: String, delta: Int): DateTime {
            return DateTimeFormat
                .forPattern(GENERAL_DATE_FORMAT)
                .parseDateTime(dateString)
                .plusSeconds(delta)
        }

        /** Converting from String to Date **/
        fun getDateWithServerTimeStamp(dateTime: String): Date? {
            val dateFormat = SimpleDateFormat(
                FULL_TIME_FORMAT_WITH_TIMEZONE, // RFC 3339 https://www.php.net/manual/ru/class.datetimeinterface.php#datetime.constants.rfc3339_extended
                Locale.getDefault()
            )
            dateFormat.timeZone = TimeZone.getDefault()
            return try {
                dateFormat.parse(dateTime)
            } catch (e: ParseException) {
                null
            }
        }

        fun getFormatDateStringFromDateString(dateString: String): String {
            return DateTimeFormat
                .forPattern(GENERAL_DATE_FORMAT)
                .parseDateTime(dateString)
                .toString(SHOWED_DATE_TIME_FORMAT)
        }

        fun getCollectionHistoryDateFormatDateString(dateString: String): String {
            return DateTimeFormat
                .forPattern(GENERAL_DATE_FORMAT)
                .parseDateTime(dateString)
                .toString(SHOWED_DATE_TIME_COLLECTION_HISTORY_FORMAT)
        }

        fun getCollectionHistoryTimeDateString(dateString: String): String {
            return DateTimeFormat
                .forPattern(GENERAL_DATE_FORMAT)
                .parseDateTime(dateString)
                .toString(SHOWED_TIME_COLLECTION_HISTORY_FORMAT)
        }

        fun getViewFormatDateStringFromDateString(dateString: String): String {
            return DateTimeFormat
                .forPattern(GENERAL_DATE_FORMAT)
                .parseDateTime(dateString)
                .toString(DATE_TIME_VIEW_FORMAT)
        }

        fun getSecondsByServerDateTime(dateTimeString: String): Int {
            return Seconds.secondsBetween(
                getDateTimeFromString(dateTimeString, GENERAL_DATE_FORMAT),
                DateTime.now()
            ).seconds
        }

        fun getMillsBetweenTwoDates(finishedAt: String, startedAt: String): Long {
            var finishedAtDateTime = getDateTimeFromString(startedAt, GENERAL_DATE_FORMAT)
            var startedAtDateTime = getDateTimeFromString(finishedAt, GENERAL_DATE_FORMAT)

            if (finishedAtDateTime.isBefore(startedAtDateTime)) {
                // Swap if date is incorrect
                finishedAtDateTime = startedAtDateTime
                    .also { startedAtDateTime = finishedAtDateTime }
            }

            return abs(
                Duration(
                    getDateTimeFromString(startedAt, GENERAL_DATE_FORMAT),
                    getDateTimeFromString(finishedAt, GENERAL_DATE_FORMAT)
                ).millis
            )
        }

        fun getHoursBetweenTwoDates(finishedAt: String, startedAt: String): Long {
            return Duration(
                getDateTimeFromString(startedAt, GENERAL_DATE_FORMAT),
                getDateTimeFromString(finishedAt, GENERAL_DATE_FORMAT)
            ).hours()
        }

        fun getSecondsBetweenTwoDates(finishedAt: String, startedAt: String): Long {
            return Duration(
                getDateTimeFromString(startedAt, GENERAL_DATE_FORMAT),
                getDateTimeFromString(finishedAt, GENERAL_DATE_FORMAT)
            ).seconds()
        }

        private fun getDateTimeFromString(dateString: String, innerFormat: String): DateTime {
            return DateTimeFormat
                .forPattern(if (innerFormat.isBlank()) GENERAL_DATE_FORMAT else innerFormat)
                .parseDateTime(dateString)
        }

        fun getTimeTimerFormat(startedAt: String?): CharSequence? {
            return DateTimeFormat
                .forPattern(GENERAL_DATE_FORMAT)
                .parseDateTime(startedAt)
                .toString(TIMER_TIME_VIEW_FORMAT)
        }

        fun getTimeTimerChatMessageFormat(startedAt: String?): CharSequence? {
            return DateTimeFormat
                .forPattern(GENERAL_DATE_FORMAT)
                .parseDateTime(startedAt)
                .toString(TIMER_MESSAGE_TIME_VIEW_FORMAT)
        }
    }
}