package ru.daotech.agroinvest.utils.components

import android.content.Context
import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.RectF
import android.util.AttributeSet
import android.view.View
import android.view.animation.Animation
import android.view.animation.Transformation
import ru.daotech.agroinvest.R

open class Circle(context: Context?, attrs: AttributeSet?) : View(context, attrs) {

    companion object {
        private const val START_ANGLE_POINT = 270
        private const val STARTED_STROKE_WIDTH = 50
        private const val INITIAL_ANGLE_VALUE = 9f
        private const val DIFF_STROKE_WIDTH = 540
    }

    private val paint: Paint
    private val rect: RectF

    var angle: Float

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        canvas.drawArc(rect, START_ANGLE_POINT.toFloat(), angle, false, paint)
    }

    init {
        val strokeWidth = STARTED_STROKE_WIDTH
        paint = Paint()
        paint.isAntiAlias = true
        paint.style = Paint.Style.STROKE
        paint.strokeWidth = strokeWidth.toFloat()
        paint.color = resources.getColor(R.color.colorTaskStandardTime, context?.theme)
        rect = RectF(
            strokeWidth.toFloat(),
            strokeWidth.toFloat(),
            (DIFF_STROKE_WIDTH + strokeWidth).toFloat(),
            (DIFF_STROKE_WIDTH + strokeWidth).toFloat()
        )
        //Initial angle value
        angle = INITIAL_ANGLE_VALUE
    }

    fun setPaintCircleExcessExcessColor() {
        paint.color = resources.getColor(R.color.accentSnackbarBackground, context?.theme)
    }
}

open class CircleAngleAnimation(
    private val circle: Circle,
    newAngle: Int,
    private val initialTime: Long,
    private val fullCircleSeconds: Int
) : Animation() {

    private val oldAngle: Float = circle.angle
    private val newAngle: Float = newAngle.toFloat()

    override fun applyTransformation(
        interpolatedTime: Float,
        transformation: Transformation?
    ) {
        val angle =
            oldAngle + (initialTime * newAngle / fullCircleSeconds) + ((newAngle - oldAngle) * interpolatedTime)
        if (angle >= FULL_DRAWABLE_CIRCLE_ANGLE) {
            circle.setPaintCircleExcessExcessColor()
        }
        circle.angle = angle
        circle.requestLayout()
    }

    companion object {
        private const val FULL_DRAWABLE_CIRCLE_ANGLE = 360
    }
}